{*
 Contains classes and types for platter objects
 }
unit cMagKatPlatter;

interface

uses classes;

type

  /// Status of the platter
  TMagKatPlatterStatus = (Online, Offline, Processed, OfflineError, OnlineFailedProcess);

  TMagKatPlatter = class;

  {*
    Event for when a platter is updated
    @param Sender the object which sent this event
    @param Platter The platter with an updated event
    }
  TMagKatPlatterStatusUpdateUpdateEvent = procedure(Sender : TObject;
    Platter : TMagKatPlatter) of object;


  /// Platter object class
  TMagKatPlatter = class
  private
    /// Platter name
    FPlatterName : String;
    /// Platter serial number
    FPlatterSerialNumber : String;
    /// Platter status
    FPlatterStatus : TMagKatPlatterStatus;
    /// Number of TXT files processed
    FTxtFilesProcessed : integer;
    /// Number of files on platter
    FTxtFilesOnPlatter : integer;
    /// Number of failed TXT files
    FFailedTxtFiles : integer;
    ///
    FIOErrors : integer;

    function getPlatterStatusString() : String;
    function getPlatterDisplayName() : String;

  public

    constructor Create(PlatterName, PlatterSerialNumber : String;
      PlatterStatus : TMagKatPlatterStatus); overload;
    constructor Create(PlatterName, PlatterSerialNumber : String;
      PlatterStatusString : String); overload;

    /// Platter Name
    property PlatterName : String read FPlatterName write FPlatterName;
    /// Platter Serial Number
    property PlatterSerialNumber : String read FPlatterSerialNumber write FPlatterSerialNumber;
    /// Platter Status
    property PlatterStatus : TMagKatPlatterStatus read FPlatterStatus write FPlatterStatus;
    /// Platter Status as a string
    property PlatterStatusString : String read getPlatterStatusString;
    /// TXT files on platter
    property TxtFilesOnPlatter : integer read FTxtFilesOnPlatter write FTxtFilesOnPlatter;
    /// TXT files processed
    property TxtFilesProcessed : integer read FTxtFilesProcessed write FTxtFilesProcessed;
    /// Failed TXT files
    property FailedTxtFiles : integer read FFailedTxtFiles write FFailedTxtFiles;
    /// Display a string name for the platter containing the name and serial number
    property PlatterDisplayName : string read getPlatterDisplayName;

    property IOErrors : integer read FIOErrors write FIOErrors;
  end;

implementation

{*
 Create platter with platter status
 @param PlatterName the name of the platter
 @param PlatterSerialNumber the serial number of the platter
 @param PlatterStatus the status of the platter
 }
constructor TMagKatPlatter.Create(PlatterName, PlatterSerialNumber : String;
  PlatterStatus : TMagKatPlatterStatus);
begin
  inherited Create;
  FPlatterName := PlatterName;
  FPlatterSerialNumber := PlatterSerialNumber;
  FPlatterStatus := PlatterStatus;
  FTxtFilesProcessed := 0;
  FTxtFilesOnPlatter := 0;
  FFailedTxtFiles := 0;
  FIOErrors := 0;
end;

{*
 Create platter with platter status string
 @param PlatterName the name of the platter
 @param PlatterSerialNumber the serial number of the platter
 @param PlatterStatusString the status of the platter as a string
 }
constructor TMagKatPlatter.Create(PlatterName, PlatterSerialNumber : String;
  PlatterStatusString : String);
begin
  inherited Create;
  FPlatterName := PlatterName;
  FPlatterSerialNumber := PlatterSerialNumber;
  if PlatterStatusString = 'Offline' then
    FPlatterStatus := Offline
  else if PlatterStatusString = 'Online' then
    FPlatterStatus := Online
  else if PlatterStatusString = 'Processed' then
    FPlatterStatus := Processed
  else if PlatterStatusString = 'OfflineError' then
    FPlatterStatus := OfflineError
  else if PlatterStatusString = 'OnlineFailedProcess' then
    FPlatterStatus := OnlineFailedProcess;
  FTxtFilesProcessed := 0;
  FTxtFilesOnPlatter := 0;
  FFailedTxtFiles := 0;
  FIOErrors := 0;
end;

{*
 Get the status of the plattter as a string
 @return The status in String format
 }
function TMagKatPlatter.getPlatterStatusString() : String;
begin
  if FPlatterStatus = Online then
    result := 'Online'
  else if FPlatterStatus = Offline then
    result := 'Offline'
  else if FPlatterStatus = Processed then
    result := 'Processed'
  else if FPlatterStatus = OfflineError then
    result := 'OfflineError'
  else if FPlatterStatus = OnlineFailedProcess then
    result := 'OnlineFailedProcess'
  else
    result := 'Unknown';
end;

{*
 Display a string name for the platter containing the name and serial number
 @return The display name to show
 }
function TMagKatPlatter.getPlatterDisplayName() : String;
begin
  result := FPlatterName + ' (' + FPlatterSerialNumber + ')';
end;

end.
