/// Utility functions and classes for MagKat
unit cMagKatUtils;

interface

uses
  forms, sysutils, iniFiles, shellapi, windows, dialogs;

  type
  /// Contains details about the screen area
  TMagScreenArea = class
  /// Left position
  left : integer;
  /// Top position
  top : integer;
  /// right position
  right : integer;
  /// bottom position
  bottom : integer;
  /// width of the screen
  width : integer;
  /// height of the screen
  height : integer;
  end;

function GetAppPath(): string;
function MagPiece(str, del: string; Piece: Integer): string;
function MAGLENGTH(STR, DEL: string): INTEGER;
procedure saveformposition(Form: Tform; IniFile : TIniFile);
procedure getformposition(Form: Tform; IniFile : TIniFile);
Function GetScreenArea : TmagScreenArea;
function getScreenWidth() : integer;
function DoIExist(semaphore: string): boolean;
function GetLogsDir() : String;
procedure CreateEmptyFile(filename : String);
procedure CreateIfNeeded(filename : String);

const
  /// Set of button options that only contains OK button
  TOKButtonOnly = [mbOK];

implementation

{*
 Saves form position
 @param Form The form to save
 @param IniFile The INI file to save the details to
 }
procedure saveformposition(Form: Tform; IniFile : TIniFile);
begin
  if form.windowstate <> wsnormal then exit;
  with IniFile do
  begin
    writestring('SYS_LastPositions', form.name, inttostr(form.left) + ',' +
      inttostr(form.top) + ',' + inttostr(form.width) + ',' + inttostr(form.height));
  end;
end;

{*
 Gets the form position for the form from the INI file
 @param Form The form to set the position
 @param IniFile the INI file that contains the form position
 }
procedure getformposition(Form: Tform; IniFile : TIniFile);
var
  FORMpos: string;
  wrksarea : TMagScreenArea;
begin
  with IniFile do
  begin
    FORMpos := readstring('SYS_LastPositions', form.name, 'NONE');
  end;
  if FORMpos <> 'NONE' then
  begin
    if ((form.BorderStyle = bsSizeToolWin) or (form.borderstyle = bssizeable))
      then FORM.setbounds(strtoint(magpiece(formpos, ',', 1)), strtoint(magpiece(formpos, ',', 2)),
        strtoint(magpiece(formpos, ',', 3)), strtoint(magpiece(formpos, ',', 4)))
    else begin
      FORM.LEFT := strtoint(magpiece(formpos, ',', 1));
      FORM.TOP := strtoint(magpiece(formpos, ',', 2));
    end;
  end;
  wrksarea := GetScreenArea;
  try
  if (form.Left + form.Width) > wrksarea.right then form.left := wrksarea.right - form.Width;
  if (form.top + form.height) > wrksarea.bottom then form.top := wrksarea.bottom - form.height;
  if (form.top < wrksarea.top) then form.top := wrksarea.top;
  if (form.left < wrksarea.left) then form.left := wrksarea.left;

finally
wrksarea.free;
end;
end;

{*
 Get the current screen area
 @return The screen area
 }
Function GetScreenArea : TmagScreenArea;
var wrkbar : APPBARDATA;
wrkspos : TMagScreenArea;
begin
wrkspos := TmagscreenArea.create;
wrkbar.cbSize := 36;
SHAppBarMessage(ABM_GETTASKBARPOS, wrkBar);

// get the left and right positions of the workstation area.
if (wrkBar.rc.Left <= 0) and (wrkbar.rc.right >= screen.width)
   then
   // taskbar is at top or bottom
     begin
      wrkspos.left := 0 ;
      wrkspos.right := getScreenWidth();// screen.width;
      end
   else if (wrkBar.rc.left <=0) then
     //taskbar is at left
     begin
       wrkspos.left := wrkBar.rc.Right;
       wrkspos.right := getScreenWidth();// screen.width;
     end
     else if (wrkbar.rc.right >=screen.width) then
     // taskbar is at right
       begin
         wrkspos.left := 0;
         wrkspos.right := getScreenWidth();// wrkbar.rc.Left;
       end;


if (wrkBar.rc.top <= 0) and (wrkbar.rc.bottom >= screen.height)
   then
   // taskbar is at top or bottom
     begin
      wrkspos.top := 0 ;
      wrkspos.bottom := screen.height;
      end
   else if (wrkBar.rc.top <=0) then
     //taskbar is at top
     begin
       wrkspos.top := wrkBar.rc.bottom;
       wrkspos.bottom := screen.height;
     end
     else if (wrkbar.rc.bottom >=screen.height) then
     // taskbar is at bottom
       begin
         wrkspos.top := 0;
         wrkspos.bottom := wrkbar.rc.top;
       end;
 wrkspos.width := wrkspos.right - wrkspos.left;
 wrkspos.height := wrkspos.bottom - wrkspos.top;

result := wrkspos;
end;

{*
  Gets the width of the screen
  @return the width of the screen
 }
function getScreenWidth() : integer;
var
  i : integer;
  width : integer;
begin
  width := 0;
  for i := 0 to screen.MonitorCount - 1 do
  begin
    width := width + screen.Monitors[i].Width;
  end;
  result := width;
end;

{*
  Returns the application path
  @return The application path
 }
function GetAppPath(): string;
begin
  result := copy(extractfilepath(application.exename), 1, length(extractfilepath(application.exename)) - 1);
end;

{*
  Implementation of the M piece function
  @param str The input string
  @param del The delimiter
  @param Piece the piece to return
  @return The piece from the string
 }
function MagPiece(str, del: string; Piece: Integer): string;
var I, K: INTEGER;
  s: string;
begin
  I := Pos(del, str);
  if (I = 0) and (PIECE = 1) then begin MAGPIECE := STR; EXIT; end;
  for K := 1 to PIECE do
  begin
    I := POS(DEL, STR);
    if (I = 0) then I := LENGTH(STR) + 1;
    S := COPY(STR, 1, I - 1);
    STR := COPY(STR, I + 1, LENGTH(STR));
  end;
  MAGPIECE := S;
end;

{*
 Returns the number of pieces in the string
 @param Str the input string
 @param DEL The delimiter
 @return The number of pieces in the string
 }
function MAGLENGTH(STR, DEL: string): INTEGER;
var I, J: INTEGER;
  ESTR: BOOLEAN;
begin
  ESTR := FALSE;
  I := 0;
  while not ESTR do
  begin
    I := I + 1;
    if (POS(DEL, STR) = 0) then ESTR := TRUE;
    J := POS(DEL, STR); STR := COPY(STR, J + 1, LENGTH(STR));
  end;
  MAGLENGTH := I;
end;

{*
 Determines if the specified application name is already active
 @param semaphore The name of the application to find
 @return True if the application is running, false otherwise
 }
function DoIExist(semaphore: string): boolean;
var
  hSem: THandle;
begin
  HSEM := OpenSemaphore(1, false, Pchar(semaphore));
  if hsem = 0
    then
  begin
    result := false;
    CreateSemaphore(nil, 0, 1, Pchar(semaphore));
  end
  else result := true;
end;

function GetLogsDir() : String;
begin
  result := GetAppPath() + '\logs';
end;

procedure CreateEmptyFile(filename : String);
var
  txtFile : TextFile;
begin
  try
    try
      assignFile(TxtFile, filename);
      rewrite(TxtFile);
    except
      on E : Exception do
      begin

      end;
    end;
  finally
    try
      CloseFile(txtFile);
    except
      on E : Exception do
      begin

      end;
    end;
  end;
end;

procedure CreateIfNeeded(filename : String);
begin
  if not FileExists(filename) then
    CreateEmptyFile(filename);
end;

end.
