unit fMagAbout;
   {
   Package: MAG - VistA Imaging
   WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
   Date Created:  1998
   Site Name: Silver Spring, OIFO
   Developers: Garrett Kirin
   Description: Imaging About Box.  Generic, to be used by
                 any Imaging Application.
   Note:
   }
(*
        ;; +---------------------------------------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +---------------------------------------------------------------------------------------------------+
 
*)

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, Sysutils,  jpeg, shellapi, Dialogs, ComCtrls,
  // va
  MagFileVersion, cMagKatUtils, uMagCRC32, fmxutils, cMagListView;

type
  TfrmAbout = class(TForm)
    pnlMain: TPanel;
    imgProgramIcon: TImage;
    lbAlign: TLabel;
    lbFileDescription: TLabel;
    pnlTopwithLogo: TPanel;
    imgVistALogo: TImage;
    lbSilverSpringinfo: TLabel;
    lbImagingProjectinfo: TLabel;
    lbDevelopedbyinfo: TLabel;
    pnlUnathorizedtext: TPanel;
    lbUnauthorizedtextinfo: TLabel;
    pnlComputedFields: TPanel;
    lbfilenameprompt: TLabel;
    lbfilename: TLabel;
    lbVersionprompt: TLabel;
    lbversion: TLabel;
    lbPatchprompt: TLabel;
    lbPatch: TLabel;
    lbfiledateprompt: TLabel;
    lbfiledate: TLabel;
    lbfilesizeprompt: TLabel;
    lbfilesize: TLabel;
    lbCRCPrompt: TLabel;
    lbVHAinfo: TLabel;
    lbCRC: TLabel;
    lbServerVersionprmpt: TLabel;
    lbServerVersion: TLabel;
    MagListView1: TMagListView;
    lbDelphiVersionprompt: TLabel;
    lbOSVersionPrompt: TLabel;
    lbDelphiVersion: TLabel;
    lbOSVersion: TLabel;
    pnlOKbtn: TPanel;
    btnOK: TBitBtn;
    lbInstalledVersions: TLabel;
    lbVerPatch: TLabel;
    lbStatusPrompt: TLabel;
    lbStatus: TLabel;
    procedure btnOKClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    procedure DrawAppIcon(Sender: TObject);
  public
    procedure execute(AppName: string = '';reqserver: string = ''; installlist : Tstrings = nil; status : string = '');
  end;

var
  frmAbout: TfrmAbout;
  const formname : string = 'frmAbout';
implementation

{$R *.DFM}

procedure TfrmAbout.btnOKClick(Sender: TObject);
begin
  close;
end;

procedure TfrmAbout.execute(AppName: string = '';reqserver: string = ''; installlist : Tstrings = nil; status : string = '');
var MagVerPatch : string   ;
CRC: integer;
stream : TmemoryStream;
begin
if AppName = '' then appname := Application.exename;
 with TfrmAbout.Create(nil) do
  begin
  { assure all are null }
  lbFileDescription.caption := '';
  lbVersion.caption := '';
  lbfilename.caption := '';
  lbfilesize.caption := '';
  lbfiledate.caption := '';
  lbPatch.caption := '';
  lbCRC.caption := '';
  lbVerPatch.caption := '';

  lbFileDescription.caption := MagGetFileDescription(AppName);
  caption := 'About: ' + MagGetProductName(AppName);
  magVerPatch := MagGetFileVersionInfo(AppName);
  lbVerPatch.Caption := magVerPatch;
  lbVersion.caption := magpiece(magVerPatch,'.',1) + '.' + magpiece(magVerPatch,'.',2);
  lbPatch.caption := magpiece(magVerPatch,'.',3); //p63 + ' T' + magpiece(magVerPatch,'.',4);
  { required server version can be passed as parameter, if it isn't it defaults to
    the version of the Client}
  if reqserver = '' then reqserver := lbVersion.caption + '.'+magpiece(MagVerPatch,'.',3)+'.'+magpiece(MagVerPatch,'.',4);
  lbServerVersion.caption := reqserver;
  lbfilename.caption := AppName;
  lbfilesize.caption := inttostr((getfilesize(AppName) div 1024) + 1) + ' KB';
  lbfiledate.caption := formatdatetime('mm/dd/yy  h:nn  am/pm', filedatetime(AppName));
  lbDelphiVersion.caption := MagGetFileVersionInfo(AppName,false);
  lbOSVersion.caption := MagGetOSVersion;
  Stream:=TMemoryStream.Create;
   Stream.LoadFromFile(AppName);
   CRC:=GetMemoryStreamCrc32(Stream);
   Stream.Free;
  (*   Label1.Caption:='CRC:  '+IntToHex(CRC,8);  *)
  lbCRC.Caption := IntToHex(CRC,8);
  DrawAppIcon(self);
  if (installlist <> nil) and (installlist.Count > 1 ) then
    maglistview1.LoadListFromStrings(installlist)
        else
          begin
            maglistview1.ClearItems;
            maglistview1.Hide;
            lbInstalledVersions.caption := lbInstalledVersions.caption + '  Not Available';
          end;
  { Patch 59.  Show status of Version}
  lbStatusPrompt.Visible := (status <> '');
  lbStatus.Caption := status; 
  showmodal;
  free;
  end;
end;



procedure TfrmAbout.FormCreate(Sender: TObject);
begin
pnlMain.Align := alclient;
 (* lbFileDescription.caption := '';
  lbVersion.caption := '';
  lbfilename.caption := '';
  lbfilesize.caption := '';
  lbfiledate.caption := '';
  lbPatch.caption := '';
  lbCRC.caption := '';
  lbFileDescription.caption := MagGetFileDescription(application.exename);
  caption := 'About: ' + MagGetProductName(application.exename);
  magVerPatch := MagGetFileVersionInfo(application.exename);
  lbVersion.caption := magpiece(magVerPatch,'.',1) + '.' + magpiece(magVerPatch,'.',2);
  lbPatch.caption := magpiece(magVerPatch,'.',3) + ' T' + magpiece(magVerPatch,'.',4);
  lbfilename.caption := application.exename;
  lbfilesize.caption := inttostr((getfilesize(Application.exename) div 1024) + 1) + ' KB';
  lbfiledate.caption := formatdatetime('mm/dd/yy  h:nn  am/pm', filedatetime(Application.exename));

  Stream:=TMemoryStream.Create;
  Stream.LoadFromFile(lbFileName.caption);
  CRC:=GetMemoryStreamCrc32(Stream);
  Stream.Free;
  lbCRC.Caption := IntToHex(CRC,8);
  DrawAppIcon(self);
  *)
  end;

procedure TfrmAbout.DrawAppIcon(Sender: TObject);
var
   IconIndex : word;
   icon : Ticon;
begin
Icon := TIcon.create;
  (*  IconIndex := 0;
      h:=ExtractAssociatedIcon
      (hInstance,'C:\WINDOWS\NOTEPAD.EXE', IconIndex) ;
      DrawIcon(image2.Canvas.Handle, 10, 10, h) ; 
   *)
  IconIndex := 0;
  icon.Handle:=ExtractAssociatedIcon(hInstance,pchar(application.exename), IconIndex) ;
 DrawIcon(imgprogramicon.Canvas.Handle, 0, 0, icon.handle) ;
 icon.free;
 (* PROGRAMICON.Invalidate;*)
end;


end.
