Unit uMagUtility;
{*
Package     : MAG - VistA Imaging
WARNING     : Per VHA Directive 10-93-142 this unit should not be modified.
Date Created:   Oct 2008
Site Name   : Silver Spring, OIFO
Developers  : Jemmy Lin, Richard Maley
Version     : MAG*3.0*98
;; +---------------------------------------------------------------------------------------------------+
;; Property of the US Government.
;; No permission to copy or redistribute this software is given.
;; Use of unreleased versions of this software requires the user
;;  to execute a written test agreement with the VistA Imaging
;;  Development Office of the Department of Veterans Affairs,
;;  telephone (301) 734-0100.
;;
;; The Food and Drug Administration classifies this software as
;; a medical device.  As such, it may not be changed
;; in any way.  Modifications to this software may result in an
;; adulterated medical device under 21CFR820, the use of which
;; is considered to be a violation of US Federal Statutes.
;; +---------------------------------------------------------------------------------------------------+

MagUtility is a GUI (Graphical User Interface)-based program that can be used to:
	Identify orphan files (both text and image) on the Imaging shares/RAID, and optionally resolve Image File (#2005) references to those files and/or remove un-resolvable files.
	Delete unused and/or erroneous RAID- and jukebox-related entries from the Network Location File (#2005.2).
	Update the Offline Images File (#2006.033) to reflect the removal or return of specific jukebox platters.
	Copy and/or transfer pixel data and text files from one physical storage location to another storage location (earlier versions of this feature were available in FUT (Field Unit Test) 49).

Delphi code  (Project) MagUtility.dpr  (Unit) MagUtility.pas/
  (Unit) MagUtilityBroker.pas
  (Unit)MagUtilityUtils
  Delphi Project search path: 	\Dev\P98\Tools\Log4delphi  (for logging)
                                \Dev\P98\Tools\MagUtility (main program)
  }

Interface
Uses
  Windows,
  ActnList,
  Buttons,
  Classes,
  cMagUtilityBroker,
  ComCtrls,
  Controls,
  ExtCtrls,
  Forms,
  Graphics,
  ImgList, 
  Menus,
  Stdctrls,
  SysUtils,
  TLoggerUnit, Gauges;

Type
  TfMagUtility = Class(TForm)
    ActionList1: TActionList;
    actStorageByDateFrom: TAction;
    actStorageByDateTo: TAction;
    actStorageLUDest: TAction;
    actStorageLUSource: TAction;
    BtnBackOnlineJB: TBitBtn;
    btnFromLoc: TBitBtn;
    btnHide: TButton;
    btnImgOFLclrLst: TBitBtn;
    btnNtLocDelRun: TBitBtn;
    BtnOflJBSel: TButton;
    btnPauseFUT49: TBitBtn;
    btnProcessFUT49: TBitBtn;
    BtnRunJBOffline: TBitBtn;
    btnRunProcess: TBitBtn;
    btnSel2Dt: TBitBtn;
    btnSel4Dt: TBitBtn;
    btnStopProcessing: TBitBtn;
    btnToLoc: TBitBtn;
    cbxFromShareLoc: TComboBox;
    cbxOfflineJBList: TComboBox;
    cbxToShareLoc: TComboBox;
    EdtEndDT: TLabeledEdit;
    edtEndIEN: TLabeledEdit;
    edtStartDT: TLabeledEdit;
    edtStartIEN: TLabeledEdit;
    FUT49: TTabSheet;
    FUT49AftHour: TCheckBox;
    FUT49RemoveOld: TCheckBox;
    FUT49SetNewLoc: TCheckBox;
    GaugeFUT49: TGauge;
    gbcbxShareLoc: TGroupBox;
    gbEndDT: TGroupBox;
    gbEndIEN: TGroupBox;
    gbFromShareLoc: TGroupBox;
    gbFUT49Right: TGroupBox;
    gbStartDT: TGroupBox;
    gbStartIEN: TGroupBox;
    gbToShareLoc: TGroupBox;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    GroupBox3: TGroupBox;
    grpProcessStatus: TGroupBox;
    lblCurrentFile: Tlabel;
    lblCurrentFileLabel: Tlabel;
    lblCurrentIEN: Tlabel;
    lblErrorCnt: Tlabel;
    lblFileEmpty: Tlabel;
    lblFileEmptyCnt: Tlabel;
    lblIEN: Tlabel;
    lblNetworkLocNote: TLabel;
    lblRecentErrorCountLabel: Tlabel;
    lblTotalcnt: Tlabel;
    lblTotalCount: Tlabel;
    MagJbOffline: TTabSheet;
    MagOrphan: TTabSheet;
    MainMenu1: TMainMenu;
    MnuFile: TMenuItem;
    mnuFileExit: TMenuItem;
    mnuFileLogin: TMenuItem;
    mnuFileLogout: TMenuItem;
    MnuHelp: TMenuItem;
    MnuHelpAbout: TMenuItem;
    NetClnChkListView: TListView;
    NwLocCleanup: TTabSheet;
    OfflineJBmkTotal: Tlabel;
    OfflineJBreport: TMemo;
    onlineJBmkTotal: Tlabel;
    ORaidListView: TListView;
    Panel1: Tpanel;
    pcUtilTabs: TPageControl;
    PnlBottom: Tpanel;
    pnlByDate: Tpanel;
    pnlByIEN: Tpanel;
    pnlByWhat: TPanel;
    pnlFUT49Left: TPanel;
    pnlMessage: Tpanel;
    pnlNLCBot: TPanel;
    pnlNLCMid: TPanel;
    pnlNLCMidRight: TPanel;
    pnlNLCTop: TPanel;
    pnlRetry: TPanel;
    pnlRetryAVI: TAnimate;
    pnlRetryBot: TPanel;
    pnlRetryTitle: TPanel;
    pnlRetryTop: TPanel;
    pnlStatusCurrentFile: Tpanel;
    pnlStatusCurrentIEN: Tpanel;
    pnlStatusFileEmpty: Tpanel;
    pnlStatusRecentErrorCountLabel: Tpanel;
    pnlStatusTotalCount: Tpanel;
    pnlVars: TPanel;
    ProgressBar1: TProgressBar;
    RAIDlocation: TGroupBox;
    rgFut49ChkType: TRadioGroup;
    rgOrphanFiles: TRadioGroup;
    Splitter1: TSplitter;
    Timer1: TTimer;
    TimerCleanup: TTimer;
    TimerOrphanEnabling: TTimer;
    TimerValidateIENValues: TTimer;
    ViewLog: TMenuItem;
    MagUtilityHelp1: TMenuItem;
    Procedure actStorageByDateFromExecute(Sender: Tobject);
    Procedure actStorageByDateToExecute(Sender: Tobject);
    Procedure actStorageLUDestExecute(Sender: Tobject);
    Procedure actStorageLUSourceExecute(Sender: Tobject);
    Procedure BtnBackOnlineJBClick(Sender: Tobject);
    Procedure btnHideClick(Sender: Tobject);
    Procedure btnImgOFLclrLstClick(Sender: Tobject);
    Procedure btnNtLocDelRunClick(Sender: Tobject);
    Procedure btnPauseFUT49Click(Sender: Tobject);
    Procedure btnProcessFUT49Click(Sender: Tobject);
    Procedure BtnRunJBOfflineClick(Sender: Tobject);
    Procedure btnRunProcessClick(Sender: Tobject);
    Procedure btnStopProcessingClick(Sender: Tobject);
    Procedure cbxFromShareLocEnter(Sender: Tobject);
    Procedure cbxToShareLocEnter(Sender: Tobject);
    procedure EdtEndDTEnter(Sender: TObject);
    Procedure edtEndIENexit(Sender: Tobject);
    procedure edtStartDTEnter(Sender: TObject);
    Procedure edtStartIENexit(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    Procedure FormCreate(Sender: Tobject);
    procedure FormDestroy(Sender: TObject);
    Procedure FormKeyPress(Sender: Tobject; Var Key: Char);
    procedure FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    Procedure FormResize(Sender: Tobject);
    Procedure FUT49AftHourClick(Sender: Tobject);
    Procedure FUT49FromLocClick(Sender: Tobject);
    Procedure FUT49RemoveOldClick(Sender: Tobject);
    Procedure FUT49SetNewLocClick(Sender: Tobject);
    Procedure FUT49ToLocClick(Sender: Tobject);
    Procedure lbRAIDSharesClick(Sender: Tobject);
    Procedure mnuFileExitClick(Sender: Tobject);
    Procedure mnuFileLoginClick(Sender: Tobject);
    Procedure mnuFileLogoutClick(Sender: Tobject);
    Procedure MnuHelpAboutClick(Sender: Tobject);
    Procedure NetClnChkListViewMouseUp(Sender: Tobject; Button: TMouseButton; Shift: TShiftState; x, y: Integer);
    procedure NetClnChkListViewSelectItem(Sender: TObject; Item: TListItem;Selected: Boolean);
    Procedure OfflineJBreportChange(Sender: Tobject);
    Procedure ORaidListViewMouseUp(Sender: Tobject; Button: TMouseButton; Shift: TShiftState; x, y: Integer);
    Procedure pcUtilTabsChange(Sender: Tobject);
    procedure pnlVarsClick(Sender: TObject);
    procedure rgOrphanFilesClick(Sender: TObject);
    Procedure StorageCopyEnabling(Sender: Tobject);
    Procedure Timer1Timer(Sender: Tobject);
    procedure TimerCleanupTimer(Sender: TObject);
    Procedure TimerOrphanEnablingTimer(Sender: Tobject);
    Procedure TimerValidateIENValuesTimer(Sender: Tobject);
    Procedure ViewLogClick(Sender: Tobject);
    procedure MagutilityHelp1Click(Sender: TObject);

  Protected
    attnerrorcnt: integer; //t17 - orphan files check, attention error cnt
    boInRetryLoop: Boolean;
    boRecordKeystrokes: Boolean;
    boSaveKeystrokes: Boolean;
    CurrentRAIDname : String;
    errorcnt: Integer;
    FCompletedLogFilename: String; // Filename of completed log file
    FErrorLogger: TLogger; // Default Error logger
    Filename: String;
    FLogger: TLogger; // Default information logger
    FLogsDir: String; // the directory to write log files to
    Fport: String; // Vista Port connected to
    FPrimarySiteStationNumber: String; // the primary site station number for the site the user is logged into
    FProcessingPaused: Boolean;
    FServer: String; // VistA Server connected to
    FUT49FromLoc: String;
    FUT49IEN1: Integer;
    FUT49Mark: String;
    FUT49Running: Boolean;
    FUT49ToLoc: String;
    FUT49warnMsg: String;
    HEC1010Running: Boolean;
    HECIENbegin: String;
    HECIENend: String;
    HideDetail: Boolean;
    LastErrorCode: integer; //t17 - show system error code
    logsummary: String; //t17 - log summary (orphan/storage copy)
    lstKeyStrokes: TStringlist;
    MagUtilityBroker: TMagUtilityBroker; // Broker connection to VistA database
    MagUtilityReady: Boolean; // Determines if it is ready to begin processing (initialization is complete)
    NetworkLocClnRunning: Boolean;
    OfflineJBImgRunning: Boolean;
    OKCNT: Integer;
    OrphanFileName: String;
    OrphanFilesRunning: Boolean;
    OrphanFixed: Integer;
    PLACE: String;
    ProblemIndex: Integer;
    PtrEmptyCnt: Integer;
    RAIDtcnt: Integer;
    RemoveOrphanFile: Boolean;
    showdetail: Integer; //display fix non 14.3 format
    totalcnt: Integer;
    Function checkOrphanFile(sgLoopName, netlocien, Path: String; FileRec: TSearchRec): Boolean;
    Function DoIExist(Semaphore: String): Boolean;
    Function ExecuteShellCommand(CmdLine: String; hidden: Boolean): Boolean;
    Function FileExistOther(MagIN, FileFulNm, Filetype, Filesize: String): Boolean;
    Function FileLastModified(Const TheFile: String): String;
    Function FileTimeToDTime(FTime: TFileTime): Tdatetime;
    Function FMEXTDT(Fmdt: String): String;
    Function Get_File_Size2(sFileToExamine: String): String;
    Function GetAppPath(): String;
    Function IsStrANumber(Const s: String): Boolean;
    Function LogIn(Server, Port: String): Boolean;
    Function MagPiece(Str, Del: String; Piece: Integer): String;
    Function NetClnListView(): Boolean;
    Function OrphanRAIDListView(): Boolean;
    Function SetFileDates(Const Filename: String; Const CreationDate: Tdatetime; Const AccessedDate: Tdatetime; Const ModifiedDate: Tdatetime): Boolean;
    Function ToCopyFile(Var MAGR: TStrings;Var Copied,NofTot: String): Boolean;
    Function validateRange(StartIEN: String; endIEN: String; Var Msg: String; Var ProblemIndex: Integer): Boolean;
    Procedure BtnFUT49resumeClick();
    Procedure DELandSET(netlocien, Filename, Fname: String; Filesize, setptr: Integer);
    Procedure DelOrphanFile(fulfile, Fname: String; Filesize: Integer);
    Procedure dirtree(RAIDindex: Integer; Path: String; FileAttrs: Integer; FileRec: TSearchRec);
    Procedure edtStartDTonExit(Sender: Tobject);
    Procedure FUT49chktime();
    Procedure UpdateLocPT(Magien, LOCien, Filename: String);
    Procedure UpdateMainFormCaption();
  Public
    bmpGo            : TBitmap;
    bmpStop          : TBitmap;
    FUT49curIEN      : Integer;
    FUT49FromShareLoc: String;
    FUT49IEN2        : Integer;
    FUT49NET0        : String;
    FUT49ToShareLoc  : String;
    Function checkOrphanFileLoop(LoopName, netlocien,Path: String; FileRec: TSearchRec;boShowMessage: Boolean): Boolean;
    Function EndDTonExit(Sender: Tobject): Boolean;
    Function EndIENexit(Sender: Tobject): Boolean;
    Function ExecuteShellCommandLoop(LoopName,CmdLine: String; hidden: Boolean): Boolean;
    Function ExitFromLoc(Sender: Tobject): Boolean;
    Function ExitToLoc(Sender: Tobject): Boolean;
    Function LookupFromShareLoc(Sender: Tobject): Boolean;
    Function LookupToShareLoc(Sender: Tobject): Boolean;
    Function TakeANap(TotalTime: Integer): Boolean;
    Function WindowsTempDirectory: String;
    Procedure AdjustEnabled(Sender: TWinControl; Enabled: Boolean);
    Procedure ArrangeStatusLabels();
    Procedure ComputeColumnWidths(LV: TListView;var FColZeroWidth: Integer;var FColWidth: Array Of Integer);
    Procedure EnableRunButtonNetworkCleanup();
    Procedure EnableRunButtonOrphanFiles();
    Procedure EnableRunButtonStorageCopy();
    Procedure EndProcessMsg(ProcessName: String;EndedSuccessfully,ViewLogs: Boolean;OKCNT,errorcnt: Integer;CopiedLabel,NotCopiedLabel: TLabel);
    Procedure FitColumnsToText(LV: TListView);
    Procedure FUT49Resume(curIEN,IEN2: Integer;NET0: String);
    Procedure FUT49Run(Sender: Tobject);
    Procedure FUT49RunInit(Sender: Tobject);
    Procedure InitAll(Sender: Tobject);
    Procedure InitStorageCopy(Sender: Tobject);
    Procedure InitStorageCopyPanelByDate(IsEnabled: Boolean);
    procedure InitStorageCopyPanelByIEN(IsEnabled: Boolean);
    Procedure MakeFocused(Sender: TWinControl);
    Procedure OrphanFilesCancel(Msg: String);
    Procedure ProgressUpdate(Prog: TGauge;Cur,Max,Min:Integer);
    Procedure SetCaptionFUT49byDt2ien(s: String);
    Procedure SetCaptionFut49byDt4ien(s: String);
    Procedure SetCaptionFut49byIEN2DT(s: String);
    Procedure SetCaptionFut49byIEN4DT(s: String);
    Procedure SetCurIEN(var curIEN: Integer;newValue: Integer);
    Procedure SetListViewWidth(LV: TListView);
    Procedure SetPnlRetryVis(IsVisible: Boolean);
    Procedure ShowMessage(const Msg: string);
    Procedure StorageCopyCancel(LogMsg,ShowMsg: String);overload;
    Procedure StorageCopyCancel(LogMsg,ShowMsg: String;errorcount: Integer);overload;
    Procedure StorageCopyWarning(Sender: TObject);
    Procedure TestTakeANap();
    Procedure UpdateLabel(l: TLabel;i: Integer); overload;
    Procedure UpdateLabel(l: TLabel;m: String); overload;
    Procedure ValidateIENChars(Var Key: Char);
    Function ExecuteNewProcess(FileName:String;Visibility:integer):integer;
    Function ExecuteKnownFileType(Handle:THandle;FileName:String): Boolean;
  End;

Var
  fMagUtility: TfMagUtility;

Implementation
Uses
  ads_DlgLU,
  cMagUtilityUtils,
  Dialogs,
  FMagAbout,
  fMagCalendar,
  Magfileversion,
  Math,
  Shellapi,
  StrUtils,
  TFileAppenderUnit;

Var
  UnitName: String = 'uMagUtility';
  
{$R *.dfm}
{$R MagUtilityResources.RES}

Procedure TfMagUtility.FormCreate(Sender: Tobject);
Var
  Server, Port: String;
  ans: Integer;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.FormCreate';
  boRecordKeystrokes:= False;
  boSaveKeystrokes:= False;
  lstKeyStrokes:= TStringlist.Create;
  FUT49AftHour.Caption:=
    'Run after-hours'+#13+#10+
    '6 PM to 6 AM weekdays'+#13+#10+
    'and all weekend.';//508
  bmpGo  := TBitmap.Create();
  bmpStop:= TBitmap.Create();
  boInRetryLoop:= False;
  InitAll(Sender);
  LastErrorCode         :=0;
  OrphanFilesRunning    := False;
  FUT49FromShareLoc     := '';
  FUT49ToShareLoc       := '';
  pcUtilTabs.ActivePage := MagOrphan;
  cbxFromShareLoc.OnExit := Nil;
  cbxToShareLoc.OnExit   := Nil;
  Server := '';
  Port := '';
  FLogsDir := CMagUtilityUtils.GetLogsDir();
  If Not Directoryexists(FLogsDir) Then
  Begin
    Forcedirectories(FLogsDir);
  End;
  FLogger := TLogger.GetInstance('MagUtilityInfo');
  FErrorLogger := TLogger.GetInstance('MagUtilityDebug');
  If DoIExist('VistA Imaging MagUtility') Then
  Begin
    // notify the user and terminate the 2nd instance of the applications
    Self.Hide();
    Application.Processmessages;
    With Application Do
    Begin
      NormalizeTopMosts;
      ans := MessageBox('Warning "VistA Imaging - MagUtility" is already running. ' + #13 + #13 +
        'Do you want to start 2nd instance?' + #13 + #13 + 'Note: Most MagUtility processes can be run concurrently, and/or from multiple instances of MagUtility on multiple servers.' + #13 +
        'However, extra care should be taken when running the Orphan Files and Storage Copy processes. It is recommended' + #13 +
        'that Orphan Files and Storage Copy be run separately, and only on one instance of MagUtility.' + #13 + #13 +
        'If there is a need to run the Orphan Files and Storage Copy processes at the same time on the same RAID shares, ' + #13 +
        'contact the Imaging Group for guidance before proceeding.', 'VistA Imaging - MagUtility', MB_OKCANCEL);
      If ans = 2 Then //cancel
      Begin
        Close();
        Terminate();
        Processmessages();
        Exit;
      End;
      RestoreTopMosts;
      FLogger.Info(#9 + 'Startup' + #9 + #9 + 'Open 2nd instance MagUtility'); //t17 LOG
      FErrorLogger.Info(#9 + 'Startup' + #9 + #9 + 'Open 2nd instance MagUtility'); //t17 LOG
    End;
  End;
  EnableRunButtonOrphanFiles();

  MagUtilityBroker := TMagUtilityBroker.Create(FLogger, FErrorLogger);
  FLogger.Info(#13 + ' '); //(t17 remove dup line)
  FErrorLogger.Info(#13 + ' ');
  FLogger.Info(#9 + 'Startup' + #9 + #9 + 'Open MagUtility'); //t17 LOG
  FErrorLogger.Info(#9 + 'Startup' + #9 + #9 + 'Open MagUtility'); //t17 LOG
  // RAID listing execute in timer1 for completion
  With pnlRetryAVI Do
  Begin
    ResName := 'FlyingRetry'; //t17 - IR T15-28
    ResHandle := hInstance;
    Active := FALSE;
  End;
  Timer1.Enabled := True;
End;

(*
 Determines if the specified application name is already active
 @param semaphore The name of the application to find
 @return True if the application is running, false otherwise
*)

Function TfMagUtility.DoIExist(Semaphore: String): Boolean;
Var
  HSem: THandle;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.DoIExist';
  HSem := OpenSemaphore(1, False, PChar(Semaphore));
  If HSem = 0 Then
  Begin
    Result := False;
    CreateSemaphore(Nil, 0, 1, PChar(Semaphore));
  End
  Else
    Result := True;
End;

(*
   Login VistA DB, verify user's `Access/Verify Codes
*)

Function TfMagUtility.LogIn(Server, Port: String): Boolean;
Var
  userKeys: Tstringlist;
  loginResult: TMagUtilityBrokerLoginResult;
  Msg, chkversion: String;
  ProcName: String;
Begin
  pcUtilTabs.ActivePage:=pcUtilTabs.Pages[0];  //T21 - switch back to Orphan screen
  ProcName:='TfMagUtility.LogIn';
  If pnlMessage.caption = 'Connected to VistA' then Exit; //already connected
  Result:=False;
  Try
    Cursor := crHourGlass;
    If Server = '' Then Server := 'Brokerserver';
    If Port = '' Then Port := '9200';
    loginResult := MagUtilityBroker.LogIn(Server, Port, Msg);
    If loginResult = MagUtilityLoginOK Then
    Begin
      Result:=True;
      chkversion := '3.0P98'; //version checking
      If Not MagUtilityBroker.MagInstall(chkversion) Then
      Begin
        Showmessage('MAG' + chkversion + ' KIDS installation is required to run MagUtility client. MagUtility will exit...');
        //FLogger.Error(',*** Version check, the host system did not have MAG'+chkversion+' KID installed.');
        FLogger.Error(#9 + 'Startup' + #9 + #9 + #9 + 'Version check, the host system did not have MAG' + chkversion + ' KID installed.');  //T18
        FErrorLogger.Error(#9 + 'Startup' + #9 + #9 + #9 + 'Version check, the host system did not have MAG' + chkversion + ' KID installed.'+' ('+UnitName+'.'+ProcName+'.21)');
        TLogger.FreeInstances;
        Application.Terminate;
      End;
      FServer := Server;
      Fport := Port;
      userKeys := Tstringlist.Create();
      MagUtilityBroker.getUserKeys(userKeys);
      Cursor := crHourGlass;
      // if there was a previous login then there would already be an appender - remove
      // it so we can set a new one which might be a new file
      Cursor := crDefault;
      pnlMessage.caption := 'Connected to VistA';
      //btnRunProcess.Enabled := True;
      ////btnProcessHEC1010.Enabled:= true;
      BtnRunJBOffline.Enabled := True;
      BtnBackOnlineJB.Enabled := True;
      btnNtLocDelRun.Enabled := True;
      //btnProcessFUT49.Enabled := True;
      mnuFileLogin.Enabled := False;
    End //MagUtilityLoginOK
    Else
      If loginResult = MagUtilityLoginContextFailed Then
      Begin
        pnlMessage.caption := 'No connection to VistA';
        Showmessage('Unable to login to database, you must have the ' +
          MagUtilityBroker.getRequiredContext() + ' context to use MagUtility' + #13 +
          'Connection to VistA was closed');
           //FLogger.Error(',*** User did not have the context(Mag Utility) to login VistA, connection closed. ,');
        FLogger.Error(#9 + 'Startup' + #9 + #9 + 'User did not have the context(Mag Utility) setup to login VistA, connection closed');
        FErrorLogger.Error(#9 + 'Startup' + #9 + #9 + 'User did not have the context(Mag Utility) setup to login VistA, connection closed');//T20
        mnuFileLogin.Enabled := True;
      End
      Else
      Begin
        pnlMessage.caption := 'No connection to VistA';
        Showmessage('Error connecting to VistA' + #13 + Msg);
        TLogger.FreeInstances;
        Application.Terminate;
      End;
  Finally //end Try
    Cursor := crDefault;
  End;
End;

//  Init all menu listing value
Procedure TfMagUtility.Timer1Timer(Sender: Tobject);
Var
  Site, uname: String;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.Timer1Timer';
  Timer1.Enabled := False;
  totalcnt := 0;
  // VistA login
  LogIn('', ''); //cancel login
  If pnlMessage.caption <>'Connected to VistA' Then Exit;
  // Get user PLACE - DUZ(2)
  MagUtilityBroker.getUserInfo(Site, uname, PLACE);
  fMagUtility.caption := ' MagUtility (' + Site + ') in use by : ' + Uname;
  // Tab#1 - RAID listing DB request
  OrphanRAIDListView();
  lblFileEmpty.Visible := False;

  // Tab#5 - Storage Copy - init value

  // Tab#3 - Offline JB image
  btnStopProcessing.Enabled := False;
  btnPauseFut49.Enabled := False;
  btnImgOflClrLst.Enabled := False;

End; //end of Timer1Timer

(*
 Menu option | Sign In | Login, logs into VistA
 @param Sender The object which sent the event
*)

Procedure TfMagUtility.mnuFileLoginClick(Sender: Tobject);
Var
  Server, Port: String;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.mnuFileLoginClick';
  If AnsiContainsStr(pnlMessage.caption, 'Connection to VistA') Then Exit;
  Server := '';
  Port := '';
  LogIn(Server, Port);
  Timer1.Enabled := True; //enable all menu options
  mnuFileLogout.Enabled := True;
  ORaidListView.Enabled := True;
  NetClnChkListView.Enabled := True;
  btnOflJBSel.Enabled := True;
  OfflineJBReport.Enabled := True;
  cbxOfflineJBList.Enabled := True;
  rgFut49ChkType.Enabled :=True;
End;

//  menu Option | Sing In | Exit

Procedure TfMagUtility.mnuFileLogoutClick(Sender: Tobject);
Var
  warnmessage: String;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.mnuFileLogoutClick';
  If OrphanFilesRunning Or HEC1010Running Or OfflineJBImgRunning Or NetworkLocClnRunning
    Or FUT49Running Then
  Begin
    If OrphanFilesRunning Then
      warnmessage := 'Orphan Files'
    Else
      If HEC1010Running Then
        warnmessage := 'HEC 10-10 Queue'
      Else
        If OfflineJBImgRunning Then
          warnmessage := 'Offline Jukebox Images'
        Else
          If NetworkLocClnRunning Then
            warnmessage := 'Network Location Cleanup'
          Else
            If FUT49Running Then warnmessage := 'Storage Copy';
    Showmessage('There is a MagUtility process - ' + warnmessage + ' running. Wait until the process is complete, or cancel the process before loging out MagUtility.');
    Exit;
  End;
  If MagUtilityBroker <> Nil Then MagUtilityBroker.Logout;
  pnlMessage.caption := '!! No connection to VistA !!';
  fMagUtility.caption := ' MagUtility';
  mnuFileLogin.Enabled := True;
  mnuFileLogout.Enabled := False;
  //disable all
  btnRunProcess.Enabled := False;
  btnStopProcessing.Enabled := False;
  BtnRunJBOffline.Enabled := False;
  btnImgOFLclrLst.Enabled := False;
  BtnBackOnlineJB.Enabled := False;
  btnNtLocDelRun.Enabled := False;
  btnProcessFUT49.Enabled := False;
  btnPauseFUT49.Enabled := False;
  mnuFileLogin.Enabled := True;
  ORaidListView.Clear;
  ORaidListView.Enabled := False;
  NetClnChkListView.Clear;
  NetClnChkListView.Enabled := False;
  cbxFromShareLoc.ClearSelection;
  cbxFromShareLoc.Items.Clear;
  cbxToShareLoc.ClearSelection;
  cbxToShareLoc.Items.Clear;
  btnOflJBSel.Enabled := False;
  OfflineJBReport.Enabled := False;
  cbxOfflineJBList.Enabled := False;
  rgFut49ChkType.Enabled := False;
  UpdateMainFormCaption();
End;

Procedure TfMagUtility.UpdateMainFormCaption();
Var
  ProcName: String;
Begin
  ProcName:='TfMagUtility.UpdateMainFormCaption';
  If MagUtilityBroker = Nil Then
    Self.caption := 'VistA Imaging MagUtility'
  Else
  Begin
    If MagUtilityBroker.Connected Then
    Begin
      Self.caption := 'VistA Imaging MagUtility : (' + FServer + ') in use by : ' +
        MagUtilityBroker.UserFullName;
    End
    Else
    Begin
      Self.caption := 'VistA Imaging MagUtility';
    End;
  End;
End;

(* ************************************************************************
  Switch tabs - Orphan | Offline image | Network Loc Cleanup | Storage Copy
*)

Procedure TfMagUtility.pcUtilTabsChange(Sender: Tobject);
Var
  Shares     : TStrings;
  warnmessage: String;
  ProcName: String;
Begin ///Re-Fresh each Tab
  //Orphan Files Tab
  ProcName:='TfMagUtility.pcUtilTabsChange';
  rgOrphanFiles.ItemIndex:=0;
  btnRunProcess.Enabled := False;
  btnNtLocDelRun.Enabled := False;
  Shares := Tstringlist.Create();
  lblFileEmpty.Visible := False;
  If AnsiContainsStr(pnlMessage.caption, 'No connection to VistA') Then Exit;
  //Orphan Files Tab
  If pcUtilTabs.ActivePage.Name = 'MagOrphan' Then
  Begin
    FLogger.Info(#9 + 'Orphan Files' + #9 + 'Selected' + #9 + 'User:' + MagUtilityBroker.UserFullName);
    FErrorLogger.Info(#9 + 'Orphan Files' + #9 + 'Selected' + #9 + 'User:' + MagUtilityBroker.UserFullName);
    If HEC1010Running Or OfflineJBImgRunning Or NetworkLocClnRunning Or FUT49Running Then
    Begin
      If OrphanFilesRunning Then
        warnmessage := 'Orphan Files'
      Else
        If HEC1010Running Then
          warnmessage := 'HEC 10-10 Queue'
        Else
          If OfflineJBImgRunning Then
            warnmessage := 'Offline Jukebox Images'
          Else
            If NetworkLocClnRunning Then
              warnmessage := 'Network Location Cleanup'
            Else
              If FUT49Running Then warnmessage := 'Storage Copy';
      Showmessage('Warning, there is a MagUtility process - "' + warnmessage + '" running. Wait until that process is complete.');
    End;
    OrphanRAIDListView();
    If FUT49Running And (Not OrphanFilesRunning) Then
    Begin
      grpProcessStatus.caption := 'Storage Copy process status';
      lblRecentErrorCountLabel.caption := 'IENs that could not be processed:'; //T20 - 228
      lblTotalCount.caption := 'IENs processed successfully:';  //T20 - 228
      lblCurrentFileLabel.Visible := False; //T13 remove 2 speed
      lblCurrentIEN.Visible := False; //T13
      lblCurrentFile.Visible := False; //T13
      lblIEN.Visible := False;
      //lblFileEmpty.Visible := True;
      //lblFileEmpty.Enabled := True;
    End
    Else
    Begin
      grpProcessStatus.caption := 'Orphan process status';
      lblRecentErrorCountLabel.caption := ' Orphan files found:';
      lblTotalCount.caption := 'Total files scanned:';
      lblCurrentFileLabel.Visible := True; //T13 remove 2 speed
      lblCurrentIEN.Visible := True; //T13
      lblCurrentFile.Enabled := True;
      lblCurrentFile.Visible := True; //T13
      lblIEN.Enabled := True;
      lblIEN.Visible := True; //T13
      lblFileEmpty.Visible := False;
      lblFileEmpty.Enabled := False;
    End;
    lblCurrentFileLabel.caption := 'Current file:';
    lblCurrentFile.caption := '';
    lblCurrentIEN.caption := 'Current IEN:';
    lblIEN.caption := '';
    lblCurrentFile.Enabled := True;
    lblIEN.Enabled := True;
    lbltotalcnt.Enabled := True;
    lblErrorcnt.Enabled := True;
    UpdateLabel(lblErrorCnt,'');
    lbltotalcnt.caption := '';
    RemoveOrphanFile := False;
  End; //MagOrphan

  //JB image offline
  If (pcUtilTabs.ActivePage.Name = 'MagJbOffline') Then
  Begin
    If MagUtilityBroker.LstOflJB(Shares) Then
      If Shares.Count > 0 Then
      Begin
        cbxOfflineJBlist.Items := Shares;
        If cbxOfflineJBlist.Items.Count > 0 Then
          If cbxOfflineJBlist.Items[0] <> '0' Then
          Begin
            onlineJBmkTotal.caption := '(total ' + Inttostr(Shares.Count) + ')';
            cbxOfflineJBlist.Text := cbxOfflineJBlist.Items[0];
            cbxOfflineJBList.ItemIndex := 0; //default 1st one
          End
          Else
            cbxOfflineJBlist.Items.Delete(0); ///='0'  nil entry
      End;
    FProcessingPaused := False;
    grpProcessStatus.caption := '';
    lblCurrentFileLabel.caption := '';
    lblCurrentFile.caption := '';
    lblCurrentIEN.caption := '';
    lblIEN.caption := '';
    lblCurrentFile.Enabled := False;
    lblIEN.Enabled := False;
    lbltotalcnt.Enabled := False;
    lblErrorcnt.Enabled := False;
    lblIEN.Enabled := False;
    lblRecentErrorCountLabel.caption := '';
    lblTotalCount.caption := '';
    UpdateLabel(lblErrorCnt,'');
    lbltotalcnt.caption := '';
    BtnRunJBOffline.Enabled := False;
    BtnBackOnlineJB.Enabled := False;
    If cbxOfflineJBList.Items.Count > 0 Then BtnBackOnlineJB.Enabled := True;
    If OfflineJBreport.Lines.Count > 0 Then BtnRunJBOffline.Enabled := True;
    FLogger.Info(#9 + 'Offline Jukebox Images' + #9 + 'Selected'+ #9 + 'User:' + MagUtilityBroker.UserFullName);
    FErrorLogger.Info(#9 + 'Offline Jukebox Images' + #9 + 'Selected'+ #9 + 'User:' + MagUtilityBroker.UserFullName);
    MakeFocused(BtnOflJBSel);
  End;
  //Network Location Cleanup Tab
  If (pcUtilTabs.ActivePage.Name = 'NwLocCleanup') Then
  Begin
    NetClnListView(); //get current network list
    //clean up the status box
    FProcessingPaused := False;
    grpProcessStatus.caption := '';
    lblCurrentFileLabel.caption := '';
    lblCurrentFile.caption := '';
    lblCurrentIEN.caption := '';
    lblIEN.caption := '';
    lblCurrentFile.Enabled := False;
    lblIEN.Enabled := False;
    lbltotalcnt.Enabled := False;
    lblErrorcnt.Enabled := False;
    lblRecentErrorCountLabel.caption := '';
    lblTotalCount.caption := '';
    UpdateLabel(lblErrorCnt,'');
    lbltotalcnt.caption := '';
    NetClnChkListView.ReadOnly := True;
    btnHide.Visible := False;
    FLogger.Info(#9 + 'Network Loc Cleanup' + #9 + 'Selected'+ #9 + 'User:' + MagUtilityBroker.UserFullName);
    FErrorLogger.Info(#9 + 'Network Loc Cleanup' + #9 + 'Selected'+ #9 + 'User:' + MagUtilityBroker.UserFullName);
  End; //NwLocCelanUp

  //Storage Copy Tab
  If pcUtilTabs.ActivePage.Name = 'FUT49' Then
  Begin
    InitStorageCopy(Sender);
    If OrphanFilesRunning Or HEC1010Running Or OfflineJBImgRunning Or NetworkLocClnRunning Then
    Begin
      If OrphanFilesRunning Then
        warnmessage := 'Orphan Files'
      Else
        If HEC1010Running Then
          warnmessage := 'HEC 10-10 Queue'
        Else
          If OfflineJBImgRunning Then
            warnmessage := 'Offline Jukebox Images'
          Else
            If NetworkLocClnRunning Then
              warnmessage := 'Network Location Cleanup'
            Else
              If FUT49Running Then warnmessage := 'Storage Copy';
      Showmessage('Warning, there is a MagUtility process - "' + warnmessage + '" running. Wait until that process is complete.');
    End;
    FProcessingPaused := False;
    If OrphanFilesRunning And (Not FUT49Running) Then
    Begin
      grpProcessStatus.caption := 'Orphan process status';
      lblRecentErrorCountLabel.caption := ' Orphan files found:';
      lblTotalCount.caption := 'Total files scanned:';
      lblFileEmpty.Enabled := False;
      lblFileEmpty.Visible := False; //T13
      lblCurrentFileLabel.Visible := True; //T13 remove 4 speed
      lblCurrentIEN.Visible := True; //T13
      lblCurrentFile.Visible := True; //T13
      lblIEN.Visible := True; //T13
    End
    Else
    Begin
      grpProcessStatus.caption := 'Storage Copy process status';
      lblCurrentFileLabel.Visible := False; //T13 remove 2 speed
      lblCurrentIEN.Visible := False; //T13
      lblCurrentFile.Visible := False; //T13
      lblIEN.Visible := False; //T13
      lblRecentErrorCountLabel.caption := 'IENs that could not be processed:'; //T20 - 228
      lblTotalCount.caption := 'IENs processed successfully:';  //T20 - 228
      //lblFileEmpty.Visible := True;
      //lblFileEmpty.Enabled := True;
    End;
    lblCurrentFileLabel.caption := ''; //'Current file:';
    lblCurrentFile.caption := '';
    lblCurrentIEN.caption := ''; //'Current IEN:';
    lblIEN.caption := '';
    lblIEN.Enabled := True;
    lbltotalcnt.Enabled := True;
    lblErrorcnt.Enabled := True;
    UpdateLabel(lblErrorCnt,'');
    lbltotalcnt.caption := '';
    ProblemIndex := 0;
    FLogger.Info(#9 + 'Storage Copy' + #9 + 'Selected'+ #9 +'User:' + MagUtilityBroker.UserFullName);
    FErrorLogger.Info(#9 + 'Storage Copy' + #9 + 'Selected'+ #9 +'User:' + MagUtilityBroker.UserFullName);
    If (Not FUT49Running) Then
    Begin
      FUT49WarnMsg :=
        'The Storage Copy utility is not a disk duplication tool.'+#13+#10+#13+#10+
        #149+#32+'Files that are not referenced by IMAGE file(#2005) in the VistA database will NOT be copied.'+#13+#10+#13+#10+
        #149+#32+'Run the Verifier and the Orphan Files utility (if the share is a RAID)'+#13+#10+
        #32+#32+'on the "source" share before proceeding with the copy.';
      InitStorageCopy(Sender); //reset all
      StorageCopyWarning(Sender);
      //MakeFocused(btnSel2Dt);
    End;
  End; //FUT49
  ArrangeStatusLabels();
End;

//  Call to refresh Orphan File RAID listing

Function TfMagUtility.OrphanRAIDListView(): Boolean;
Var
  chkvar   : String;
  i        : Integer;
  icnt     : Integer;
  Ien      : String;
  iplace   : String;
  Listitem : TListItem;
  NewColumn: TListColumn;
  OnLine   : String;
  Rname    : String;
  route    : String;
  share    : String;
  Shares   : TStrings;
  Status   : Boolean;
  xline    : String;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.OrphanRAIDListView';
  Result:=False;
  ORaidListView.Clear;
  ORaidListView.Columns.CleanupInstance;
  Status := MagUtilityBroker.getRAIDLocations(Shares);
  If Status = True Then
  Begin
    //list OrphanRAIDlistView.Items w/ fileter from Shares;
    icnt := 1;
    If ORaidListView.Columns.Count < 2 Then //build cloumn header
      For i := 0 To 3 Do
      Begin
        Newcolumn := ORaidListview.Columns.Add;
        If i = 0 Then
          Newcolumn.caption := 'RAID network-location share'
        Else
          If i = 1 Then
            Newcolumn.caption := ' Name '
          Else
            If i = 2 Then
              Newcolumn.caption := 'IEN#'
            Else
              Newcolumn.caption := 'Online';

        //ORaidListView.Columns[i].Width := ColumnHeaderWidth;
        ORaidListView.Columns[i].Width := 20;
      End;
    For i := 0 To Shares.Count - 1 Do
    Begin
      xline := Shares[i];
      share := MagPiece(xline, '^', 2);
      xline := Uppercase(xline);
      Ien := MagPiece(xline, '^', 1);
      OnLine := MagPiece(xline, '^', 4);
      iplace := MagPiece(xline, '^', 10);
      RName := MagPiece(xline, '^', 11); //RAID share name
      If ((iplace = ' ') Or (iplace = PLACE)) And (OnLine = '1') Then /// RAID is the same PLACE/SITE as the user's
      Begin
        chkvar := '$P($G(^MAG(2005.2,' + Ien + ',0)),U,9)_" "';
        route := MagUtilityBroker.FnmGetNode(chkvar);
        If (route = ' ') Or (route = '0 ') Then /// not the router loc
        Begin
          Listitem := ORaidListView.Items.AddItem(Nil, icnt);
          Listitem.caption := Inttostr(icnt);
          Listitem.SubItems.Add(share); //0
          Listitem.SubItems.Add(Rname); //1
          Listitem.SubItems.Add(Ien);   //2
          Listitem.SubItems.Add('Yes'); //3
          //ListItem.SubItems.Add(site);
          icnt := icnt + 1;
        End;
      End;
      Result:=True;
    End; //for I
    RAIDtcnt := icnt - 1;
    ORaidListView.ReadOnly := True;
  End;
  //508 Start
  If Shares.Count>0 Then
  Begin
    If ORaidListView.ItemIndex= -1 Then
    Begin
      ORaidListView.ItemIndex:= 0;
      ORaidListView.Items[0].Selected:=True;
      ActiveControl:=ORaidListView;
      EnableRunButtonOrphanFiles();
    End;
  End;
  //508 End
  Application.ProcessMessages();
  If ORaidListView.Columns.Count>1 Then SetListViewWidth(ORaidListView);
End;

(*
  Menu #1| Orphan RAID file | Select RAID - select one or more RAID share(s)
  for orphan files search
*)

Procedure TfMagUtility.lbRAIDSharesClick(Sender: Tobject);
Var
  netlocien, netpath: String;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.lbRAIDSharesClick';
  If OrphanFilesRunning And btnStopProcessing.Enabled Then
  Begin
    Showmessage('Error - Orphan files task is running now');
    Exit;
  End;
  totalcnt := 0;
  errorcnt := 0;
  attnerrorcnt := 0; //t17
  btnRunProcess.Enabled := (ORAIDListVIew.SelCount>0);
  FProcessingPaused := False;
  btnStopProcessing.Enabled := False;
  lblCurrentFileLabel.caption := 'Current file:';
  lblCurrentIEN.caption := 'Current IEN:';
  lblRecentErrorCountLabel.caption := ' Orphan files found:';
  lblTotalCount.caption := 'Total files scanned:';
  lblCurrentFile.caption := '';
  lblIEN.caption := '';
  UpdateLabel(lblErrorCnt,'');
  lbltotalcnt.caption := '';
  If ORAIDListVIew.ItemIndex < 0 Then Exit; //click null/space
  netlocien := ORaidListView.Selected.SubItems[2];//IR# T16- 15
  netpath := ORaidListView.Selected.SubItems[0];
End;

(*
   menu #1 | Orphan Files | Cancel
*)

Procedure TfMagUtility.btnStopProcessingClick(Sender: Tobject); //Cancel Orphan task
Var
  ProcName: String;
Begin
  ProcName:='TfMagUtility.btnStopProcessingClick';
  OrphanFilesCancel(#9 + 'Orphan Files' + #9 + #9 + 'Canceled by user: ' + MagUtilityBroker.UserFullName); //t17
End;

(*
  Menu #1| Orphan RAID file |  Run
*)

Procedure TfMagUtility.btnRunProcessClick(Sender: Tobject);
Var
  DirPath: String;
  FileAttrs: Integer;
  FileRec: TSearchRec;
  i: Integer;
  netlocien: String;
  netpath: String;
  RAIDname: String;
  Selection: String;
  sgLoopName: String;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.btnRunProcessClick';
  If FUT49Running Then
  Begin
    Showmessage(
      'Warning!  A Storage Copy process is already running.' + #13 + #10+
      'Only one process can run at a time.  Try again when ' + #13 + #10+
      'the Storage Copy process has completed.');
    Exit;
  End;
  boInRetryLoop:=False;
  sgLoopName:='Orphan';
  LastErrorCode :=0;
  //Run Orphan files
  totalcnt := 0;
  errorcnt := 0;
  OrphanFixed :=0; //T20
  attnerrorcnt := 0; //t17
  lblIEN.caption := '';
  lblCurrentFile.caption := '';
  UpdateLabel(lblErrorCnt,'');
  lbltotalcnt.caption := '';
  If ORaidListView.Selcount < 1 Then //must select at least one RAID share
  Begin
    Showmessage('Select one or more shares to process.');
    Exit;
  End;
  //ReadSelectedOrphans();
  OrphanFilesRunning := True;
  btnStopProcessing.Enabled := True;
  btnRunProcess.Enabled := False;
  CurrentRAIDname := ''; //T21
  ProblemIndex := 0;
  Selection := '';
  RemoveOrphanFile := False;
  FProcessingPaused := False;
  lblCurrentFileLabel.caption := 'Current file:';
  lblCurrentIEN.caption := 'Current IEN:';
  lblRecentErrorCountlabel.caption := ' Orphan files found:';
  lblTotalCount.caption := 'Total files scanned:';
  FileAttrs := FaAnyFile;

  If rgOrphanFiles.ItemIndex=0 Then
    RemoveOrphanFile := False //showmessage('Reporting orphan files on RAID only')
  Else
    RemoveOrphanFile := True;
  If RemoveOrphanFile Then
    Selection := 'Clean up orphan files and generate report'
  Else
    Selection := 'Generate report only';

  FLogger.Info(#9 + 'Orphan Files' + #9 + 'Run'+ ' by user:' + MagUtilityBroker.UserFullName); //t17
  FErrorLogger.Info(#9 + 'Orphan File' + #9 + 'Run'+ ' by user:' + MagUtilityBroker.UserFullName); //t17
  For i := 0 To ORAIDListVIew.Items.Count - 1 Do
  Begin //selected RAID(s)
    If ORaidListView.Items[i].Selected And (FProcessingPaused = False) Then
    Begin
      netlocien := ORaidListView.Items[i].SubItems[2];
      netpath := ORaidListView.Items[i].SubItems[0];   //physical ref
      RAIDname := ORaidListView.Items[i].SubItems[1];  //T20 - CR200 RAID name
      CurrentRAIDname := RAIDname;  //T21  - place name in LOG
      FLogger.Info(#9 + 'Orphan Files' + #9 + 'Selected RAID location '+RAIDname+ '(ien '+netlocien+') "' + netpath + '" with option "' + Selection + '" by user:' + MagUtilityBroker.UserFullName); //t17
      FErrorLogger.Info(#9 + 'Orphan File' + #9 +'Selected RAID location '+RAIDname+ '(ien '+netlocien+') "' +netpath + '" with option "'+Selection+'" by user:' + MagUtilityBroker.UserFullName); //t17
      If Not Directoryexists(netpath) Then //check if the share is accessible
      Begin
        Showmessage('Selected RAID location path '+RAIDname + '(ien '+netlocien+') "' + netpath + '" cannot be accessed.  Check if it is online.');
        FLogger.Error(#9 + 'Orphan Files' + #9 + 'Error - RAID location ' + RAIDname + '(ien '+netlocien+') "' + netpath + '" cannot be accessed');
        FErrorLogger.Error(#9 + 'Orphan Files' + #9 + 'Error - RAID location ' + RAIDname + '(ien '+netlocien+') "' + netpath + '" cannot be accessed'+' ('+UnitName+'.'+ProcName+'.23)');  //T18
        btnStopProcessing.Enabled := False;
        OrphanFilesRunning := False;
        Exit;
      End;

      If Not Directoryexists(netpath) Then
        Showmessage('Network path *' + netpath + '* cannot be accessed.  Check if it is online.')
      Else //loop dir tree/path
      Begin
        lblCurrentFile.caption := '';
        DirPath := netpath;
        If FindFirst(DirPath + '\*.*', (Fadirectory), FileRec) = 0 Then
          While (FindNext(FileRec) = 0) And OrphanFilesRunning Do
          Begin
            //if (FileRec.Attr and FileAttrs) = FileRec.Attr then
            If (FileRec.Name <> '.') And (FileRec.Name <> '..') Then
            Begin
              //showmessage(FileRec.Name);
              If (FileRec.Attr = 32) Then
              Begin
                If Not checkOrphanFileLoop(sgLoopName, netlocien,netPath,FileRec,True) Then Exit;
              End
              Else
                If (FileRec.Attr = Fadirectory) Then dirtree(i, DirPath + '\' + FileRec.Name + '\', FileAttrs, FileRec);
            End;
          End;
        FindClose(FileRec);
      End; //loop dir tree /path
      lblCurrentFileLabel.caption := 'Last File:';
      lblCurrentIEN.caption := 'Last File IEN:';
    End;
  End;
  // Give a summary in LOG  & require attention orphan file //t17
  //T21 - Total files reviewed: aaaaa - orphan files found: xxxx, orphan files fixed: yyyy, orphan files needing review: zzzz
  //logsummary := 'Total '+ inttostr(totalcnt) + ' files reviewed - '; //T18
  logsummary := 'Total files reviewed: ' + inttostr(totalcnt) + ' - ' + 'orphan files found: ' +inttostr(errorcnt); //T21
  If RemoveOrphanFile Then //clean up and report
     logsummary := logsummary + ' - orphan files fixed: ' + inttostr(errorcnt-attnerrorcnt) + ' - orphan files needing review: ' + inttostr(attnerrorcnt);//T21
  FLogger.Info(#9 + 'Orphan Files' + #9 + #9 + logsummary); //t17
  FErrorLogger.Info(#9 + 'Orphan Files' + #9 + #9 + logsummary); //t17
  If RemoveOrphanFile and (errorcnt>0) then UpdateLabel(lblErrorCnt,inttostr(errorcnt)+ '    Fixed:'+ inttostr(errorcnt-attnerrorcnt)) //T18  - replace attn*
  Else UpdateLabel(lblErrorCnt,inttostr(errorcnt));  //T18 - last freshed screen
  Application.ProcessMessages(); //T18 - last freshed status screen
  If OrphanFilesRunning Then
      Showmessage('Orphan Files check is complete. View log file for details'); //t17 9/28/09 - Complete Pop up message
  If FProcessingPaused Then
    Begin
    FLogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Orphan files check canceled.');
    FErrorLogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Orphan files check canceled.') ;  //t17
    End
  Else
    Begin
    FLogger.Info(#9 + 'Orphan Files' + #9 + 'Completed Orphan files check.');
    FErrorLogger.Info(#9 + 'Orphan Files' + #9 + 'Completed Orphan files check. (selected RAID ' + RAIDname +')'); //T20, t17
    End;
  btnStopProcessing.Enabled := False;
  OrphanFilesRunning := False;
End;

(*
Directory tree - search all files and folder
Called by  btnRunProcessClick
*)

Procedure TfMagUtility.dirtree(RAIDindex: Integer; Path: String; FileAttrs: Integer; FileRec: TSearchRec);
Var
  netlocien: String;
  netlocphy: String;
  sgLoopName: String;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.dirtree';
  ///SHOWMESSAGE(path + #13 +FileRec.name);  //+#13+ORaidListView.Items[RAIDindex].SubItems[0]+#13+ORaidListView.Items[RAIDindex].SubItems[1]+#13+ORaidListView.Items[RAIDindex].SubItems[2]+#13+ORaidListView.Items[RAIDindex].SubItems[3]+#13 +ORaidListView.Items[RAIDindex].SubItems[4]+'*');
  sgLoopName:= 'Orphan';
  netlocien := ORaidListView.Items[RAIDindex].SubItems[2]; ///ntw loc ien
  netlocphy := ORaidListView.Items[RAIDindex].SubItems[0]; ///physical ref.
  If FindFirst(Path + '*.*', FileAttrs, FileRec) = 0 Then
    Repeat ///searching
      If (FileRec.Attr And FileAttrs) = FileRec.Attr Then
      Begin
        If (FileRec.Name <> '.') And (FileRec.Name <> '..') Then
        Begin
          lbltotalcnt.caption := Inttostr(totalcnt);
          UpdateLabel(lblErrorCnt,errorcnt);
          If FProcessingPaused Then Exit; /// user does Cancel task
          If (FileRec.Attr = Fadirectory) Then dirtree(RAIDindex, Path + FileRec.Name + '\', FileAttrs, FileRec); //directory
          If (FileRec.Attr = 32 ) Or (FileRec.Attr = 33) Then
          Begin
            //showmessage(FileRec.Name+#13+ InttoStr(FileRec.Attr));
            If Not checkOrphanFileLoop(sgLoopName, netlocien,Path,FileRec,True) Then Exit;
          End;
        End;
      End //= FileRec.Att
      Else  //T21 - in case of compressed files
        If (FileRec.Attr <> Fadirectory) Then
        Begin
          If Not checkOrphanFileLoop(sgLoopName, netlocien,Path,FileRec,True) Then Exit;
        End;
    Until (FindNext(FileRec) <> 0) Or FProcessingPaused; ///searching
End;

(*
check if image file is the orphan on RAID
called by dirtree or btnRunProcessClick
*)

Function TfMagUtility.checkOrphanFile(sgLoopName, netlocien, Path: String; FileRec: TSearchRec): Boolean;
Var
  chkvar   : String;
  DUPE     : String;
  FileDTM  : String;
  filefulnm: String;
  Filesize : Integer;
  filesize0: String;
  Filextn  : String;
  Fname    : String;
  Ien      : Integer;
  IQ       : String;
  jbID     : String;
  Magien   : Integer;
  netlocphy: String;
  Node0    : String;
  NodeBig  : String;
  oldname  : String;
  ptrRAID  : String;
  raidID   : String;
  selRAID  : String;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.checkOrphanFile';
  Result:=True;
  ///showmessage(FileRec.Name+#13+ InttoStr(FileRec.Attr));//debug
  If (FileRec.Attr = Fadirectory) Then Exit; //T21
  ProblemIndex := 0;
  If Not AnsiContainsStr(FileRec.Name, '.') then
  Begin
      If (Copy(Path,length(Path),1)<>'\') then filefulnm := Path + '\' + FileRec.Name  //T20
      Else filefulnm := Path + FileRec.Name; ///full file pathname - T20 - fix \\filename
      filesize0 := Inttostr(FileRec.Size);
      FileDTM := Formatdatetime('mm/dd/yy  h:nn  am/pm', FileDateToDateTime(FileAge(filefulnm)));
      FLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, missing file extension, ' + filefulnm + ' ' + FileDTM + '. file size ' + filesize0);   //t21 - no file ext
      FErrorLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, missing file extension,' + filefulnm + ' ' + FileDTM + '. file size ' + filesize0);  //T20 - CR209
      errorcnt := errorcnt + 1; //orphan file count
      attnerrorcnt := attnerrorcnt +1 ; //t17 - attention error
      totalcnt := totalcnt + 1;
      lbltotalcnt.caption := Inttostr(totalcnt);
      ProblemIndex :=1 ; //no file ext
      Exit;
  End;
  If AnsiContainsStr(FileRec.Name, '.') then
  Try
    If FProcessingPaused And (Not boInRetryLoop) Then Exit; /// user does Cancel task
    lbltotalcnt.caption := Inttostr(totalcnt);
    totalcnt := totalcnt + 1;
    Fname := MagPiece(FileRec.Name, '.', 1);
    Fname := Uppercase(Fname);
    Filextn := MagPiece(FileRec.Name, '.', 2);
    Filextn := Uppercase(filextn);
    lblCurrentFile.caption := Fname + '.' + Filextn;
    If (Copy(Path,length(Path),1)<>'\') then filefulnm := Path + '\' + FileRec.Name  //T20
    Else filefulnm := Path + FileRec.Name; ///full file pathname - T20 - fix \\filename
    OrphanFilename:= filefulnm; //T21
    Filesize :=  FileRec.Size; //(FileRec.Size Div 1024) + 1;
    If Filesize<0 Then Filesize:=0;
    filesize0 := Inttostr(Filesize);
    ///SHOWMESSAGE(filefulnm);
    FileDTM := Formatdatetime('mm/dd/yy  h:nn  am/pm', FileDateToDateTime(FileAge(filefulnm)));
    ProblemIndex := 0;
    Application.Processmessages; ///refresh process status

    // (1) P98 SRS - chk file extension
    If Filextn = '' Then
    Begin
      FLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, missing file extension, ' + FileRec.Name + ' ' + FileDTM + '. file size ' + filesize0);   //t21
      FErrorLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, missing file extension,' + FileRec.Name + ' ' + FileDTM + '. file size ' + filesize0);  //T20 - CR209
      errorcnt := errorcnt + 1; //orphan file count
      attnerrorcnt := attnerrorcnt +1 ; //t17 - attention error
      ProblemIndex :=1 ;
    End
    Else
    Begin //validate file extn (type)
      chkvar := '$O(^MAG(2005.021,"B","' + Filextn + '",0))+0';
      Ien := MagUtilityBroker.FnmGetIEN(chkvar);
      If Ien = 0 Then
      Begin
        lblIEN.caption := 'not found'; //T20 - CodeCR 210
        FLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, "' + Filextn + '" is not an approved file extension, ' + filefulnm + ' ' + FileDTM + '. file size ' + filesize0 );  //T21
        FErrorLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, "' + Filextn + '" is not an approved file extension, ' + filefulnm + ' ' + FileDTM + '. file size ' + filesize0 );  //T21
        errorcnt := errorcnt + 1; //orphan file count
        attnerrorcnt := attnerrorcnt + 1; //t17 - attention cnt
        ProblemIndex :=1;
      End
      Else
      Begin ///approved file type ext
        //IR#T15-2   lblIEN.caption := '0';
        // (2) P98 SRS - Zero length file
        If FileRec.Size = 0 Then ///* Perform 0-size cleanup
        Begin
          chkvar := '$O(^MAG(2005,"F","' + Fname + '",0))+0'; //T21 - even show 0 size file  CR279
          Magien := MagUtilityBroker.FnmGetIEN(chkvar);
          If Magien <> 0 Then lblIEN.caption := Inttostr(Magien)
          Else lblIEN.caption := 'not found';
          lblCurrentFile.caption := filefulnm;
          DelOrphanFile(filefulnm, Fname, 0); //del 0 size from RAID
          // 0-size cleanup
          If (filextn <> 'TXT') Then DELandSET(netlocien, filefulnm, Fname, 0, 0); //update pointer in #2005
          errorcnt := errorcnt + 1;
          ProblemIndex := 2; //size 0 issue
        End // of size 0
        Else
          If (FileRec.Size > 0) Then
          Begin
            // (4) P98 SRS - check "F" cross-ref IMAGE file
            chkvar := '$O(^MAG(2005,"F","' + Fname + '",0))+0';
            Magien := MagUtilityBroker.FnmGetIEN(chkvar);
            If Magien <> 0 Then //found IEN from VistA
            Begin
              lblIEN.caption := Inttostr(Magien);
              lblCurrentFile.caption := filefulnm;
              chkvar := '$G(^MAG(2005,' + Inttostr(Magien) + ',0))_"^"';
              Node0 := MagUtilityBroker.FnmGetNode(chkvar); // image info
              IQ := MagPiece(Node0, '^', 11);
              DUPE := MagPiece(Node0, '^', 12);
              raidID := MagPiece(Node0, '^', 3); //full img file raid
              jbID := MagPiece(Node0, '^', 5); //full img file jb
              // (5) P98 SRS - IQ or DUPE in IMAGE file, log event
              If (IQ = '1') Or (DUPE = '1') Then  //T18 - BP Verfier found '0' or not at all ''
              Begin
                FLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, IQ or DUPE fields are set, ' + filefulnm + ' ' + FileDTM + '. file size ' + filesize0);   //T18
                FErrorLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, IQ or DUPE fields are set, ' + filefulnm + ' ' + FileDTM + '. file size ' + filesize0);   //T18
                errorcnt := errorcnt + 1;
                attnerrorcnt := attnerrorcnt + 1; //t17 - attention cnt
                ProblemIndex := 5; //QA issue
              End
              Else //no issue - Verifier QA / IQ DUPE
              Begin //decide the RAID ptr by file extn
                If Filextn = 'ABS' Then
                  raidID := MagPiece(Node0, '^', 4)
                Else
                Begin //find 2005 RAID ptr for scanning file
                  If Filextn = 'BIG' Then
                  Begin
                    chkvar := '$G(^MAG(2005,' + Inttostr(Magien) + ',"FBIG"))_"^"';
                    NodeBig := MagUtilityBroker.FnmGetNode(chkvar);
                    raidID := MagPiece(NodeBig, '^', 1);
                  End
                  Else
                    If Filextn = 'TXT' Then //all possible RAID ptr
                    Begin
                      chkvar := '$G(^MAG(2005,' + Inttostr(Magien) + ',"FBIG"))_"^"';
                      NodeBig := MagUtilityBroker.FnmGetNode(chkvar);
                      ptrRAID := '^' + raidID + '^' + MagPiece(Node0, '^', 4) + '^' + MagPiece(NodeBig, '^', 1) + '^'; //ful^abs^big
                      selRAID := '^' + netLocIen + '^';
                      If ANSIcontainsStr(ptrRAID, selRAID) Then
                        raidID := netLocIen //matched RAID ptr for TXT
                      Else
                      Begin //no RAID ptr matched selected RAID for TXT
                         //LOG ONLY*t14-
                         //DelOrphanFile(filefulnm, Fname, FileRec.Size );  //,0);T14 bug
                      End;
                    End; //='TXT'
                End;
                // (6) P98 SRS - Image/Audit file Missing
                If (Node0 = '^') Then
                Begin
                  FLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, there is no VistA #2005 IMAGE file entry for (IEN ' + Inttostr(Magien) + '), ' + filefulnm +
                    ' ' + FileDTM + '. file size ' + filesize0); //T21
                  FErrorLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, there is no VistA #2005 IMAGE file entry for (IEN ' + Inttostr(Magien) + '), ' + filefulnm +
                    ' ' + FileDTM + '. file size ' + filesize0 +' - (6) '+ProcName); //T21
                  If ProblemIndex=0 then  attnerrorcnt := 1 + attnerrorcnt;
                  ProblemIndex := 6;
                End;
                // (7) P98 SRS - if RAID Network Loc is set in file 2005
                If (raidID <> '') Then //(7) Yes - RAID ptr is set
                Begin ///raidID <>''
                  // (10) P98 SRS - if selected share is same as current RAID ptr of IMAGE file (netlocien '= raidID)
                  If (netlocien <> raidID) Then //(10) No match RAID ptr
                  Begin // Perform dupe check on RAID
                    // (11) P98 SRS - same File is recorded co-exist on other RAID - raidID
                    If (FileExistOther(Inttostr(Magien), filefulnm, Filextn, FileSize0) = False) Then
                    Begin
                      If ProblemIndex = 0 Then //no other file and no issue with other file
                        //  (12) P98 SRS - if .TXT no update pointer, but log
                        If (Filextn <> 'TXT') Then
                          UpdateLocPT(Inttostr(Magien), netLOCien, FileRec.Name)
                        Else
                          Begin
                          chkvar := '$G(^MAG(2005.2,' + raidID + ',0))';
                          Node0 := MagUtilityBroker.FnmGetNode(chkvar);
                          //FLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, '+ MAGPiece(Node0,'^',1)+'(ien ' + raidID + '), the share pointer referenced in IMAGE 2005 file,  does not match ' +CurrentRAIDname+'(ien ' + netlocien + ') the share pointer selected for review ' +
                          FLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, '+ MAGPiece(Node0,'^',1)+'(#' + raidID + '), the share pointer in the 2005 file,  does not match '+CurrentRAIDname+'(#' + netlocien + ') the share pointer selected for review ' + filefulnm + '.  ' + FileDTM + '. file size ' + //from RON's Errata_log_files
                                        filefulnm + '  ' + FileDTM + '. file size ' + Inttostr(Filesize) ); //T21 - reword
                          //FErrorLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, '+ MagPiece(Node0,'^',1)+ '(ien ' + raidID + '), the share pointer(ien '+raidID+') referenced in IMAGE 2005 file,  does not match ' + CurrentRAIDname+ '(ien ' + netlocien + ') the share pointer selected for review ' +
                          FErrorLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, '+ MAGPiece(Node0,'^',1)+'(#' + raidID + '), the share pointer in the 2005 file,  does not match '+CurrentRAIDname+'(#' + netlocien + ') the share pointer selected for review ' + filefulnm + '.  ' + FileDTM + '. file size ' +
                                            filefulnm + '  ' + FileDTM + '. file size ' + Inttostr(Filesize) + ' - (12) '+ProcName ); //T21
                          attnerrorcnt := attnerrorcnt + 1;  //T18 - no fix, review
                          ProblemIndex := 12;
                          End
                    End
                    Else
                    Begin //other files matched
                      // (14) P98 SRS - if same Physical Reference
                      chkvar := '$G(^MAG(2005.2,' + raidID + ',0))';
                      Node0 := MagUtilityBroker.FnmGetNode(chkvar);
                      chkvar :=  '$P($G(^MAG(2005.2,' + netlocien + ',0)),"^",2)';
                      netlocphy := MagUtilityBroker.FnmGetNode(chkvar); //physical reference
                      If (netlocphy <> MagPiece(Node0, '^', 2)) Then
                        DelOrphanFile(filefulnm, Fname, Filesize)
                      Else
                        Begin
                        FLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Found two or more RAID pointers pointing to the same physical reference, "' + netlocphy + '" - '+ MagPiece(Node0, '^', 1) + '(ien ' + raidID + ') ' + filefulnm + ' ' + FileDTM + '. file size ' + Inttostr(Filesize)); //T21
                        FErrorLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Found two or more RAID pointers pointing to the same physical reference, "' + netlocphy + '" - '+ MagPiece(Node0, '^', 1) + '(ien ' + raidID + ') vs. selected ' + CurrentRAIDname+ ' (ien ' + netlocien + ') '+ filefulnm + ' ' + FileDTM + '. file size ' + Inttostr(Filesize) + ' - (14) '+ProcName);//T21
                        If ProblemIndex=0 Then attnerrorcnt := attnerrorcnt + 1;
                        ProblemIndex := 14;  //cannot delete on same physical referenced share
                        End;
                    End;
                    errorcnt := errorcnt + 1; //(10) no match - orphan
                  End;
                  // file okay
                End //raidID <>'' (7) Yes
                Else //(7) No - RAID ptr was not set 2005
                Begin // *Perform JB check
                  errorcnt := errorcnt + 1;
                  If ProblemIndex=0 Then  //T20 - CodeCR0000214
                  // (8) P98 SRS - check if it is on Jukebox?
                  If (jbID = '') Then //Is file on JB (No)
                  Begin
                    //queueJB(inttostr(MagIEN)) //(8)No RAID & no JB ptr, log only, can't confirm RAID/JB files match
                    attnerrorcnt := attnerrorcnt + 1; //t17 - attention cnt
                    ProblemIndex := 8;
                    FLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, no RAID or JB pointer ' + filefulnm + ' ' + FileDTM + '. file size ' + filesize0 ); //T21
                    FErrorLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, no RAID or JB pointer ' + filefulnm + ' ' + FileDTM + '. file size ' + filesize0 +' - (8) ('+UnitName+'.'+ProcName+'.1)'); //T20 - CodCR000216
                  End
                  Else //Is file on JB (Yes)
                    If FileExistOther(Inttostr(Magien), filefulnm, Filextn, Filesize0) Then
                           UpdateLocPT(Inttostr(Magien), netLOCien, FileRec.Name) //same size as on JB
                    Else
                    Begin
                      If ProblemIndex = 0 then attnerrorcnt := attnerrorcnt + 1; //t17 - attention cnt
                      ProblemIndex :=8;
                    End;
                End; //(7) No - 2005 raidID =''
              End; //no Verifier QA issue - IQ DUPE (5)
            End //IF find IEN from VistA  (4)
            Else // no "F" x-ref for the filename  MagIEN = 0
              If (FileRec.Attr <> Fadirectory ) Then //check file only , no directory
              Begin
                chkvar := '$O(^MAG(2005.1,"F","' + Fname + '",0))+0'; //chk deleted&audited image file
                Magien := MagUtilityBroker.FnmGetIEN(chkvar);
                If Magien <> 0 Then
                Begin
                    DelOrphanFile(filefulnm, Fname, Filesize);  //delete IMAGE audit file from RAID
                End
                Else
                Begin
                    lblIEN.caption := 'not found'; //T20 - CodeCR 210
                    FLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, no IEN "F" x-ref for the filename ' + filefulnm + '. ' + FileDTM + '. file size ' + filesize0 );
                    FErrorLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, no IEN "F" x-ref for the filename ' + filefulnm + '. ' + FileDTM + '. file size ' + filesize0 + ','+' ('+UnitName+'.'+ProcName+'.24)');
                End;
                errorcnt := errorcnt + 1;
                If ProblemIndex = 0 then attnerrorcnt := attnerrorcnt + 1;//t17 - attention cnt
                ProblemIndex := 4;
                UpdateLabel(lblErrorCnt,errorcnt);
                lblCurrentFile.caption := filefulnm;
                oldname := Fname;
              End;
          End; //filesize <> 0
      End; //approved file type extn
    End; //valid file extn
    If RemoveOrphanFile then
        If errorcnt>0 then UpdateLabel(lblErrorCnt,inttostr(errorcnt)+ '    Fixed:'+ inttostr(errorcnt-attnerrorcnt)) //t17 - add user atthention error cnt
    Else UpdateLabel(lblErrorCnt,errorcnt);
    lblCurrentFile.caption := filefulnm;
    lbltotalcnt.caption := Inttostr(totalcnt);
  Except
    On e:Exception Do
    Begin
      Result:=False;
      FErrorLogger.Error(#9 + 'Orphan Files'+ #9 + #9 +'A network error occurred: the arguments were netlocien='+netlocien+ #9 + 'Path='+Path+' ('+UnitName+'.'+ProcName+'.3)');
    End;
  End;
End;

(*
(SUB) P98 SRS - 3.2.1.5
Delete the orphan file from RAID, update pt if needed
Log the event
Called by checkOrphanFile
*)

Procedure TfMagUtility.DelOrphanFile(fulfile, Fname: String; Filesize: Integer);
Var
  Magien: Integer;
  FileDTM, filextn, otherfulfile, output1, chkvar: String;
  OkToDel: Boolean;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.DelOrphanFile';
  FileDTM := Formatdatetime('mm/dd/yy  hh:mm  am/pm', FileDateToDateTime(FileAge(fulfile)));
  filextn := MagPiece(fulfile, '.', 2);
  filextn := Uppercase(filextn);
  If RemoveOrphanFile Then
  Begin //cleanup orphan file
    If (Filesize = 0) Then
      OKtoDEL := True ///delete it any way
    Else //file size>0
    Begin // ien lookup
      OkToDel := False;
      chkvar := '$O(^MAG(2005,"F","' + Fname + '",0))+0';
      Magien := MagUtilityBroker.FnmGetIEN(chkvar);
      If Magien = 0 Then
        OKtoDel := True ///no ien
      Else
      Begin
        output1 := MagUtilityBroker.MagFindfile(Inttostr(Magien), filextn); //rpc - MAGG IMG INFO
            //FLogger.Error(#9+'Orphan Files'+#9+#9+output1); //debug
        If MagPiece(output1, '^', 1) <> '1' Then //check #2005 if other share file(s) exist at all
        Begin
          FLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, orphan file can not be deleted because no archive copy exists for (IEN ' + Inttostr(Magien)+ ') ' + fulfile + ' ' + FileDTM + '. file size ' + Inttostr(Filesize));
          FErrorLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, orphan file can not be deleted because no archive copy exists for (IEN ' + Inttostr(Magien)+ ') ' + fulfile + ' ' + FileDTM + '. file size ' + Inttostr(Filesize)+' ('+UnitName+'.'+ProcName+'.25)');
          If ProblemIndex = 0 then attnerrorcnt := attnerrorcnt + 1; //t17 - attention cnt
          ProblemIndex := 99; //no archive copy exists
        End
        Else
        Begin // verify other file "Modified Date" same as
          If filextn = 'ABS' Then
            otherfulfile := MagPiece(output1, '^', 4) //ABS file
          Else
            If filextn = 'TXT' Then
            Begin
              otherfulfile := MagPiece(output1, '^', 3);
              otherfulfile := MagPiece(otherfulfile, '.', 1) + '.TXT';
            End
            Else
              If filextn = 'BIG' Then
              Begin
              If MagPiece(output1, '^', 19)<>'' Then otherfulfile := MagPiece(output1, '^', 19);
              otherfulfile := MagPiece(otherfulfile, '.', 1) + '.BIG';
              End
              Else
              otherfulfile := MagPiece(output1, '^', 3); //full file
          If (FileLastModified(fulfile) = FileLastModified(otherfulfile)) Then
            OKtoDel := True
          Else
          Begin
            FLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, image file with different "Date Modified" ' +
              fulfile + '@' + FileLastModified(fulfile) + ' vs. ' + otherfulfile + '@' + FileLastModified(otherfulfile) + ', ' + fulfile+ ' ' + FileDTM + '. file size ' + Inttostr(Filesize));
            FErrorLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, image file with different "Date Modified" ' +
              fulfile + '@' + FileLastModified(fulfile) + ' vs. ' + otherfulfile + '@' + FileLastModified(otherfulfile) + ', ' + fulfile+ ' ' + FileDTM + '. file size ' + Inttostr(Filesize) +' ('+UnitName+'.'+ProcName+'.26)');
          End;
        End;
      End;
    End; //of file size>0
    If OKtoDel Then OktoDel := DeleteFile(fulfile); ///*delete file
    If OktoDel Then
    Begin
      //lblIEN.caption := '*';  //T14-3  removed
      If Filesize = 0 Then
      Begin
        Flogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, deleted zero length file, ' + fulfile + ' ' + FileDTM + '. file size 0'); //T21
        //T21 replaced - FLogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found zero length file, ' + fulfile + '.  ' + FileDTM + '. file size 0'); //T19 CodeCR=IMAG0000233
        //T21 - FLogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, deleted file ' + fulfile + ' ' + FileDTM + '. file size 0');//T19 CodeCR=IMAG0000233
      End
      Else
        FLogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, deleted file ' + fulfile + ' ' + FileDTM + '. file size ' + Inttostr(Filesize));//T19 CodeCR=IMAG0000233
      OrphanFixed:= OrphanFixed +1;  //not use
      FErrorLogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, deleted file, ' + fulfile + ' ' + FileDTM + '. file size ' + Inttostr(Filesize)+'. '+' ('+UnitName+'.'+ProcName+'.28)');
    End //delete okay
    Else
    Begin
      FLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, Error - cannot delete the file, ' + fulfile + ' ' + FileDTM + '. file size ' + Inttostr(Filesize));  //T18
      FErrorLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, Error - cannot delete the file, DeleteFile() failed ' + fulfile + '  ' + FileDTM + '. file size ' + Inttostr(Filesize) + ','+' ('+UnitName+'.'+ProcName+'.29)');
      If ProblemIndex = 0 then attnerrorcnt := attnerrorcnt + 1; //t17 - user needed attention cnt
      ProblemIndex := 99; //delete file failed
    End;
  End //of cleanup orphan file
  Else //log only
    Begin
      FLogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found - Delete candidate, ' + fulfile + '.  ' + FileDTM + '. file size ' + Inttostr(Filesize) );
      FErrorLogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found - Delete candidate, ' + fulfile + '.  ' + FileDTM + '. file size ' + Inttostr(Filesize)+', ('+UnitName+'.'+ProcName+'.30)');
    End;
End;

// (13) (11) P98 SRS - If the pt is set to other location and orphan file found
(*
Updatet the RAID pointer at MAG(2005
either : Setptr = o  ;unset,clear ptr
Setptr = RAIDien  ;set network loc ien
*)

Procedure TfMagUtility.DELandSET(netlocien, Filename, Fname: String; Filesize, setptr: Integer);
Var
  chkvar, Node0, raidID, raidIDabs, filextn, FileDTM, NETien, OtherRAIDnm: String;
  Magien: Integer;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.DELandSET';
  If netlocien = '' Then Exit;
  If Filesize = 0 Then
  Begin
    FileDTM := ' - '; //been deleted for filesize=0
    setptr := 0; //size 0 for unset only
  End
  Else
    FileDTM := Formatdatetime('mm/dd/yyyy  hh:mm  am/pm', FileDateToDateTime(FileAge(Filename)));
  chkvar := '$O(^MAG(2005,"F","' + Fname + '",0))+0';
  Magien := MagUtilityBroker.FnmGetIEN(chkvar);
  If Magien <> 0 Then //found IEN from VistA
  Begin
    chkvar := '$G(^MAG(2005,' + Inttostr(Magien) + ',0))_"^"';
    Node0 := MagUtilityBroker.FnmGetNode(chkvar); // image info
    raidID := MagPiece(Node0, '^', 3);  //full
    raidIDabs := MagPiece(Node0,'^',4); //ABS
    filextn := MagPiece(Filename, '.', 2);
    filextn := Uppercase(filextn);
    //(3) P98 SRS - if update the share loc pt in #2005
    If (raidID = netlocien) or ((filextn='ABS') and (raidIDabs = netlocien)) Then   ///point to current selected RAID
      ///point to current RAID but file size is 0, update ptr to nil
      If (setptr = 0) And (Filesize = 0) Then //unset P3 or P4 or BIG raid (3) P98 SRS
      Begin
        If MagUtilityBroker.unsetNETloc(Fname, filextn, Magien) Then
        Begin
          //showmessage('"'+FileDTM+'"'); //T21 - debug
          If (FileDTM = ' - ') Then FileDTM := ''; //T21 - since file size 0 been deleted not file DTM info
          FLogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, cleared or updated 2005 file pointer '+ CurrentRAIDname + ' (ien '+ Netlocien + ') for ' + Filename + '  ' + FileDTM + '. file size ' + Inttostr(Filesize) );  //T19 - CodeCR 209 been deleted
          FErrorLogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Orphan found file, cleared or updated 2005 file pointer '+ CurrentRAIDname +' (ien '+ Netlocien + ') for ' + Filename + '  ' + FileDTM + '. file size ' + Inttostr(Filesize)+' ('+UnitName+'.'+ProcName+'.1)');
        End
        Else
        Begin
          FLogger.Error(#9 + 'Orphan Files, error clearing or updating 2005 file pointer with ' + CurrentRAIDname + '(ien '+ Netlocien + ') '+ Filename + '.  ' + FileDTM + '. file size ' + Inttostr(Filesize));   //T21
          FErrorLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file , error clearing or updating 2005 file pointer with '+ CurrentRAIDname + '(ien '+ Netlocien + ') '+ Filename + '.  ' + FileDTM + '. file size ' + Inttostr(Filesize) +' ('+UnitName+'.'+ProcName+'.2)');
          If ProblemIndex = 0 then attnerrorcnt := attnerrorcnt + 1; //t17 - user needed attention cnt
          ProblemIndex := 999 ; //VistA pointer update failed
        End
      End
      Else ///Dupe on RAID check
        /// (11) P98 SRS - SUB1
        If (raidID = '') And (setptr <> 0) Then //find the file ptr not set, update it
        Begin
          If AnsiEndsStr('.ABS', Fname) Then
            NETien := netlocien + '@'
          Else
            NETien := netlocien + '*';
          If MagUtilityBroker.setNETloc(Inttostr(Magien), NETien) Then
          Begin
            FLogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, reset 2005 file pointer ' +CurrentRAIDname +'(ien '+Netlocien+') ' + Filename + ' ' + FileDTM + '. file size ' + Inttostr(Filesize)); //T21
            FErrorLogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found , reset 2005 file pointer ' + CurrentRAIDname + '(ien '+ Netlocien + ') '+ Filename + ' ' + FileDTM + '. file size ' + Inttostr(Filesize)+' ('+UnitName+'.'+ProcName+'.3)');
          End
        End
        Else
          If (raidID <> netlocien) Then
          Begin
            chkvar := '$G(^MAG(2005.2,' + raidID + ',0))_"^"';
            OtherRAIDnm := MagUtilityBroker.FnmGetNode(chkvar); // T21 - network location info
            OtherRAIDnm := MagPiece(OtherRAIDnm,'^',1);
            FLogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, file deleted since the same file was found on other RAID '+ OtherRAIDnm + '(ien '+ raidID +') ' + Filename + ' ' + FileDTM + '. file size ' + Inttostr(Filesize)); //T21
            FErrorLogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, file deleted since the same file was found on other RAID '+ OtherRAIDnm + '(ien '+ raidID +') ' + Filename + ' ' + FileDTM + '. file size ' + Inttostr(Filesize)+' ('+UnitName+'.'+ProcName+'.4)');
          End;
  End; //MagIEN <> 0
  If (Magien = 0) And (Filesize > 0) Then
  Begin
    FLogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found , file deleted - cannot find associated 2005 file IEN for '+ Filename + ' ' + FileDTM + '. file size ' + Inttostr(Filesize)); //T19 CodeCR=IMAG0000233
    FErrorLogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found , file deleted - cannot find associated("F" xref) 2005 file IEN for '+ Filename + ' ' + FileDTM + '. file size ' + Inttostr(Filesize)+' ('+UnitName+'.'+ProcName+'.5)');
    //T20 - if deleted not needing review
  End;
End;

(*
  check if the file exist on othe RAID/Jukebox location and in same size
*)

Function TfMagUtility.FileExistOther(MagIN, FileFulnm, Filetype, Filesize: String): Boolean;
Var
  output1, fulfile, magtype, jboffline, otherfilesize: String;
  othfilesize: Integer;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.FileExistOther';
  Result := False;
  ProblemIndex := 0;
  output1 := MagUtilityBroker.MagFindfile(MagIN, Filetype); //rpc - MAGG IMG INFO
  //OUTPUT1 = $P(1^2^3)     IEN^Image FullPath and name^Abstract FullPath and Name
  //          $p(10)        return the ABSTYPE  'M' magnetic 'W' worm  'O' offline
  //          $p(11)        is  'A' accessable   'O' offline
  ////showmessage(output1);
  If MagPiece(output1, '^', 1) = '1' Then
  Begin
    fulfile := MagPiece(output1, '^', 3); //full
    Filetype := Uppercase(Filetype);
    If Filetype = 'ABS' Then
      fulfile := MagPiece(output1, '^', 4)
    Else
      If Filetype = 'BIG' Then
      Begin
           If MagPiece(output1, '^', 19)<>'' Then fulfile := MagPiece(output1, '^', 19); //T20 -BIG saved loc
      End;
    fulfile := MagPiece(fulfile, '.', 1) + '.' + Filetype; //2005 reference file
    If fulfile = '' Then ProblemIndex := 11;
    magtype := MagPiece(output1, '^', 11);
    jboffline := MagPiece(output1, '^', 12);
    // (9) P98 SRS - JB File size
    // (13) P98 SRS - File sizes & Modify Date the same on both share locations
    If (magtype = 'O') Or (jboffline = 'O') Then
    Begin
      FLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Cannot verify the off-line image file to process (#13) ' + fulfile ); //T19 CodeCR=IMAG0000233
      FErrorLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Cannot verify the off-line image file to process (#13) ' + fulfile );
    End
    Else //chk file size
      If ProblemIndex = 0 Then //no problem so far
      Begin
        otherfilesize := Get_File_Size2(fulfile);
        othfilesize := Strtoint(otherfilesize); //othfilesize := othfilesize Div 1024 + 1;
        If othfilesize > 0 Then
        Begin
          otherfilesize := Inttostr(othfilesize);
          If (otherfilesize = Filesize) Then
          Begin
             If (FileLastModified(fulfile) = FileLastModified(FileFulnm)) Then Result := True ///other RAID file co-exist
             Else
                Begin
                FLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, image file with different "Date Modified" ' +
                              FileFulnm + '@' + FileLastModified(FileFulnm) + ' vs. ' + fulfile + '@' + FileLastModified(fulfile));
                FErrorLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, image file with different "Date Modified" ' +
                              FileFulnm + '@' + FileLastModified(FileFulnm) + ' vs. ' + fulfile + '@' + FileLastModified(fulfile)+' ('+UnitName+'.'+ProcName+'.6)'); //T20 - add dup log for ERRORLOG
                If ProblemIndex = 0 Then attnerrorcnt := attnerrorcnt + 1; //t17 - user needed attention cnt
                ProblemIndex:= 13; //not same Modify DT
                end
          End
          Else ///(13) sizes mismatch
          Begin
            FLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, file size variance between the two Network Locations ' + '(' + Filesize + ') vs (' + otherfilesize + ') ' + FileFulnm + ' and '  + fulfile );
            FErrorLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, file size variance between the two Network Locations ' + '(' + Filesize + ') vs (' + otherfilesize + ') ' + FileFulnm + ' and '  + fulfile +' ('+UnitName+'.'+ProcName+'.7)');
            If ProblemIndex = 0 Then attnerrorcnt := attnerrorcnt + 1; //t17 - user needed attention cnt
            ProblemIndex := 13; //not same size
          End;
        End
        Else
          If NOT RemoveOrphanFile Then  //T20 - CR207 213, report only
          Begin
             If otherfilesize='-1' then //other file not found
             Begin
                FLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, pointer needs to be updated. IMAGE 2005 (IEN '+MagIN+') referenced file does not exist, but found '+FileFulnm+' '+
                              Formatdatetime('mm/dd/yyyy  hh:mm  am/pm', FileDateToDateTime(FileAge(FileFulnm)))+'. file size '+ Filesize );
                FErrorLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, pointer needs to be updated. IMAGE 2005 (IEN '+MagIN+') referenced file does not exist, but found '+FileFulnm+' '+
                                   Formatdatetime('mm/dd/yyyy  hh:mm  am/pm', FileDateToDateTime(FileAge(FileFulnm)))+'. file size '+ Filesize +' ('+UnitName+'.'+ProcName+'.8) - #11');
                If ProblemIndex = 0 Then attnerrorcnt := attnerrorcnt + 1;
                ProblemIndex := 13;
             End
             Else  // file size=0
             Begin
                otherfilesize := '0 or no-exist';
                FLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, 2005 file (IEN '+MagIN+') referenced '+ fulfile + ' file size is 0, but found '+FileFulnm+' '+ Formatdatetime('mm/dd/yyyy  hh:mm  am/pm', FileDateToDateTime(FileAge(FileFulnm)))+'. file size '+ Filesize);
                FErrorLogger.Error(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, 2005 file (IEN '+MagIN+') referenced '+ fulfile + ' file size is 0, but found '+FileFulnm+' '+ Formatdatetime('mm/dd/yyyy  hh:mm  am/pm', FileDateToDateTime(FileAge(FileFulnm)))+'. file size '+ Filesize+', other: ' + fulfile+' ('+UnitName+'.'+ProcName+'.62) - (13)');
                If ProblemIndex = 0 Then attnerrorcnt := attnerrorcnt + 1;
                ProblemIndex := 13;
             End;
          End; //report only
      End;
  End; //RPC - MagFindfile =1
End;

(*
  Return file size in byte
*)

Function TfMagUtility.Get_File_Size2(sFileToExamine: String): String;
Var
  SearchRec: TSearchRec;
  sgPath: String;
  inRetval, I1: Integer;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.Get_File_Size2';
  sgPath := ExpandFileName(sFileToExamine);
  Try
    inRetval := FindFirst(ExpandFileName(sFileToExamine), FaAnyFile, SearchRec);
    If inRetval = 0 Then
      I1 := SearchRec.Size
    Else
      I1 := -1;
  Finally
    SysUtils.FindClose(SearchRec);
  End;
  Result := Inttostr(I1);
End;

(*
  Menu - MagUtility | file | Exit
*)

Procedure TfMagUtility.mnuFileExitClick(Sender: Tobject);
Var
  warnmessage: String;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.mnuFileExitClick';
  If OrphanFilesRunning Or HEC1010Running Or OfflineJBImgRunning Or NetworkLocClnRunning
    Or FUT49Running Then
  Begin
    If OrphanFilesRunning Then
      warnmessage := 'Orphan Files'
    Else
      If OfflineJBImgRunning Then
        warnmessage := 'Offline Jukebox Images'
      Else
        If NetworkLocClnRunning Then
          warnmessage := 'Network Location Cleanup'
        Else
          If FUT49Running Then warnmessage := 'Storage Copy';
    Showmessage('There is a MagUtility process - ' + warnmessage + ' running. Wait until the process is complete, or cancel the process before exiting MagUtility.');
    Exit;
  End;
  FLogger.Info(#9 + 'Exit' + #9 + #9 + 'Closed MagUtility menu '+ #9 +'User:' + MagUtilityBroker.UserFullName);
  FErrorLogger.Info(#9 + 'Exit' + #9 + #9 + 'Closed MagUtility menu '+ #9 +'User:' + MagUtilityBroker.UserFullName);
  TLogger.FreeInstances;
  Application.Terminate;
End;

// Called by checkOrphanFile, to update #2005 ntw loc pt

Procedure TfMagUtility.UpdateLocPT(Magien, LOCien, Filename: String);
Var
  Filextn, SetLocien, FileDTM, FileSize: String;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.UpdateLocPT';
  Filextn := MagPiece(Filename, '.', 2);
  Filextn := Uppercase(Filextn);
  FileDTM := Formatdatetime('mm/dd/yyyy  hh:mm  am/pm', FileDateToDateTime(FileAge(OrphanFileName)));
  FileSize := Get_File_Size2(OrphanFileName);
  SetLOCien := '';
  If Filextn = 'TXT' Then //no ptr set needed for .txt
  Begin //T20 - CodeCR0000215
    If RemoveOrphanFile Then
    Begin
        FLogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Orphan TXT file found and removed "' + Filename + '"');
        FErrorLogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Orphan TXT file found and removed "' + Filename + '"');
    End
    Else
      Begin
         FLogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Orphan TXT file found "' + Filename + '"');
         FErrorLogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Orphan TXT file found "' + Filename + '"'+' ('+UnitName+'.'+ProcName+'.7)');
      End;
  End
  Else
    Begin
      If Filextn = 'ABS' Then SetLOCien := LOCien + '@'
      Else
      If Filextn = 'BIG' Then SetLOCien := LOCien + 'b'
      Else
        SetLOCien := LOCien + 'r'; //IR#T15-1 raid full file Set, e.g. TGA, TIF
    End;
  If SetLOCien <> '' Then
  Begin
    If RemoveOrphanFile Then
    Begin
      If MagUtilityBroker.setNetLoc(Magien, SetLOCien) then OrphanFixed := OrphanFixed +1;
      FLogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, reset ^MAG(2005,' + Magien + ',0) pointer with selected RAID reference '+ CurrentRAIDname + '(ien ' + LOCien+') ' + Filename + ' ' + FileDTM + '. file size ' + FileSize); //T21 Filename _ TDM _ filesize
      FErrorLogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found, reset MAG(2005,' + Magien + ',0) pointer with selected RAID reference '+ CurrentRAIDname +' (ien ' + LOCien+') ' + Filename + ' ' + FileDTM + '. file size ' + FileSize+' ('+UnitName+'.'+ProcName+'.8)');
    End
    Else //just report only
    Begin
      FLogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found ^MAG(2005,' + Magien + ',0) pointer needs to be updated ' + Filename + ' ' + FileDTM + '. file size ' + FileSize); //T21
      FErrorLogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Orphan file found ^MAG(2005,' + Magien + ',0) pointer needs to be updated ' + Filename + ' ' + FileDTM + '. file size ' + FileSize +' ('+UnitName+'.'+ProcName+'.9)');
    End;
  End;
End;

(*
Implementation of the M piece function
@param str The input string
@param del The delimiter
@param Piece the piece to return
@return The piece from the string
*)

Function TfMagUtility.MagPiece(Str, Del: String; Piece: Integer): String;
Var
  i, k: Integer;
  s: String;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.MagPiece';
  i := Pos(Del, Str);
  If (i = 0) And (Piece = 1) Then
  Begin
    MagPiece := Str;
    Exit;
  End;
  For k := 1 To Piece Do
  Begin
    i := Pos(Del, Str);
    If (i = 0) Then i := Length(Str) + 1;
    s := Copy(Str, 1, i - 1);
    Str := Copy(Str, i + 1, Length(Str));
  End;
  MagPiece := s;
End;

(*
@return The current application path
*)

Function TfMagUtility.GetAppPath(): String;
Var
  ProcName: String;
Begin
  ProcName:='TfMagUtility.GetAppPath';
  Result := Copy(ExtractFilePath(Application.ExeName), 1, Length(ExtractFilePath(Application.ExeName)) - 1);
End;

Function TfMagUtility.FMEXTDT(Fmdt: String): String;
Var
  Yy, MM, DD: Integer;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.FMEXTDT';
  Yy := Strtoint(Fmdt) Div 10000;
  MM := Strtoint(Fmdt) Mod 10000;
  DD := MM Mod 100;
  MM := MM Div 100;
  Result := Inttostr(MM) + '/' + Inttostr(DD) + '/' + Inttostr(Yy);
End;

(*
  menu #2 | Track Offline JB images - select the JB platter report text file
*)

Procedure TfMagUtility.OfflineJBreportChange(Sender: Tobject);
Var
  //FileToFind: string;
  OpenDialog: TOpenDialog;
  i, total: Integer;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.OfflineJBreportChange';
  btnImgOflClrlst.Enabled := False;
  OpenDialog := TOpenDialog.Create(Self);
  With OpenDialog Do
  Begin
    Title := 'Select Platter Report'; //(t17 - replaced title 'Open')
    InitialDir := 'c:\';
    DefaultExt := '*.TXT';
    Filter := 'Text files (*.txt)|*.txt';
    FilterIndex := 0; { start the dialog showing all files }
    Options := [ofFileMustExist, ofPathMustExist, ofEnableSizing, ofAllowMultiSelect];
    If Not Execute Then
      Exit
    Else
    Begin
      For i := 0 To openDialog.Files.Count - 1 Do
      Begin
        Filename := openDialog.Files[i];
        If Pos('TXT', Uppercase(Filename)) <> 0 Then
        Begin
          With OfflineJBreport Do
          Begin
            If (Filename <> '') And (Lines.Indexof(Filename) < 0) Then Lines.Add(Filename);
          End;
        End;
      End; //for i:=
    End; //else
    //If OfflineJbReport.Text <> '' Then btnImgOflClrlst.Enabled := True;
    If OfflineJbReport.Lines.Count>0  Then btnImgOflClrlst.Enabled := True;//rlm 20100104
  End; //end - with OpenDialog
  OpenDialog.Free;
  total := OfflineJBreport.Lines.Count;//rlm 20100104
  If total > 0 Then
  Begin
    offlineJBmkTotal.caption := '(total ' + Inttostr(total) + ')';
    btnImgOFLclrLst.Enabled := True;
    //OfflineJbReport.Text := OfflineJbReport.Items[0]; //rlm 20100104
    btnImgOflClrlst.Enabled := True;
    /// t17 - 508 removed "If ..." since .TXT read only//If OfflineJbReport.Text <> '' Then
    BtnRunJBOffline.Enabled := True;
  End;
  //rlm 20100104
  If OfflineJBreport.Lines.Count>0 Then
  Begin
    OfflineJBreport.SelStart:=0;
    OfflineJBreport.SelLength := 0;
  End;
End;

(*
menu #2 | Track Offline JB images | Back Online
User inserted the JB platter and now mark image from offline to online JB file
*)

Procedure TfMagUtility.BtnBackOnlineJBClick(Sender: Tobject);
Var
  OflJB: String;
  sel: Integer;
  ProcName: String;
Begin
  ProcName:='BtnBackOnlineJBClick';
  If (cbxOfflineJBlist.Text = '') And (cbxOfflineJBlist.ItemIndex < 0) Then
  Begin
    Showmessage('Select a platter first');
    Exit;
  End;
  If (cbxOfflineJBlist.ItemIndex < 0) Then
    sel := 0
  Else
    sel := cbxOfflineJBlist.ItemIndex;
  With cbxofflineJBlist Do
  Begin
    OflJB := Items[sel];
    If (OflJB = '') Then Exit;
    If (MagPiece(Items[sel], '(', 1) = 'done') Then Exit;
    FLogger.Info(#9 + 'Offline Jukebox Images' + #9 + 'Marked platter: #' + oflJB + ' images online' +#9+' by user:' + MagUtilityBroker.UserFullName); //t17 - LOG
    FErrorLogger.Info(#9 + 'Offline Jukebox Images' + #9 + 'Marked platter: #' + oflJB + ' images online' +#9+' by user:' + MagUtilityBroker.UserFullName);
    If MagUtilityBroker.BKOnlJB(OflJB) Then
    Begin
      Items.Delete(sel);
      //T18 - FLogger.Info(#9 + 'Offline Jukebox Images' + #9 + 'Completed marking Jukebox images online processing'); //t17 - LOG, info log #4
      //T18 -FErrorLogger.Info(#9 + 'Offline Jukebox Images' + #9 + 'Completed marking Jukebox images online processing'); //t17  - LOG , info log #4
      FLogger.Info(#9 + 'Offline Jukebox Images' + #9 + 'All requested images have been marked online.'); //T18  - comments (pg 8/13 #5)
      FErrorLogger.Info(#9 + 'Offline Jukebox Images' + #9 + 'All requested images have been marked online.'); //T18
      //T18 - Showmessage('Successfully marked all images on selected platter(#' + oflJB + ') online.'); //t17 - changed msg 09/21/09
      Showmessage('All requested images have been marked online. See info log for details.'); //T18 - changed msg
      Application.Processmessages;
    End;
    onlineJBmkTotal.caption := '';
    If Items.Count > 0 Then
    Begin
      cbxOfflineJBlist.Text := cbxOfflineJBlist.Items[0];
      cbxOfflineJBList.ItemIndex := 0;  //T20 - default 1st one
      onlineJBmkTotal.caption := '(total ' + Inttostr(Items.Count) + ')';
    End
    Else
      cbxOfflineJBlist.Text := ''
  End;
  cbxOfflineJBlist.Refresh;
  BtnBackOnlineJB.Enabled := False;
  If cbxOfflineJBList.Items.Count > 0 Then BtnBackOnlineJB.Enabled := True;
End;

(*
  Menu #2 | Track Offline JB images | Take Offline
*)

Procedure TfMagUtility.BtnRunJBOfflineClick(Sender: Tobject);
Var
  atidx, i, OflCnt, pltcnt, oflerror: Integer;
  TotalOflCnt: Int64;
  JBrpt, TXTLine, Magplat, TotalPlt, BadPlat, dupemsg, dupeplatter,Chkvar: String;
  TxtFile: Textfile;
  Shares: TStrings;
  ProcName, iname: String;
  isFileListing   : Boolean;
Begin
  TotalOflCnt := 0;
  dupeplatter:= ''; //T21- show duplicate platter(s)
  BadPlat:= '';
  ProcName:='TfMagUtility.BtnRunJBOfflineClick';
  If (offlineJBreport.Lines.Count < 1) Then  //t17  //rlm 20100104
  Begin
    Showmessage('Must select a DEX platter report first.');
    Exit;
  End;
  Shares := Tstringlist.Create();
  OfflineJBImgRunning := True;
  btnImgOflClrLst.Enabled := False;
  oflerror := 0;
  pltcnt := 0;
  TotalPlt := MagPiece(MagPiece(offlineJBmkTotal.caption, ' ', 2), ')', 1);
  For atidx := 0 To offlineJBreport.Lines.Count - 1 Do //rlm 20100104
  Begin
    isFileListing := False;
    MagPlat := '';
    iname   := '';
    JBrpt := OfflineJBreport.Lines[atidx];//rlm 20100104
    //if JBrpt is Binary file??
      AssignFile(TxtFile, JBrpt);
      Reset(TxtFile);
      dupemsg:= '0'; //T18 - dupe platter flag
      OflCnt:= 0; //T21- each platter, image files count
      While (Not Eof(TxtFile)) Do //reading JB platter report
      Begin
        ReadLn(TxtFile, TXTLine);
        If MagPiece(TxtLine, '"', 1) = 'Media ' Then
        Begin
          Magplat := MagPiece(TxtLine, '"', 2);
          //t21- FLogger.Info(#9 + 'Offline Jukebox Images' + #9 + 'Marked platter: #' + Magplat + ' offline'+ #9+' by user:' + MagUtilityBroker.UserFullName + ' - ' +JBrpt); //T18 - add by user
          //t21- FErrorLogger.Info(#9 + 'Offline Jukebox Images' + #9 + 'Marked platter: #' + Magplat + ' offline' +#9+' by user:' + MagUtilityBroker.UserFullName + ' - ' + JBrpt);  //T18 - add by user
          //chk dupe platter
          Begin
              chkvar := '$D(^MAGQUEUE(2006.033,"C","'+MagPlat+'"))'; //T18
              dupemsg := MagUtilityBroker.FnmGetNode(chkvar);  //t18 - See if platter already exist
              If dupemsg <> '0' Then   //T21 - already offline JB, record it dupe
              Begin
                 if dupeplatter='' then dupeplatter := #13+ #13+ 'Note: One or more of requested platters have been previously marked offline. Total number of platters does not count duplicate requests.'+#13+#9+'Dupe platter(s): ';  //t18 -Walt replaced
                 dupeplatter:= dupeplatter + MagPlat + '. '; //T21 - show dupe platter
                 oflerror:=oflerror+1;
              End;
          End;
        End
        Else
          If (TxtLine <> '') And (Magplat <> '') And (dupemsg = '0') Then   //T21 -skip dupe
          Begin
            If ANSIcontainsStr(TXTLine,'File Listing  (') then isFileListing := True  //T18 - skip top part summary report
            Else
            If isFileListing then
               If MagUtilityBroker.JBOffLn(TXTLine, MagPlat) Then
               Begin
                 OflCnt := OflCnt + 1;
                 TotalOflCnt:=TotalOflCnt+1;  //T21 - total image files processed on platter(s)
                 i := Pos('\', TXTLine);
                 iname:= Copy(TXTLine, i + 1, Length(TXTline)); //t17 - image file name
               //T18 - Agenda #3 FLogger.Info(#9 + 'Offline Jukebox Images' + #9 + #9+ 'Marked `'+iname+ '` Offline'); //t17 - LOG review - info log #2
               End;
          End;
      End; //while not End of File
    If MagPlat = '' Then
    Begin
      //T21 - Showmessage('Selected "' + JBrpt + '" is not in a correct platter report format and will be skipped'); //t17
      If (BadPlat='') then  BadPlat:=#13+#9+'Incorrect platter report format: ';
      BadPlat:= BadPlat+JBrpt+'. ';
      FLogger.Error(#9 + 'Offline Jukebox Images' + #9 + 'Selected "' + JBrpt + '" is not in a correct platter report format and will be skipped.');
      FErrorLogger.Error(#9 + 'Offline Jukebox Images' + #9 + 'Selected "' + JBrpt + '" is not in a correct platter report format and will be skipped.'+' ('+UnitName+'.'+ProcName+'.32)');//T17 - add error log too
      oflerror := oflerror + 1;
    End
    Else
    Begin  // w/ platter name
         If (dupemsg='0') then
         Begin
             Pltcnt := Pltcnt + 1;
             FLogger.Info(#9 + 'Offline Jukebox Images' + #9 + 'Marked platter: #' + Magplat + ' offline'+ #9+'by user:' + MagUtilityBroker.UserFullName + ' - ' +JBrpt); //T18 - add by user
             FErrorLogger.Info(#9 + 'Offline Jukebox Images' + #9 + 'Marked platter: #' + Magplat + ' offline ('+IntToStr(OflCnt)+')'+#9+'by user:' + MagUtilityBroker.UserFullName + ' - ' + JBrpt);  //T18 - add by user
         End
         Else
         Begin  //dupe no process
            FLogger.Error(#9 + 'Offline Jukebox Images' + #9 + 'Selected "' + JBrpt +'" has been previously marked offline, platter: #' + Magplat + ' will be skipped'+ #9 + ' User:' + MagUtilityBroker.UserFullName);
            FErrorLogger.Error(#9 + 'Offline Jukebox Images' + #9 + 'Selected "' + JBrpt +'" has been previously marked offline, platter: #' + Magplat + ' will be skipped'+ #9 + ' User:' + MagUtilityBroker.UserFullName+' ('+UnitName+'.'+ProcName+'.33)');  //T21 - add dupe  by user
         End;
    End;
  End;
  offlineJBreport.Clear;
  OfflineJBImgRunning := False;
  If (TotaloflCnt > 0) And MagUtilityBroker.LstOflJB(Shares) Then //refresh - off-line JB list
  Begin
    cbxOfflineJBlist.Items.Clear;
    cbxOfflineJBlist.Items := Shares;
    cbxOfflineJBList.Text := Shares[0];
    FLogger.Info(#9 + 'Offline Jukebox Images' + #9 + 'Marked image files offline from ' + IntToStr(pltcnt)+ ' jukebox platter(s)'); //T20 - CodeCR000225
    FErrorLogger.Info(#9 + 'Offline Jukebox Images' + #9 + 'Marked total '+ IntToStr(TotaloflCnt)+ ' files offline from '+ IntToStr(pltcnt)+ ' jukebox platter(s)'); //T18
    If oflerror > 0 then Showmessage('Not all requested images have been marked offline.  One or more platter reports could not be read.  See info log for details.'+ BadPlat + dupeplatter)  //T20 - CodeCR000225
    Else Showmessage('All requested images have been marked offline. See info log for details.'); //T18 - changed msg (agenda#3 Jan25, 2010)
  End;
  If Totaloflcnt = 0 Then
  Begin
    FLogger.Error(#9 + 'Offline Jukebox Images' + #9 + 'Task has completed but there were no image files to process. '); //T18 - comments (pg 12/13 #2) 1/20/10
    FErrorLogger.Error(#9 + 'Offline Jukebox Images' + #9 + 'Task has completed but there were no image files to process. ');
    Showmessage('Not all requested images have been marked offline. See log file for details.'+BadPlat+ dupeplatter);
  End;
  offlineJBmkTotal.caption := '';
  onlineJBmkTotal.caption := '';
  BtnRunJBOffline.Enabled := False;
  If cbxOfflineJBlist.Items.Count > 0 Then
  Begin
    onlineJBmkTotal.caption := '(total ' + Inttostr(cbxOfflineJBlist.Items.Count) + ')';
    cbxOfflineJBlist.Text := cbxOfflineJBlist.Items[0];
    cbxOfflineJBList.ItemIndex := 0; //T20 - default 1st one listed
    BtnBackOnlineJB.Enabled := True;
  End;
  //T18 - FLogger.Info(#9 + 'Offline Jukebox Images' + #9 +'Completed Marking Jukebox images');
{* T21 - removed - replaced by Mark xxx files offline on yyyy platter report(s)....
  If oflerror = 0 then
  Begin
    FLogger.Info(#9 + 'Offline Jukebox Images' + #9 + 'All requested images have been marked offline.'); //T18  - comments (pg 8/13 #5)
    FErrorLogger.Info(#9 + 'Offline Jukebox Images' + #9 + 'All requested images have been marked offline.'); //T18
  End
  Else
  Begin   //T18 - Agenda #3 -3
    FLogger.Info(#9 + 'Offline Jukebox Images' + #9 + 'Not all requested images have been marked offline.  One or more platter reports could not be read.'); //T18  - comments (pg 8/13 #5)
    FErrorLogger.Info(#9 + 'Offline Jukebox Images' + #9 + 'Not all requested images have been marked offline.  One or more platter reports could not be read. '); //T18
  End; //oflerror
  } //T21
End;

// Listing menu#4 Network locations (offline)

Function TfMagUtility.NetClnListView(): Boolean;
Var
  chkvar   : String;
  i        : Integer;
  icnt     : Integer;
  Ien      : String;
  iplace   : String;
  Listitem : TListItem;
  NetName  : String;
  NewColumn: TListColumn;
  OnLine   : String;
  route    : String;
  share    : String;
  Shares   : TStrings;
  Site     : String;
  Stype    : String;
  xline    : String;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.NetClnListView';
  Result:=False;
  icnt:=1;
  Try
    NetClnChkListView.Clear;
    NetClnChkListView.Columns.CleanupInstance;
    If NetClnChkListView.Columns.Count < 2 Then
    Begin
      //If (btnNtLocDelRun.Enabled = False) Then Exit;
      For i := 0 To 5 Do //Net Loc clean-up cloumn header
      Begin
        Newcolumn := NetClnChkListview.Columns.Add;
        If i = 0 Then
          Newcolumn.caption := 'Network-location share'
        Else
          If i = 1 Then
            Newcolumn.caption := 'Name  '
          Else
            If i = 2 Then
              Newcolumn.caption := 'IEN#'
            Else
              If i = 3 Then
                Newcolumn.caption := 'Place'
              Else
                If i = 4 Then
                  Newcolumn.caption := 'Site'
                Else
                  If i = 5 Then Newcolumn.caption := 'Storage Type';
            //NetClnChkListview.Columns[I].AutoSize := true;
        //NetClnChkListView.Columns[i].Width := ColumnHeaderWidth;
        NetClnChkListView.Columns[i].Width := 20;
      End;
    End;
    MagUtilityBroker.GetNetworkLocations(Shares);
    icnt := 1;
    For i := 0 To Shares.Count - 1 Do
    Begin
      xline := Shares[i];
      share := MagPiece(xline, '^', 2); //RPC(shares)- listing sorted by physical ref.
      xline := Uppercase(xline);
      Ien := MagPiece(xline, '^', 1);
      Stype := MagPiece(xline, '^', 3);
      Stype := MagPiece(Stype, '-', 1); //storage type
      OnLine := MagPiece(xline, '^', 4);
      Site := MagPiece(xline, '^', 9);
      iplace := MagPiece(xline, '^', 10);
      NetName := MagPiece(xline, '^', 11);
      If (Stype = 'MAG') Or (Stype = 'WORM') Then ///now RAID & JB only
        If (iplace = '') Or (iplace = PLACE) Then /// share is the same PLACE/SITE as the user's
        Begin
          chkvar := '$P($G(^MAG(2005.2,' + Ien + ',0)),U,9)_" "';
          route := MagUtilityBroker.FnmGetNode(chkvar);
          If (route = ' ') Or (route = '0 ') Then /// not the router loc
            If (OnLine <> '1') Then //OFFLINE only
            Begin
              Listitem := NetClnChkListView.Items.AddItem(Nil, icnt);
              Listitem.caption := Inttostr(icnt);
              Listitem.SubItems.Add(share);
              Listitem.SubItems.Add(NetName);
              Listitem.SubItems.Add(Ien);
              Listitem.SubItems.Add(iplace);
              Listitem.SubItems.Add(Site);
              Listitem.SubItems.Add(Stype);
              icnt := icnt + 1;
            End;
        End; //match PLACE
    End;
    Result:=True;
  Except
  End;
  Application.ProcessMessages();
  If NetClnChkListView.Columns.Count>1 Then SetListViewWidth(NetClnChkListView);
  If icnt>1 Then //508 Start
  Begin
    If NetClnChkListView.ItemIndex= -1 Then
    Begin
      TimerCleanup.Enabled:=True;
    End;
  End; //508 End
End; //NetClnChkListView.Items

(*
  menu # 3 | Network Location File cleanup | Clean Up
*)

Procedure TfMagUtility.btnNtLocDelRunClick(Sender: Tobject);
Var
  chkcln: String;
  chkvar: String;
  cien  : Integer;
  cnt   : Integer;
  i     : Integer;
  ID    : String;
  ids   : String;
  lopbeg: Integer;
  lopend: Integer;
  maxien: Int64;
  Output: String;
  ps    : Integer;
  stp   : String;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.btnNtLocDelRunClick';
  chkcln := ''; // NetIEN1^NetIEN2^...
  ids := '';
  If NetClnCHkListView.Selcount < 1 Then //must select at least one share
  Begin
    Showmessage('Error - Must select a network location first to remove.');
    Exit;
  End;
  For i := 0 To NetClnChkListVIew.Items.Count - 1 Do
  Begin //selected share(s) to be cleanup
    If NetClnChkListView.Items[i].Selected Then
    Begin
      ID := NetClnChkListView.Items[i].SubItems[2];
      chkcln := chkcln + '^' + ID;
      ids := '#' + ID + ' ' + NetClnChkListView.Items[i].SubItems[1] + ', ' + ids;
    End;
  End;
  If chkcln = '' Then
  Begin
    Showmessage('Error - Must select a network location first to remove.');
    Exit;
  End;
  NetworkLocClnRunning := True;
  btnNtLocDelRun.Enabled := False;
  NetClnChkListView.ReadOnly := False;
  FLogger.Info(#9 + 'Network Loc Cleanup' + #9 + 'Selected Network Loc.: ' + ids + ' by user:' + MagUtilityBroker.UserFullName);
  FErrorLogger.Info(#9 + 'Network Loc Cleanup' + #9 + 'Selected Network Loc.: ' + ids + ' by user:' + MagUtilityBroker.UserFullName);
  chkvar := '$P($G(^MAG(2005,0)),"^",3)';
  Output := MagUtilityBroker.FnmGetNode(chkvar); //total IEN 2005 entries
  maxien := Strtoint(Output);
  If maxien < 65 Then maxien := 65;
  ProgressBar1.Max := 65;
  ps := maxien Div 64;
  cnt := 1;
  Output := '1,start';
  While (cnt < 66) And (MagPiece(Output, ',', 1) = '1') Do
  Begin
    if ps>10000 then   //T21 - put smaller loop when IEN range became too large and put 'no respond' msg
    Begin
       lopbeg:= (cnt-1)*ps; //loop begin
       lopend:= cnt*ps;     //loop end
       for i:= 1 to (ps Div 10000) Do
       Begin
          cien := lopbeg + ((i-1)*10000);
          If cien>maxien then cnt:=99;
          stp := Inttostr(cien) + '#' + Inttostr(lopbeg+(i*10000));
          ProgressBar1.StepIt;
          Output := MagUtilityBroker.NtLocDel(chkcln, PLACE, stp);
          If MagPiece(Output, ',', 1) = '-1' Then
          Begin
            Showmessage('Cannot remove network location '+ ids + #13 + 'See error log file for details'); //-1, err msg
            FLogger.Error(#9 + 'Network Loc Cleanup' + #9 + 'cannot remove network location as image file references were found.'); //T18 - Agenda #5_2
            FErrorLogger.Error(#9 + 'Network Loc Cleanup' + #9 + 'cannot remove network location as image file references were found.'+ #9 + MagPiece(Output, ',', 2) + '.  ');//T18
            NetworkLocClnRunning := False;
            btnNtLocDelRun.Enabled := True;
            ProgressBar1.Position := 0;
            Exit;
          End;
       End;
       stp:=  MagPiece(stp,'#',2)+ '#' + inttostr(lopend);
       Output := MagUtilityBroker.NtLocDel(chkcln, PLACE,stp);
    End
    Else
       Begin
       cien := (cnt - 1) * ps;
       If cien>maxien then Exit;
       stp := Inttostr(cien) + '#' + Inttostr(ps * cnt); //step (begin IEN +'#'+ end IEN range to chk)
       ProgressBar1.StepIt;
       Output := MagUtilityBroker.NtLocDel(chkcln, PLACE, stp); //chkln: loc ien1^ien2^...
    End;
    If MagPiece(Output, ',', 1) = '-1' Then //error - found image reference link
    Begin
      Showmessage('Cannot remove network location '+ ids + #13 + 'See error log file for details'); //-1, err msg
      FLogger.Error(#9 + 'Network Loc Cleanup' + #9 + 'cannot remove network location as image file references were found.'); //T18 - Agenda #5_2
      FErrorLogger.Error(#9 + 'Network Loc Cleanup' + #9 + 'cannot remove network location as image file references were found.'+ #9 + MagPiece(Output, ',', 2) + '.  ');//T18
      NetworkLocClnRunning := False;
      btnNtLocDelRun.Enabled := True;
      ProgressBar1.Position := 0;
      Exit;
    End;
    cnt := cnt + 1;
    ProgressBar1.Position := cnt;
    //showmessage(chkcln+#13+PLACE+#13+stp);  //debug
    Application.ProcessMessages();
  End; //end of while

  Application.ProcessMessages();
  If (cnt > 1) And (MagPiece(Output, ',', 1) <> '-1') Then
  Begin
    stp := Inttostr(maxien) + '#' + Inttostr(maxien); //last step to cleanup
    Output := MagUtilityBroker.NtLocDel(chkcln, PLACE, stp);
  End;
  ProgressBar1.Position := 0;
  If MagPiece(Output, ',', 1) = '-1' Then //found image reference link - error
  Begin
    Showmessage('Cannot remove network location '+ ids + #13 + 'See error log file for details'); //-1, err msg
    FLogger.Error(#9 + 'Network Loc Cleanup' + #9 + 'cannot remove network location as image file references were found.'); //T18 - Agenda #5_2
    FErrorLogger.Error(#9 + 'Network Loc Cleanup' + #9 + 'cannot remove network location as image file references were found.'+ #9 + MagPiece(Output, ',', 2) + '.  ');//T18
    NetworkLocClnRunning := False;
    btnNtLocDelRun.Enabled := True;
    ProgressBar1.Position := 0;
  End; // -1, err
  If MagPiece(Output, ',', 1) = '0' Then //success
  Begin
    Showmessage('Removed network location successfully. (' + MagPiece(ids,',',1) +')');
    FLogger.Info(#9 + 'Network Loc Cleanup' + #9 + 'Network location (' + MagPiece(ids,',',1) +') has been removed.'); //t17 - LOG review , info log #1
    FErrorLogger.Info(#9 + 'Network Loc Cleanup' + #9 + 'Network location (' + MagPiece(ids,',',1) +') has been removed.'); //t17 - Log review  , err log #1
  End; //0, done
  NetClnListView();
  NetworkLocClnRunning := False;
  btnNtLocDelRun.Enabled := True;
End;

// clear list of offline JB report

Procedure TfMagUtility.btnImgOFLclrLstClick(Sender: Tobject);
Begin
  FProcessingPaused := True;
  offlineJBreport.Clear;
  offlineJBmkTotal.caption := '';
  BtnRunJBOffline.Enabled := False;
  BtnImgOFLclrlst.Enabled := False;
End;


//=========== Storage Copy ==========================
Procedure TfMagUtility.EdtStartIENexit(Sender: Tobject);
Var
  Magien: int64;
  chkvar, Cdate: String;
  ProcName: String;
  Date: String;
Begin
  ProcName:='TfMagUtility.edtStartIENexit';
  If (edtStartIEN.Text = '*') or (edtStartIEN.Text = '0') Then //default 1st IEN #2005
  Begin
    chkvar := '$O(^MAG(2005,0))';
    edtStartIEN.Text := MagUtilityBroker.FnmGetNode(chkvar);
    If edtStartIEN.Text = '' Then edtStartIEN.Text := '1';
  End;

  //If (edtStartIEN.Text = '') or (Not IsStrANumber(edtStartIEN.Text)) Then
  If (edtStartIEN.Text = '') Then Exit;
  If Not IsStrANumber(edtStartIEN.Text) Then
  Begin
    Showmessage('IEN must be numeric integer' );
    edtStartIEN.Text := '';
    MakeFocused(edtStartIEN);
    Exit;
  End;
  If Not FUT49Running Then btnPauseFUT49.Enabled := False;
  Magien := strtoint64(edtStartIEN.Text);
  If Magien <> 0 Then //found IMG create DT from VistA
  Begin
    chkvar := '$P($G(^MAG(2005,' + Inttostr(Magien) + ',2)),"^")\1+17000000';
    Cdate := MagUtilityBroker.FnmGetNode(chkvar); // image creation date
    If Cdate = '17000000' Then
    Begin
      Exit;
    End;
    Date:=FormatDateTime('mm/dd/yyyy',Now());
    if FMEXTDT(Cdate) = Date then  //t17 - exclude today image IR T16-3
    Begin
      Showmessage('From IEN: Image from today will not be copied');
      edtStartIEN.Text := '';
      MakeFocused(edtStartIEN);
      Exit;
    End;
    SetCaptionFut49byIEN4DT('(date. ' + FMEXTDT(Cdate) + ' )');  //T18 - approx.
  End;
End;

Procedure TfMagUtility.edtEndIENexit(Sender: Tobject);
Var
  ProcName: String;
Begin
  ProcName:='TfMagUtility.edtEndIENexit';
  If FUT49Running Then Exit;
  EnableRunButtonStorageCopy();
  UpdateLabel(lblCurrentFile,''); //reset progress panel
  lblTotalcnt.caption := '';      //
  lblCurrentFile.caption := '';   //
  //T18 -If cbxFromShareLoc.Text <> '' Then Exit; //already sel From source
  If (edtEndIen.Text = '') Then Exit; //CodeCR398
  If Not EndIENexit(Sender) Then Exit;
  If (cbxToShareLoc.Items.Count = 0) And (Trim(cbxFromShareLoc.Text) <> '')Then  //T20 - CodeCR 234  - re-RUN
  Begin //reset source/destination
      cbxFromShareLoc.ClearSelection;
      cbxFromShareLoc.Text := '';
      cbxToShareLoc.ClearSelection;
      cbxToShareLoc.Text := '';
  End;
  EnableRunButtonStorageCopy();
  //T20 - CodeCr 216
  lblCurrentFile.caption := '';
  lblIEN.caption := '';
  lblTotalcnt.caption := '';
  UpdateLabel(lblErrorCnt,'');
  Application.ProcessMessages(); //T20 - CodeCR 216
End;

Procedure TfMagUtility.edtStartDTonExit(Sender: Tobject);
Var
  Magien, DTRange, limit: String;
  dt0, dt1: Integer;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.edtStartDTonExit';
  If Not FUT49Running then btnPauseFUT49.Enabled := False;  //T21
  DTRange := edtStartDT.Text;
  If DTRange = '' Then
  Begin
    SetCaptionFut49byDt4ien('');
    Exit;
  End;
  limit := '1';
  Magien := MagUtilityBroker.DTRangeIEN(DTRange, limit); //RPC
  If (MagPiece(Magien, ',', 1) = '-1') Or (Magien = '0') Or (Strtoint(Magien) < 1) Then
  Begin
    Showmessage('Invalid date selected (' + edtStartDT.Text + ')  ' + MagPiece(Magien, ',', 2));
    edtStartDT.Text := '';
    SetCaptionFut49byDt4ien('');
    MakeFocused(btnSel4Dt);
    Exit;
  End;
  If edtEndDT.Text <> '' Then //chk DT range
  Begin
    dt0 := Strtoint(MagPiece(DTrange, '/', 3)) * 10000 +
      Strtoint(MagPiece(DTrange, '/', 1)) * 100 +
      Strtoint(MagPiece(DTrange, '/', 2));
    dt1 := Strtoint(MagPiece(edtEndDT.Text, '/', 3)) * 10000 +
      Strtoint(MagPiece(edtEndDT.Text, '/', 1)) * 100 +
      Strtoint(MagPiece(edtEndDT.Text, '/', 2));
    If dt0 > dt1 Then //from dt should older than to date
    Begin
      Showmessage('Error: Invalid date, From date must be earlier than To date');
      edtStartDT.Text := '';
      SetCaptionFut49byDt4ien('');
      MakeFocused(btnSel4Dt);
      Exit
    End;
  End; // chk DT range
  SetCaptionFut49byDt4ien('(approx. IEN=' + Magien + ' )');
  edtStartDT.Text := DTRange;
End;

Function TfMagUtility.validateRange(StartIEN: String; endIEN: String;
  Var Msg: String; Var ProblemIndex: Integer): Boolean;
Var
  First, Last: int64;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.validateRange';
  Msg := '';
  ProblemIndex := 0;
  If StartIEN='*' Then StartIEN := '1';
  Try
    First := strtoint64(StartIEN);
  Except
    On e: Exception Do
    Begin
      FLogger.Error(#9+ e.Message);
      FErrorLogger.Error(#9+ e.Message+ ' ' +StartIEN+' '+endIEN+' ('+UnitName+'.'+ProcName+'.33) ');
      Msg := 'From IEN must be integer value';
      Result := False;
      ProblemIndex := 1;
      Exit;
    End;
  End;

  Try
    Last := strtoint64(endIEN);
  Except
    On e: Exception Do
    Begin
      FLogger.Error(#9 + e.Message);
      FErrorLogger.Error(e.Message+ ' ' +StartIEN+' '+endIEN+' ('+UnitName+'.'+ProcName+'.34) ');
      Result := False;
      Msg := 'To IEN must be integer value';
      ProblemIndex := 2;
      Exit;
    End;
  End;
  If First < 0 Then
  Begin
    Result := False;
    Msg := 'From IEN must be greater than 0';
    ProblemIndex := 1;
  End
  Else
    If First <= Last Then
      Result := True
    Else
    Begin
      Result := False;
      Msg := 'From IEN must be before To IEN';
      ProblemIndex := 1;
    End;
End;

Procedure TfMagUtility.FUT49FromLocClick(Sender: Tobject);
Var
  xline, Ien, share, chkvar, stgtype, netloc0: String;
  sel, cnt: Integer;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.FUT49FromLocClick';
  FUT49RemoveOld.Enabled := True;
  FUT49FromLoc := '';
  sel := cbxFromShareLoc.ItemIndex;
  If sel = -1 Then sel := 0; //take 1st default, without clk from listing
  If rgFut49ChkType.ItemIndex=0 Then
    If (EdtEndDT.Text = '') Or (EdtStartDT.Text = '') Then
    Begin
      Showmessage('Error - Must specify a full date range before selecting the From location.');
      cbxFromShareLoc.ClearSelection;
      cbxToShareLoc.ClearSelection;
      If (EdtEndDT.Text = '') Then
        MakeFocused(btnSel2Dt)
      Else
        If (EdtStartDT.Text = '') Then
          MakeFocused(btnSel4Dt);
      Exit;
    End;

  If rgFut49ChkType.ItemIndex=1 Then
    If (EdtEndIen.Text = '') Or (EdtStartIen.Text = '') Then
    Begin
      Showmessage('Error - Must specify a full IEN range before selecting the From location.');
      cbxFromShareLoc.ClearSelection;
      cbxToShareLoc.ClearSelection;
      If (EdtEndIen.Text = '') Then
        MakeFocused(EdtEndIen)
      Else
        If (EdtStartIen.Text = '') Then
          MakeFocused(EdtStartIen)
        Else
          MakeFocused(rgFut49ChkType);
      Exit;
    End;
  If sel < 0 Then Exit;
  FUT49RemoveOld.Enabled := True;
  xline := Uppercase(cbxFromShareLoc.Items[sel]);
  Ien := MagPiece(xline, '#', 1);
  share := MagPiece(xline, '*', 2);
  //chkvar := MagPiece(xline, '*', 1);
  stgtype := MagPiece(xline, '*', 1);
  stgtype := MagPiece(stgtype, ' ', 2);
  If Not Directoryexists(share) Then
  Begin
    Showmessage('Storage Copy - ' + #13 + ' Network Path "' + share + '" cannot be accessed.' + #13 + 'Verify that the share is online and accessible.');
    cbxFromShareLoc.ClearSelection;
    Exit;
  End;
  FUT49FromLoc := Ien; //net loc IEN
  //business rule - can't remove source file from JB-WORM
  If ANSIcontainsStr(stgtype, 'WORM') Then
  Begin
    FUT49RemoveOld.Enabled := False; //selected
    If FUT49RemoveOld.Checked Then Showmessage('Storage Copy - cannot remove source files from jukebox');
    FUT49RemoveOld.Checked := False; //JB file can't remove
  End;
  // T21 - Check if is Hashed subdirectory
  chkvar := '$G(^MAG(2005.2,' + Ien + ',0))_" "';
  netloc0 := MagUtilityBroker.FnmGetNode(chkvar);
  showmessage(  netloc0);
  If MagPiece(netloc0, '^', 8) <> 'Y' Then showmessage('Warning - Selected '+xline+ ' is not a hashed subdirectory structure, will require additional time to copy.');
  //remove selected From LOC and fill in TO LOC
  cbxToShareLoc.Items.Clear;
  cnt := cbxFromShareLoc.Items.Count - 1;
  For sel := 0 To cnt Do
  Begin
    xline := Uppercase(cbxFromShareLoc.Items[sel]);
    Ien := MagPiece(xline, '#', 1);
    stgtype := MagPiece(xline, '*', 1);
    stgtype := MagPiece(stgtype, ' ', 2);
    If Ien <> FUT49FromLoc Then
    Begin
      chkvar := '$G(^MAG(2005.2,' + Ien + ',0))_" "';
      netloc0 := MagUtilityBroker.FnmGetNode(chkvar);
      If MagPiece(netloc0, '^', 8) = 'Y' Then //copy dest must set HASH 'Y'
      Begin
        cbxToShareLoc.Items.Add(xline);
      End;
    End
  End;
  If (cbxToShareLoc.Items.Count > 0) Then cbxToShareLoc.Text := cbxToShareLoc.Items[0];
  MakeFocused(btnToLoc);
End;

Procedure TfMagUtility.FUT49ToLocClick(Sender: Tobject);
Var
  sel: Integer;
  xline, Ien, share, chkvar, route: String;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.FUT49ToLocClick';
  If cbxFromShareLoc.Text = '' Then
  Begin
    Showmessage('Specify the "FROM image network location" first before selecting the "To image network location".');
    cbxToShareLoc.ClearSelection;
    MakeFocused(btnFromLoc);
    Exit;
  End;
  sel := cbxToShareLoc.ItemIndex;
  If sel = -1 Then sel := 0;
  If rgFut49ChkType.ItemIndex=1 Then
    If (EdtEndIen.Text = '') Or (EdtStartIen.Text = '') Then
    Begin
      Showmessage('Error - Must specify a full IEN range before selecting the To location.');
      cbxFromShareLoc.ClearSelection;
      cbxToShareLoc.ClearSelection;
      If (EdtEndIen.Text = '') Then
        MakeFocused(EdtEndIen)
      Else
        If (EdtStartIen.Text = '') Then
          MakeFocused(EdtStartIen)
        Else
          MakeFocused(rgFut49ChkType);
      Exit;
    End;

  If rgFut49ChkType.ItemIndex=0 Then
    If (EdtEndDT.Text = '') Or (EdtStartDT.Text = '') Then
    Begin
      Showmessage('Error - Must specify a full date range before selecting the To location.');
      cbxFromShareLoc.ClearSelection;
      cbxToShareLoc.ClearSelection;
      If (EdtEndDT.Text = '') Then
        MakeFocused(btnSel2Dt)
      Else
        If (EdtStartDT.Text = '') Then
          MakeFocused(btnSel4Dt);
      Exit;
    End;
  If sel < 0 Then Exit;
  FUT49ToLoc := '';
  If FUT49SetNewLoc.Enabled Then FUT49SetNewLoc.Checked := True;
  With cbxToShareLoc Do
  Begin
    If ItemIndex = -1 Then ItemIndex := 0;
    sel := ItemIndex;
    If sel < 0 Then Exit;
    xline := Uppercase(Items[sel]);
    Ien := MagPiece(xline, '#', 1);
    share := MagPiece(xline, '*', 2);
    If Not Directoryexists(share) Then //check to see if the share is on-line
    Begin
      Showmessage('Storage Copy - ' + #13 + ' Network Path "' + share + '" cannot be accessed.' + #13 + 'Verify that the share is online and accessible.');
      cbxToShareLoc.ClearSelection;
      Exit;
    End;
    chkvar := '$P($G(^MAG(2005.2,' + Ien + ',0)),U,9)_" "';
    route := MagUtilityBroker.FnmGetNode(chkvar);
    If route = '1 ' Then
    Begin
      Showmessage('Storage Copy - ' + #13 + 'Because the destination share is designated as a Routing share:' + #13
        + '1)	You may need to map a drive to this share to perform the copy;' + #13
        + '2)	#2005 references will not be changed; and' + #13
        + '3)	Source files cannot be deleted.');

      FUT49SetNewLoc.Enabled := False; /// the router loc cannot set #2005 reference
      FUT49SetNewLoc.Checked := False;
      FUT49RemoveOld.Enabled := False; /// nor remove files from source location
      FUT49RemoveOld.Checked := False;
    End
    Else
      If FUT49RemoveOld.Checked = False Then FUT49SetNewLoc.Enabled := True;
    FUT49ToLoc := Ien;
  End;
  MakeFocused(btnProcessFUT49);
End;


Procedure TfMagUtility.ViewLogClick(Sender: Tobject);
  function GetTempDirectory: String;
  var
    tempFolder: array[0..MAX_PATH] of Char;
  begin
    GetTempPath(MAX_PATH, @tempFolder);
    result := StrPas(tempFolder);
  end;
Var
  OpenDialog: TOpenDialog;
  ProcName: String;
  S: TFileStream;
  TempFileName: String;
Begin
  ProcName:='TfMagUtility.ViewLogClick';
  btnImgOflClrLst.Enabled := False;
  OpenDialog := TOpenDialog.Create(Self);
  With OpenDialog Do
  Begin
    InitialDir := GetAppPath() + '\logs';
    DefaultExt := 'MagUtil*.*';
    Filter := 'Text files (*.log)|*.log*';
    FilterIndex := 0; { start the dialog showing all files }
    Options := [ofFileMustExist, ofPathMustExist, ofEnableSizing];
    If Not Execute Then
      Exit
    Else
    Begin
      try
         Begin
         TempFileName:= GetTempDirectory+ExtractFileName(FileName);
         If FileExists(TempFileName) Then DeleteFile(TempFileName);
         CopyFile(PAnsiChar(FileName),PAnsiChar(TempFileName),False);
         sleep(500);
         S:= TfileStream.Create(TempFileName, fmOpenRead or fmShareDenyNone); //t17 - IR T16-4
         WinExec(PChar('NOTEPAD.exe ' + TempFilename), SW_SHOW);   //T17 - log open failed
         S.Free;
         End;
      Except
         Begin
         showmessage('Sorry, '+Filename+ ' log file filled and locked.'+#13+#10+ 'Please restart application to'+
                      ' clear lock, and view log.'); //t17 - IR T16-4
         Exit;
         End;
      end;  //try view log
      //T18 off - FLogger.Info(#9+'Open Log '+ Filename + ' by user: ' + MagUtilityBroker.UserFullName+#13+#10);
    End;
  End; //end of with OpenDialog
End;


Procedure TfMagUtility.FUT49RemoveOldClick(Sender: Tobject);
Var
  ProcName: String;
Begin
  ProcName:='TfMagUtility.FUT49RemoveOldClick';
  StorageCopyWarning(Sender);
  If FUT49RemoveOld.Checked Then
  Begin
    Showmessage('Storage Copy - ' + #13 + 'Note: source files will be moved and #2005 references will be updated to point to the files on the destination share.');
    FUT49SetNewLoc.Checked := True;
    FUT49SetNewLoc.Enabled := False;
  End
  Else
  Begin
    FUT49SetNewLoc.Checked := False;
    FUT49SetNewLoc.Enabled := True;
  End;
  UpdateLabel(lblErrorCnt,'');
  lblTotalCnt.caption := '';
  lblFileEmptyCnt.caption := '';
End;

Procedure TfMagUtility.FUT49AftHourClick(Sender: Tobject);
Var
  ProcName: String;
Begin
  ProcName:='TfMagUtility.FUT49AftHourClick';
  StorageCopyWarning(Sender);
  UpdateLabel(lblErrorCnt,'');
  lblTotalCnt.caption := '';
  lblFileEmptyCnt.caption := '';
End;

(*
menu #4 | Storage Copy | RUN
*)

Procedure TfMagUtility.btnProcessFUT49Click(Sender: Tobject);
Var
  ProcName: String;
Begin
  ProcName:='TfMagUtility.btnProcessFUT49Click';
  LastErrorCode:=0; //T21
  FUT49Run(Sender);
End;

(*
Check if the task should be run at current time -
#5 Storage Copy - select 'Run After-Hours' option
*)

Procedure TfMagUtility.FUT49chktime();
Var
  TMclock, TMhr, TMpm, Spin: String;
  Day, workhr, TMsleep, wakeup: Integer;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.FUT49chktime';
  if not FUT49RUNNING then exit;  //task cancel
  TMSleep := 1 * 1000; //if it runs aft hours, sleep at work hours (every 1 sec)
  wakeup := 0;
  Spin := '|';
  While wakeup = 0 Do
  Begin
    lblCurrentFile.Visible := True;
    If btnProcessFUT49.Focused And (btnProcessFUT49.caption = '&Cancel') Then wakeup := 1;
    TMclock := TimeToStr(Time);
    TMhr := MagPiece(TMclock, ':', 1);
    TMpm := MagPiece(TMclock, ' ', 2);
    Day := DayOfWeek(Date);
    workhr := Strtoint(TMhr);
    ///showmessage('Day '+inttostr(Day)+#13+'Hr '+Tmhr+TmPm);
    If (Day > 1) And (Day < 7) Then //days[1] := 'Sunday', days[7] := 'Saturday';
    Begin
      If (Day = 2) And (TMpm = 'AM') And (workhr = 12) Then wakeup := 1 //early Monday morning
      Else
      If (TMpm = 'AM') And (workhr <> 12) And (workhr > 5) Then //wk hours 6pm ~ 6 am
      Begin
        lblCurrentFile.caption := Spin + ' ** Storage Copy is scheduled.  Process will start at 6 PM.  **';
        //t17 - keep no. of lblIEN.caption := ''; //while in business hours
        //t17 - keep no. of lblTotalcnt.caption := '';
        //t17 UpdateLabel(lblErrorCnt,'');
        if (btnPauseFUT49.caption = '&Resume') and btnProcessFut49.Focused then wakeup:=1;  //T17 - IR15-24 refresh process status
        Sleep(TMsleep);
        If Spin = '|' Then
          Spin := '-'
        Else
          Spin := '|';
      End
      Else
        If (TMpm = 'PM') And ((workhr < 6) Or (workhr = 12)) Then
        Begin
          lblCurrentFile.caption := Spin + ' ** Storage Copy is scheduled.  Process will start at 6 PM.  **';
          if (btnPauseFUT49.caption = '&Resume') and btnProcessFut49.Focused then wakeup:=1;  //T17 - IR15-24 refresh process status
          Sleep(TMSleep);
          If Spin = '|' Then
            Spin := '-'
          Else
            Spin := '|';
        End
        Else
          wakeup := 1; //aft-hours now
    End;
    If (btnPauseFUT49.caption = '&Pause') then Application.Processmessages //t17
    Else UpdateLabel(lblCurrentFile,'');//t17
    If (Day = 1) Or (Day = 7) Then wakeup := 1;
    If (cbxFromShareLoc.Text = '') And (cbxToShareLoc.Text = '') Then wakeup := 1; //task been canceled
  End; //while wakeup =0
  UpdateLabel(lblCurrentFile,'');//t17 IR T15-24
  ArrangeStatusLabels();
End;

(*
Storage Copy - Copy file execute
MAGR (1) := RPC status
MAGR (2) := Dir UNC/file.*  (check source file exist)
MAGR (3) := mkdir hashed folder
MAGR (4) := command code 1
MAGR (n) := command code n
*)

function TfMagUtility.ToCopyFile(var MAGR: TStrings; var Copied,NofTot: String): Boolean;
Var
  CmdLine : String;
  FilePath: String;
  i       : Integer;
  Ien     : Integer;
  inTot   : Integer;
  N       : Integer;
  ProcName: String;
  sgExt   : String;
  St      : String;
Begin
  ProcName:='TfMagUtility.ToCopyFile';
  ProblemIndex:=0; //monitor error
  Result  := False;
  St      := MagPiece(MAGR[0], ':', 1);
  Ien     := Strtoint(St);
  inTot   := 0;
  For i := 3 To MAGR.Count - 1 Do
  Begin
    sgExt   := Trim(ExtractFileExt(MAGR[i]));
    If sgExt<>'' Then inTot:=inTot+1;
  End;
  N       := 0;
  Copied  := '';
  NofTot  := IntToStr(N)+' of '+IntToStr(inTot);
  If Ien > 0 Then
  Begin
    lblIEN.caption := Inttostr(Ien);
    FilePath := MagPiece(MAGR[1], ' ', 2);
    If FilePath = '' Then Exit; //no file
    If (FUT49Mark = '') or (NOT FUT49Running) Then Exit; //T17 - task cancel
    If btnProcessFUT49.Focused And (btnProcessFUT49.caption = '&Cancel') Then Exit; //T17
    If Not FileExists(FilePath) Then
    Begin
      FLogger.Error(#9 + 'Storage Copy' + #9 + 'The source file was not found - ' + FilePath + '.');
      FErrorLogger.Error(#9 + 'Storage Copy' + #9 + 'The source file was not found - ' + FilePath + '. ,'+' ('+UnitName+'.'+ProcName+'.13)');
      Exit; //no file to copy
    End;
    CmdLine := MAGR[2]; //create hashed folder
    If Not ExecuteShellCommandLoop('Storage Copy',CmdLine,True) Then Exit;
    For i := 3 To MAGR.Count - 1 Do
    Begin
      CmdLine := MAGR[i];
      If Not ExecuteShellCommandLoop('Storage Copy',CmdLine,True) Then Exit;
      sgExt   := ExtractFileExt(CmdLine);
      If sgExt<>'' Then
      Begin
        sgExt:=Trim(sgExt);
        If Copied<>'' Then Copied:=Copied+', '+sgExt Else Copied:=sgExt;
        N:=N+1;
        NofTot  := IntToStr(N)+' of '+IntToStr(inTot);
      End;
      If ProblemIndex > 0 Then
      Begin
        Result := False; //error cmdline
        Exit;
      End;
    End;
    If ProblemIndex=0 Then Result := True;
  End; //ien>0
End; //fut49 COPYfILE

Procedure TfMagUtility.btnPauseFUT49Click(Sender: Tobject);
Var
  ProcName: String;
Begin
  ProcName:='TfMagUtility.btnPauseFUT49Click';
  If btnPauseFUT49.caption = '&Pause' Then
  Begin
    boInRetryLoop           := False;
    FProcessingPaused       := True;
    btnProcessFUT49.caption := '&Cancel'; //can be stop
    FLogger.Info(#9 + 'Storage Copy' + #9 + 'The task paused at ' + TimeToStr(Time) + ' by user:'+ MagUtilityBroker.UserFullName); //t17 - IR T16-2
    FErrorLogger.Info(#9 + 'Storage Copy' + #9 + 'The task paused at ' + TimeToStr(Time) + ' by user:'+ MagUtilityBroker.UserFullName);
    If bmpStop.Width>3 Then btnProcessFUT49.Glyph.Assign(bmpStop);
    //btnProcessFUT49.Glyph.Assign();
    btnProcessFUT49.Enabled := True;
    btnPauseFUT49.caption := '&Resume';
    If bmpGo.Width>3 Then btnPauseFUT49.Glyph.Assign(bmpGo);
    btnPauseFUT49.Enabled := True;       //can be resume
    //t17 - keep current IEN info ~~lblCurrentFile.caption := '';
    Showmessage('Pause - Click `Cancel` or `Resume` when you are ready to continue.');
    //MakeFocused(btnProcessFUT49);
    If FUT49AftHour.Checked Then UpdateLabel(lblCurrentFile,''); //t17 - IR15-24 rmv aft-hour msg (GUI)
    If btnHide.caption = 'Show processing detail' Then grpProcessStatus.Visible := False;
    SetPnlRetryVis(False);
  End
  Else
    If btnPauseFUT49.caption = '&Resume' Then
    Begin //resume
      boInRetryLoop           := False;
      FUT49Running            := True;
      FProcessingPaused       := False;
      btnProcessFUT49.caption := '&Run';
      FLogger.Info(#9 + 'Storage Copy' + #9 + 'The task resumed at ' + TimeToStr(Time) + ' by user:'+ MagUtilityBroker.UserFullName); //t17 - IR T16-2
      FErrorLogger.Info(#9 + 'Storage Copy' + #9 + 'The task resumed at ' + TimeToStr(Time) + ' by user:'+ MagUtilityBroker.UserFullName);
      If bmpGo.Width>3 Then btnProcessFUT49.Glyph.Assign(bmpGo);
      btnProcessFUT49.Enabled := False;
      btnPauseFUT49.caption := '&Pause';   //can be pause (only)
      If bmpStop.Width>3 Then btnPauseFUT49.Glyph.Assign(bmpStop);
      btnPauseFUT49.Enabled := True;
      MakeFocused(pcUtilTabs);
      //ShowMessage(
      //  'curIEN='+IntToStr(FUT49curIEN)+#13+#10+
      //  'IEN2='+IntToStr(FUT49IEN2)+#13+#10+
      //  'NET0='+FUT49NET0);
      FUT49Resume(FUT49curIEN,FUT49IEN2,FUT49NET0);
      If btnHide.caption = 'Hide processing detail' Then grpProcessStatus.Visible := True;
    End;
End;

(*
menu #4 - Storage Copy - Resume/restart task after paused the task
*)

Procedure TfMagUtility.BtnFUT49resumeClick();
Var
  boFUT49AftHourEnabled  : Boolean;
  boFUT49RemoveOldEnabled: Boolean;
  boFUT49SetNewLocEnabled: Boolean;
  borgFut49ChkTypeEnabled: Boolean;
  childIEN               : String;
  chkvar                 : String;
  Copied                 : String;
  CopyfileType           : String; //ful '*', abs '@', big 'b'
  curIEN                 : Integer;
  doneMsg                : String;
  Fname                  : String;
  fulfile                : String;
  IEN0                   : String;
  IEN2                   : Integer;
  MAGR                   : TStrings;
  NET0                   : String;
  NewptrRef              : String; //RAID or JB
  NofTot                 : String;
  ProcName               : String;
  Res                    : String;
  Set2005pt              : String;

Begin
  ProcName:='TfMagUtility.BtnFUT49resumeClick';
  BtnPauseFUT49.Enabled := True;
  BtnPauseFUT49.caption := 'Pause';
  If bmpStop.Width>3 Then btnPauseFUT49.Glyph.Assign(bmpStop);
  btnprocessFUT49.Enabled := False;
  grpProcessStatus.Visible := True;
  If FUT49Mark = '' Then Exit; //nothing to restart
  If btnProcessFUT49.Focused And (btnProcessFUT49.caption = '&Cancel') Then Exit;//t17
  FUT49Running := True;
  IEN0 := MagPiece(FUT49Mark, '^', 1);
  NET0 := MagPiece(FUT49Mark, '^', 2);
  curIEN := Strtoint(MagPiece(IEN0, ':', 1));
  IEN2 := Strtoint(MagPiece(IEN0, ':', 2));
  doneMsg := '';
  Set2005pt := '';
  boFUT49AftHourEnabled  := FUT49AftHour.Enabled;
  boFUT49RemoveOldEnabled:= FUT49RemoveOld.Enabled;
  boFUT49SetNewLocEnabled:= FUT49SetNewLoc.Enabled;
  borgFut49ChkTypeEnabled:= rgFut49ChkType.Enabled;
  Try
    FUT49AftHour.Enabled  := False;
    FUT49RemoveOld.Enabled:= False;
    FUT49SetNewLoc.Enabled:= False;
    rgFut49ChkType.Enabled:= False;
    While FUT49Running And Not (curIEN < IEN2) Do
    Begin
      Application.ProcessMessages();
      If btnProcessFUT49.Focused And (btnProcessFUT49.caption = '&Cancel') Then
      Begin
        lblCurrentFile.caption := '';
        lblIEN.caption := '';
        lblTotalcnt.caption := '';
        UpdateLabel(lblErrorCnt,'');
        FUT49Running := False;
        FUT49Mark:= '';
        FUT49AftHour.Enabled  := boFUT49AftHourEnabled;  //T21 - CR 239
        FUT49RemoveOld.Enabled:= boFUT49RemoveOldEnabled;
        FUT49SetNewLoc.Enabled:= boFUT49SetNewLocEnabled;
        rgFut49ChkType.Enabled:= borgFut49ChkTypeEnabled;
        StorageCopyCancel(#9 + 'Storage Copy' + #9 + 'The task canceled at ' + TimeToStr(Time),'Storage Copy was canceled by user:'+MagUtilityBroker.UserFullName);
        Break; ///Exit;
      End;
      While (btnProcessFUT49.caption = '&Cancel') And (btnPauseFUT49.caption = '&Resume') Do
      Begin
        lblCurrentFile.caption := '';
        lblIEN.caption := '';
        lblTotalcnt.caption := '';
        UpdateLabel(lblErrorCnt,'');
        Sleep(100);
        Application.Processmessages;
        If btnProcessFUT49.Focused Then
        Begin
            FUT49AftHour.Enabled  := True;  //T21 - CR 239
            FUT49RemoveOld.Enabled:= True;
            FUT49SetNewLoc.Enabled:= True;
            rgFut49ChkType.Enabled:= True;
            GaugeFUT49.Visible:=False;
            Break;  //Cancel/Stop task
        End;
      End;

      If not FUT49Running and (FUT49Mark='') then Break; //IR#T15-25
      // check 'Run after hours' option
      If FUT49AftHour.Checked Then FUT49chktime();
      If not FUT49Running and (FUT49Mark='') then Break;  //t17 IR T16-13
      If btnProcessFUT49.Focused And (btnProcessFUT49.caption = '&Cancel') Then Exit;//T17
      UpdateLabel(lblCurrentFile,'');//t17 IR T16-11
      // check if #2005 entry exist
      chkvar := '$G(^MAG(2005,' + Inttostr(curIEN) + ',0))_" "';
      FulFile := MagUtilityBroker.FnmGetNode(chkvar);
      If Fulfile = ' ' Then
      Begin
        //T21 - removed,  FLogger.Info(#9 + 'Storage Copy' + #9 + 'image IEN# ' + Inttostr(curIEN) + ' image entry had been deleted, will be skipped.');
        //                FErrorLogger.Info(#9 + 'Storage Copy' + #9 + 'image IEN# ' + Inttostr(curIEN) + ' image entry had been deleted, will be skipped.');
        curIEN := curIEN - 1; //skip to next
        IEN0 := Inttostr(curIEN) + ':' + Inttostr(IEN2);
        FUT49Mark := IEN0 + '^' + NET0;
        ptrEmptyCnt := ptrEmptyCnt + 1;
        //errorcnt:=errorcnt+1;
      End
      Else //with #2005 file entry
      Begin
        showdetail:=0;
        //check if it is a group/parent node - no full image file ref/no ptr
        If MagPiece(FulFile, '^', 2) = ' ' Then //fileRef
        Begin
          chkvar := '$P($G(^MAG(2005,' + Inttostr(curIEN) + ',1,+$O(^MAG(2005,' + Inttostr(curIEN) + ',1,0)),0)),U,1)_" "';
          childIEN := MagUtilityBroker.FnmGetNode(chkvar);
          If (childIEN <> ' ') Then
          Begin
            childIEN := MagPiece(childIEN, ' ', 1);
            If (Strtoint(childIEN) > IEN2) And (OKCNT > 0) Then
              OKCNT := OKCNT + 1
            Else
              errorcnt := errorcnt + 1;
            //T21 - FLogger.Info(#9 + 'Storage Copy' + #9 + 'IEN# ' + Inttostr(curIEN) + ' *Group/parent node of # ' + childIEN);
            //T21 - FErrorLogger.Info(#9 + 'Storage Copy' + #9 + 'IEN# ' + Inttostr(curIEN) + ' *Group/parent node of # ' + childIEN);
          End
          Else
            ptrEmptyCnt := ptrEmptyCnt + 1;
          curIEN := curIEN - 1; //skip to next
          IEN0 := Inttostr(curIEN) + ':' + Inttostr(IEN2);
          FUT49Mark := IEN0 + '^' + NET0;
          //lblFileEmptyCnt.caption := Inttostr(ptrEmptyCnt);
        End //2005 2nd piece null
        Else
        Begin
            If (Length(MagPiece(FulFile, '^', 2))>12) And (Length( MagPiece(FulFile, '^', 2))<18) Then showdetail:=1;//T20 - show Fix 9.3, 10.3 to 14.3 formt #CodeCR 229
        End;
      End;

      MAGR := Tstringlist.Create(); //RPC call - MAGR returned value
     try
      If MagUtilityBroker.FETCH(MAGR, IEN0, NET0) Then //RPC success
      Begin ///MAGR[0] (returned value) IM_":"_ENDIM_",CMD summit "_LNO_U_WRTLOC_U_$G(FRMLOC)_U_DT_U_$G(MOVOLD)_"@"_DATE_"|"_DT2_"|"_$G(StorageTYP)_"|"_FileNAME_"|"_+$G(XBIG,0)
        Res := MagPiece(MAGR[0], ',', 1);
        curIEN := Strtoint(MagPiece(Res, ':', 1));
        Fname := MagPiece(MAGR[0], '|', 4);
        If curIEN > 0 Then
        Begin
          If FUT49Mark = '' Then Break; //T17 - task cancel
          totalcnt := totalcnt + 1;
          lbltotalcnt.caption := Inttostr(totalcnt);
          ProblemIndex:=0; //monitor error
          If ToCOPYFILE(MAGR,Copied,NofTot) Then
          Begin
            If (FUT49RemoveOld.Checked) Or (FUT49SetNewLoc.Checked) Then
            Begin
              chkvar := '$G(^MAG(2005,' + Inttostr(curIEN) + ',0))_" "';
              FulFile := MagUtilityBroker.FnmGetNode(chkvar); //IR#T15-7
              If (MagPiece(FulFile, '^', 3)=Fut49FromLoc) or (MagPiece(FulFile, '^', 5)=Fut49FromLoc) then
              Begin
                  CopyFileType:='*'; //set default full file
                  If (MagPiece(FulFile, '^', 3)=Fut49FromLoc) And (MagPiece(FulFile, '^', 4)<>Fut49FromLoc) Then CopyFileType:='r'; //IR#T15-10 P3 only
              End
              Else
                   if (MagPiece(FulFile, '^', 4)=Fut49FromLoc) then CopyFileType:='@' //set ABS
                   Else
                   Begin
                       CopyFileType:='b'; //set BIG
                       //*DEBUG  showmessage(FulFile+#13+Fut49FromLoc+#9+inttostr(CurIEN)+#9+'resume');
                   End;
              NewptrRef:='RAID';
              If (MagPiece(MAGR[0], '|', 3) = 'RJ') or (MagPiece(MAGR[0], '|', 3) = 'JJ') Then NewptrRef:='JB '; //t17 T16-7 log what ptr was set
              Set2005pt :='';
              If MagUtilityBroker.setNETloc(Inttostr(curIEN), FUT49ToLoc + CopyFileType + '|' +FUT49FromLoc) Then ///set new Dest reference
              Begin
                Set2005pt := ', set new '+NewptrRef+' reference ' + MagPiece(cbxToShareLoc.Text,' ',2) + '.';
                //FLogger.Info(#9 + 'Storage Copy' + #9 + 'Set image IEN ' + Inttostr(curIEN) + ' with new '+NewptrRef+' reference ' + MagPiece(cbxToShareLoc.Text,' ',2) + '.');  //T19 CodeCR=IMAG0000233
                //FErrorLogger.Info(#9 + 'Storage Copy' + #9 + 'Set image IEN ' + Inttostr(curIEN) + ' with new '+NewptrRef+' reference ' + MagPiece(cbxToShareLoc.Text,' ',2) + '.'+' ('+UnitName+'.'+ProcName+'.13)');
              End;
                ///Only if copy from RAID to JB, and remove source file, keep only JB (unset 0 P3, P4 or FBIG P1 for RAID)
              If (FUT49RemoveOld.Checked) And (MagPiece(MAGR[0], '|', 3) = 'RJ') Then
                If MagUtilityBroker.unsetNETloc(Fname, MagPiece(Fname, '.', 2), curIEN) Then
                Begin
                  MagUtilityBroker.unsetNETloc(Fname, 'ABS', curIEN); //unset ABS ref
                  If (MagPiece(MAGR[0], '|', 5) <> '0') Then MagUtilityBroker.unsetNETloc(Fname, 'BIG', curIEN);
                  Set2005pt := ', cleared RAID pointer '+ MagPiece(cbxFromShareLoc.Text,' ',2);
                  //FLogger.Info(#9 + 'Storage Copy' + #9 + 'Cleared RAID pointer for image IEN ' + Inttostr(curIEN));  //T19 CodeCR=IMAG0000233
                  //FErrorLogger.Info(#9 + 'Storage Copy' + #9 + 'Cleared RAID pointer for image IEN ' + Inttostr(curIEN)+' ' +Fname+' ('+UnitName+'.'+ProcName+'.14)');
                End
                Else //can't unset
                Begin
                  FLogger.Error(#9 + 'Storage Copy' + #9 + 'Error occurred, clearing RAID pointer for image IEN ' + Inttostr(curIEN));
                  FErrorLogger.Error(#9 + 'Storage Copy' + #9 + 'Error occurred, clearing RAID pointer for image IEN ' + Inttostr(curIEN)+ ' - '+Fname +' ('+UnitName+'.'+ProcName+'.15)');
                End;
            End;
            FulFile := MagPiece(FulFile, '^', 2);
            FLogger.Info(#9 + 'Storage Copy' + #9 + 'Copied all files found for image IEN ' + Inttostr(curIEN) + ' ('+Copied+') ('+NofTot+')' + Set2005pt); //T20 CodeCR=IMAG00000233
            FErrorLogger.Info(#9 + 'Storage Copy' + #9 + 'Copied all files found for image IEN ' + Inttostr(curIEN) + ' ('+Copied+') ('+NofTot+')'+ Set2005pt + ' ('+UnitName+'.'+ProcName+'.16)');
            OKCNT := OKCNT + 1;
            lbltotalcnt.caption := Inttostr(OKCNT);
          End
          Else
          Begin //Copy failed
            If FUT49Running then
            Begin
               errorcnt := errorcnt + 1;
               UpdateLabel(lblErrorCnt,errorcnt);
               If Copied<> '' Then
               Begin
                 FLogger.Info(#9 + 'Storage Copy' + #9 + 'Not all files copied for image IEN ' + Inttostr(curIEN) + ' ('+Copied+'*) stopped at ('+NofTot+')');
                 FErrorLogger.Info(#9 + 'Storage Copy' + #9 + 'Not all files copied for image IEN ' + Inttostr(curIEN) + ' ('+Copied+'*) stopped at ('+NofTot+')');
               End;
               If (GetLastError=112) Or (GetLastError=39) then  //T22 - forced to stop, when it is Disk full
               Begin
	                FUT49Running:=False;
	                FUT49Mark:='';
               End;
            End;
          End;
        End;
        If (curIEN = 0) And (MagPiece(MAGR[0], ',', 2) = 'Done') Then //completed last ien
        Begin
          curIEN := IEN2 - 1; //done & stop loop
          If OKCNT > 0 Then
          Begin
            If (doneMsg = '') and FUT49Running Then
            Begin
              Application.Processmessages;
              If errorcnt = 0 Then
                Showmessage('All requested IENs have been copied' + #13 + 'Total (' + Inttostr(OKCNT) + ') copied, (' + Inttostr(errorcnt) + ') not copied, '
                  + 'see log file for details')
              Else
                Showmessage('All requested IENs have been copied' + #13 + 'Total (' + Inttostr(OKCNT) + ') copied, (' + Inttostr(errorcnt) + ') not copied.'
                  + #13 + #13 + 'Some files may not be copied, see log file for details');
              doneMsg := '1';
              FUT49Running := False;
              FLogger.Info(#9 + 'Storage Copy' + #9 +#9 + 'Completed coping files for requested IEN or date range - ' + Inttostr(OKCNT) + ' IENs copied, ' + Inttostr(errorcnt) + ' IENs not copied.'); //T21, t17 - log summary
              FErrorLogger.Info(#9 + 'Storage Copy' + #9 +#9 + 'Completed coping files for requested IEN or date range - ' + Inttostr(OKCNT) + ' IENs copied, ' + Inttostr(errorcnt) + ' IENs not copied.'); //T21, T18
            End;
          End
          Else  //OKCNT=0
            If doneMsg = '' Then
            Begin
              Showmessage('Storage Copy completed. No files were copied.' + #13 +#10
              + 'See log and check patch documentation for troubleshooting suggestions.'); //t17 - IR 15-34
              FLogger.Info(#9 + 'Storage copy' + #9 + 'Task is complete. No files were copied.'+#9+ 'No files exist at source location'); //T21 - log
              FErrorLogger.Info(#9 + 'Storage copy' + #9 + 'Task is complete. No files were copied.'+#9+ 'No files exist at source location'); //T21 - log
              doneMsg := '1';
              FUT49Running := False;
            End;
        End;
        If Res = '-2' Then   //T24 - broker timeout
        Begin
            FLogger.Error(#9 + 'Storage Copy' + #9 + 'Continued, RPC - ' + MagPiece(MAGR[0], ',', 2)); //T21 - log
            FErrorLogger.Error(#9 + 'Storage Copy' + #9 + 'Continued, RPC - ' + MagPiece(MAGR[0], ',', 2) + ', ' + FUT49Mark+', '+Fname+' ('+UnitName+'.'+ProcName+'.15)');
            SetCurIEN(curIEN,Strtoint(MagPiece(MagPiece(MAGR[0],'(', 2),'~',1))); //broker timeout, continued
        End;
        If curIEN < 0 Then {Error occured, log and continue next ien}
        Begin
          If FUT49Running then errorcnt := errorcnt + 1;
          UpdateLabel(lblErrorCnt,errorcnt);
          curIEN := Strtoint(MagPiece(FUT49Mark, ':', 1));
          FLogger.Error(#9 + 'Storage Copy' + #9 + 'Error occurred, RPC - ' + MagPiece(MAGR[0], ',', 1)); //T21 - log
          FErrorLogger.Error(#9 + 'Storage Copy' + #9 + 'Error occurred, RPC - ' + MagPiece(MAGR[0], ',', 1) + ', ' + FUT49Mark+', '+Fname+' ('+UnitName+'.'+ProcName+'.15)');
        End;
        curIEN := curIEN - 1; //process next ien
        If curIEN < IEN2 Then //complete ien range
        Begin
          If FUT49Running Then
          Begin
            UpdateLabel(lblErrorCnt,errorcnt);
            lbltotalcnt.caption := Inttostr(OKCNT);
            Application.Processmessages;
            If OKCNT > 0 Then
            Begin
              If (doneMsg = '') and FUT49Running Then
              Begin
                If errorcnt = 0 Then
                  Showmessage('All requested IENs have been copied' + #13 + 'Total (' + Inttostr(OKCNT) + ') copied, (' + Inttostr(errorcnt) + ') not copied, see log file for details')
                Else
                  Showmessage('All requested IENs have been copied' + #13 + 'Total (' + Inttostr(OKCNT) + ') copied, (' + Inttostr(errorcnt) + ') not copied.'
                    + #13 + #13 + 'Some files may not be copied, see log file for details');
                doneMsg := '1';
                FUT49Running := False;
                FLogger.Info(#9 + 'Storage Copy' + #9 +#9 + 'Completed copying files for requested IEN or date range - ' + Inttostr(OKCNT) + ' IENs copied, ' + Inttostr(errorcnt) + ' IENs not copied.'); //T21- log, t17 - IR T16-05 log summary
                FErrorLogger.Info(#9 + 'Storage Copy' + #9 +#9 + 'Completed copying files for requested IEN or date range - ' + Inttostr(OKCNT) + ' IENs copied, ' + Inttostr(errorcnt) + ' IENs not copied.'); //T21- log, T18
              End;
            End
            Else
              If doneMsg = '' Then
              Begin
                Showmessage('Storage Copy completed. No files were copied.' + #13 +#10
                            + 'See log and check patch documentation for troubleshooting suggestions.'); //t17 - IR 15-34
                FLogger.Info(#9 + 'Storage copy' + #9 + 'Task is complete'+#9+'No files were copied.'+#9+ 'No files exist at source or file not found');
                FErrorLogger.Info(#9 + 'Storage copy' + #9 + 'Task is complete'+#9+'No files were copied.'+#9+ 'No files exist at source or file not found');
                doneMsg := '1';
                FUT49Running := False;
              End;
          End;
          if (not FUT49Running) and (FUT49Mark= '') then   //t17 - Cancel task, give log summary
          Begin
              FLogger.Info(#9 + 'Storage Copy' + #9 +#9 + 'Canceled copying files for requested IEN or date range - ' + Inttostr(OKCNT) + ' IENs copied, ' + Inttostr(errorcnt) + ' IENs not copied.'); //T21 - log, T18 //t17 - log summary
              FLogger.Info(#9 + 'Storage Copy' + #9 + 'The task canceled at ' + TimeToStr(Time) + ' by user'+ MagUtilityBroker.UserFullName); //t17
              FErrorLogger.Info(#9 + 'Storage Copy' + #9 +#9 + 'Canceled copying files for requested IEN or date range -' + Inttostr(OKCNT) + ' IENs copied, ' + Inttostr(errorcnt) + ' IENs not copied.'); //TT21 - log, T18
              FErrorLogger.Info(#9 + 'Storage Copy' + #9 + 'The task canceled at ' + TimeToStr(Time) + ' by user'+ MagUtilityBroker.UserFullName); //t17
          End;
          cbxFromShareLoc.Items.Clear;
          cbxToShareLoc.Items.Clear;
          btnProcessFUT49.Enabled := True;
          FUT49Running := False;
        End; //of If curIEN < IEN2
        UpdateLabel(lblErrorCnt,errorcnt);
        lbltotalcnt.caption := Inttostr(OKCNT);
        IEN0 := Inttostr(curIEN) + ':' + Inttostr(IEN2);
        FUT49Mark := IEN0 + '^' + NET0; //for resume/restart point mark
      End; //RPC success
     Finally
       FreeAndNil(MAGR);
     end; //try  T24 - memory release
    End; //while IEN2>curIEN   (in IEN1:IEN2 range)
    // reset all
    FUT49Running := False;
    SetCaptionFut49byDt4ien('');
    SetCaptionFUT49byDt2ien('');
    SetCaptionFut49byIEN4DT('');
    SetCaptionFut49byIEN2DT('');
    lblcurrentfile.caption := '';
    lblIEN.caption := '';
    cbxFromShareLoc.ClearSelection;
    cbxFromShareLoc.Items.CleanupInstance;
    cbxToShareLoc.ClearSelection;
    cbxToShareLoc.Items.CleanupInstance;
    btnProcessFUT49.Enabled := False;
    BtnPauseFUT49.Enabled := False;
    AdjustEnabled(gbcbxShareLoc, False);
    Application.Processmessages;
  Finally
    FUT49AftHour.Enabled  := boFUT49AftHourEnabled;
    FUT49RemoveOld.Enabled:= boFUT49RemoveOldEnabled;
    FUT49SetNewLoc.Enabled:= boFUT49SetNewLocEnabled;
    rgFut49ChkType.Enabled:= borgFut49ChkTypeEnabled;
    If NOT FUT49RemoveOld.Checked then FUT49SetNewLoc.Enabled := True; //T20 - CodeCR239
  End;
End;

Procedure TfMagUtility.MnuHelpAboutClick(Sender: Tobject);
Var
  Rstat: Boolean;
  Rlist: Tstringlist;
  Magver, Vstat: String; //98
  ProcName: String;
Begin
  ProcName:='TfMagUtility.MnuHelpAboutClick';
  Rlist := Tstringlist.Create;
  Try
    MagUtilityBroker.RPMaggInstall(Rstat, Rlist);
    {p98 Display Version Status : Alpha/Beta or Released}
    Magver := MagGetFileVersionInfo(Application.ExeName); //98
    MagUtilityBroker.RPMagVersionStatus(Vstat, Magver); //98
    FrmAbout.Execute('', '', Rlist, MagPiece(Vstat, '^', 2)); //98
  Finally
    Rlist.Free;
  End;
End;

(*
  Execute a command line and wait until it has finished. (Storage Copy)
*)

Function TfMagUtility.ExecuteShellCommand(CmdLine: String; hidden: Boolean): Boolean;
Const
  Flags: Array[Boolean] Of Integer = (SW_SHOWNORMAL, SW_HIDE);
Var
  AccessDT  : Tdatetime;
  CreateDT  : Tdatetime;
  fileDest  : String;
  fileDir   : String;
  fileSource: String;
  ModifyDT  : Tdatetime;
  n         : Integer;
  nFileCount: Integer;
  ProcName  : String;
  s         : String;
  sgEMsg    : String;
  Sr        : TSearchRec;
  SR1       : TSearchRec;
Begin
  ProcName:='TfMagUtility.ExecuteShellCommand';
  Result:=True;
  Try
    If FProcessingPaused And (Not boInRetryLoop) Then
    Begin
      FLogger.Error(#9 + 'Storage Copy' + #9 + 'The task paused at ' + TimeToStr(Time) + ' by user: '+ MagUtilityBroker.UserFullName); //T21
      FErrorLogger.Error(#9 + 'Storage Copy' + #9 + 'The task paused at ' + TimeToStr(Time) + ' by user: '+ MagUtilityBroker.UserFullName + 'FProcessingPaused=True; boInRetryLoop=False;CmdLine=' + CmdLine+'; Result:= True'); //T21
      Exit;
    End;
    // step 1 - check DIR exist
    If ansicontainsStr(CmdLine, 'MKDIR ') Then
    Begin
      filesource := MagPiece(CmdLine, ' ', 2);
      If Directoryexists(fileSource) = False Then
      Begin
        Mkdir(fileSource);
        If (GetLastError=112) Or (GetLastError=39) then
        Begin
          If ProblemIndex=0 Then
          Begin
              showmessage('There is not enough space on destination - The disk is full');
              FLogger.Error(#9 + 'Storage Copy' +#9 + 'Failed Execute Shell Command, ' + #9 + 'there is not enough space on destination - The disk is full');
              FErrorLogger.Error(#9 + 'Storage Copy' +#9 + 'Failed Execute Shell Command, ' + #9 + 'there is not enough space on destination - The disk is full'+#9+' ('+UnitName+'.'+ProcName+'.1)');
          End;
          ProblemIndex:=112; //Disk is full
          Result:=False;
          Exit;
        End
      End
    End
    // step 2 - check 'MOVE' or 'COPY/v '
    Else
      If ansicontainsStr(CmdLine, 'MOVE/Y ') Then ///Move file from soure to dest
      Begin //cmdline = 'move source dest'
        filesource := MagPiece(CmdLine, ' ', 2);
        fileDest := MagPiece(CmdLine, ' ', 3);
        fileDir := MagPiece(fileSource, '\', 1);
        n := 2;
        Repeat
          Begin
            s := MagPiece(fileSource, '\', n);
            If Not ansicontainsStr(s, '.') Then fileDir := fileDir + '\' + s;
            Inc(n);
          End;
        Until ansicontainsStr(s, '.');
        s := fileDir + '\*.*'; //search any files
        If FindFirst(fileSource, FaAnyFile, Sr) = 0 Then
        Begin
          {$WARNINGS OFF}
          CreateDT := FileTimeToDTime(Sr.Finddata.FtCreationTime);
          AccessDT := FileTimeToDTime(Sr.Finddata.FtLastAccessTime);
          ModifyDT := FileTimeToDTime(Sr.Finddata.ftLastWriteTime);
          {$WARNINGS ON}
          FindClose(Sr);
          If MoveFile(PChar(filesource), PChar(fileDest)) Then
          Begin
            If showdetail=1 then  //T20 - CodeCR229
            Begin
            sgEMsg:=#9 + 'Storage Copy' + #9 + 'Converted and copied the file to a 14.3 file name format.' + #9 + 'The file-rename succeeded: filesource='+ filesource +'; fileDest='+fileDest;
            FLogger.Info(sgEMsg);
            FErrorLogger.Info(sgEMsg +' ('+UnitName+'.'+ProcName+'.54)'); //T18 - Agenda #6_4 , no report
            End
          End
          Else
          Begin
            sgEMsg:=#9 + 'Storage Copy' + #9 + 'MoveFile failed: filesource='+filesource+'; fileDest='+fileDest;
            FLogger.Error(sgEMsg);
            FErrorLogger.Error(sgEMsg+' ('+UnitName+'.'+ProcName+'.55)');
            If (GetLastError=112) Or (GetLastError=39) then   //T22 - Disk full
            Begin
              If ProblemIndex=0 Then
              Begin
                  showmessage('There is not enough space on destination - The disk is full');
                  FLogger.Error(#9 + 'Storage Copy' +#9 + 'Failed Execute Shell Command, ' + #9 + 'there is not enough space on destination - The disk is full');
                  FErrorLogger.Error(#9 + 'Storage Copy' +#9 + 'Failed Execute Shell Command, ' + #9 + 'there is not enough space on destination - The disk is full'+#9+' ('+UnitName+'.'+ProcName+'.1)');
              End;
              ProblemIndex:=112; //Disk is full
              Result:=False;
              Exit;
            End;
            ProblemIndex:=1; //Move file failed:1
            //Result:=False;
          End;
          If FindFirst(fileDest, FaAnyFile, SR1) = 0 Then
          Begin
            If SetFileDates(fileDest, CreateDT, AccessDT, ModifyDT) Then
            Begin
              //T18 - No report //sgEMsg:=#9 + 'Storage Copy' + #9 + 'SetFileDates succeeded: fileDest='+fileDest+'; CmdLine='+CmdLine+' ('+UnitName+'.'+ProcName+'.58)';
              //FLogger.Info(sgEMsg);
              //FErrorLogger.Info(sgEMsg);
            End
            Else
            Begin
              If FUT49Running then errorcnt := errorcnt + 1;
              sgEMsg:=#9 + 'Storage Copy' + #9 + 'Set File Date failed: fileDest='+fileDest+'; CreateDT='+FormatDateTime('MM/DD/YY hh:nn:ss',CreateDT)+'; AccessDT='+FormatDateTime('MM/DD/YY hh:nn:ss',AccessDT)+'; ModifyDT='+FormatDateTime('MM/DD/YY hh:nn:ss',ModifyDT)+'; CmdLine='+CmdLine;
              FLogger.Error(sgEMsg);
              FErrorLogger.Error(sgEMsg+' ('+UnitName+'.'+ProcName+'.59)');
              ProblemIndex:=2; //SetFileDates failed:2
              //Result:=False;
              //Exit;
            End;
            FindClose(SR1);
          End;
        End
        Else  //can't find source file to move
        Begin
          If Not Directoryexists(ExtractFilePath(fileSource)) Then
          Begin
            If FUT49Running then errorcnt := errorcnt + 1;
            sgEMsg:=#9 + 'Storage Copy' + #9 + 'Could not find the source file or directory ' + fileSource;
            FLogger.Error(sgEMsg);
            FErrorLogger.Error(sgEMsg+' ('+UnitName+'.'+ProcName+'.51)');
            ProblemIndex:=3; //can't find file:3
            //Result:=False;
            Exit;
          End
          Else
          Begin
            sgEMsg:=#9 + 'Storage Copy' + #9 + 'Could not find the source file ' + fileSource + ', skipped the Copy.';
            FLogger.Error(sgEMsg);
            FErrorLogger.Error(sgEMsg + ' ('+UnitName+'.'+ProcName+'.52)');
            ProblemIndex:=3; //can't find file:3
            //Result:=True;
            Exit;
          End;
        End;
        //MoveFile();end chk
        If FindFirst(fileSource, FaAnyFile, Sr) = 0 Then //source still there
        Begin //MOVE can't overwrite exist
          FindClose(Sr);
          If FUT49Running then errorcnt := errorcnt + 1;
          problemIndex := 5; //File exist already:5
          sgEMsg:=#9 + 'Storage Copy' + #9 + fileDest + ' same file already exist at destination, could not move the file ' + fileSource;
          FLogger.Error(sgEMsg);
          FErrorLogger.Error(sgEMsg);
          //Result:=True;
          Exit;
        End
        Else //source moved  clean up empty dir
        Begin
          FindClose(Sr);
          nFileCount := 0;
          If FindFirst(s, FaAnyFile, Sr) = 0 Then
          Begin
            Repeat
              If ((Sr.Attr And Fadirectory) <> Fadirectory) Then Inc(nFileCount);
            Until FindNext(Sr) <> 0;
            FindClose(Sr);
            If nFileCount = 0 Then // if folder is empty then remove it
            Begin
              If RemoveDir(PChar(fileDir)) Then
              Begin
                If showdetail=1 then    //T20 - CodeCR 229
                Begin
                  sgEMsg:=#9 + 'Storage Copy' + #9 + fileDir + ' directory was removed because it was empty';
                  FLogger.Info(sgEMsg);
                End;
                FErrorLogger.Info(sgEMsg);
              End
              Else
              Begin
                sgEMsg:=#9 + 'Storage Copy' + #9 + fileDir + ' directory is empty, however it could not be removed';
                FLogger.Error(sgEMsg);
                FErrorLogger.Error(sgEMsg);
              End;
            End;
          End;
        End;
      End // 'MOVE/Y ' file completed
      Else  //'Copy' Task
        If ansicontainsStr(CmdLine, 'COPY/Y /V') Then
        Begin
          fileSource := MagPiece(CmdLine, ' ', 3);
          fileDest := MagPiece(CmdLine, ' ', 4);
          If FindFirst(fileSource, FaAnyFile, Sr) = 0 Then
          Begin
            {$WARN SYMBOL_PLATFORM OFF}
            CreateDT := FileTimeToDTime(Sr.Finddata.FtCreationTime);
            AccessDT := FileTimeToDTime(Sr.Finddata.FtLastAccessTime);
            ModifyDT := FileTimeToDTime(Sr.Finddata.ftLastWriteTime);
            {$WARN SYMBOL_PLATFORM ON}
            FindClose(Sr);
            If CopyFile(PChar(fileSource), PChar(fileDest), False) Then
            Begin
              If showdetail=1 then   //T20- codeCR 229
              Begin
                sgEMsg:=#9 + 'Storage Copy' + #9 + 'CopyFile succeeded; fileSource='+ fileSource+'; fileDest='+fileDest+'; CmdLine='+CmdLine;
                FLogger.Info(sgEMsg);
                FErrorLogger.Info(sgEMsg);
              End;
            End
            Else
            Begin
              If FUT49Running then errorcnt := errorcnt + 1;
              sgEMsg:=#9 + 'Storage Copy' + #9 + 'Copy File failed; fileSource='+ fileSource+'; fileDest='+fileDest+'; CmdLine='+CmdLine;
              FLogger.Error(sgEMsg);
              FErrorLogger.Error(sgEMsg+' ('+UnitName+'.'+ProcName+'.52)');
              If (GetLastError=112) Or (GetLastError=39) then
              Begin
                  If ProblemIndex=0 Then
                  Begin
                      showmessage('There is not enough space on destination - The disk is full');
                      FLogger.Error(#9 + 'Storage Copy' +#9 + 'Failed Execute Shell Command, ' + #9 + 'there is not enough space on destination - The disk is full');
                      FErrorLogger.Error(#9 + 'Storage Copy' +#9 + 'Failed Execute Shell Command, ' + #9 + 'there is not enough space on destination - The disk is full'+#9+' ('+UnitName+'.'+ProcName+'.1)');
                  End;
                  ProblemIndex:=112; //Disk is full
                  Result:=False;
                  Exit;
              End
              Else ProblemIndex:=4;//Copy failed:4
              //Result:=False;
              Exit;
            End;
            If SetFileDates(fileDest, CreateDT, AccessDT, ModifyDT) Then
            Begin
              //T18 - No report , sgEMsg:=#9 + 'Storage Copy' + #9 + 'SetFileDates succeeded: fileDest='+fileDest+'; CmdLine='+CmdLine;
              //T18 - Agenda #6_4, FLogger.Info(sgEMsg);
              //T18 - FErrorLogger.Info(sgEMsg+' ('+UnitName+'.'+ProcName+'.60)');
            End
            Else
            Begin
              If FUT49Running then errorcnt := errorcnt + 1;
              sgEMsg:=#9 + 'Storage Copy' + #9 + 'Set File Date failed: fileDest='+fileDest+'; CreateDT='+FormatDateTime('MM/DD/YY hh:nn:ss',CreateDT)+'; AccessDT='+FormatDateTime('MM/DD/YY hh:nn:ss',AccessDT)+'; ModifyDT='+FormatDateTime('MM/DD/YY hh:nn:ss',ModifyDT)+'; CmdLine='+CmdLine;
              FLogger.Error(sgEMsg);
              FErrorLogger.Error(sgEMsg+' ('+UnitName+'.'+ProcName+'.61)');
              ProblemIndex:=2;  //Set file date failed:2
              //Result:=False;
              //Exit;
            End;
            If FindFirst(fileDest, FaAnyFile, SR1) = 0 Then
            Begin
              FindClose(SR1);
            End;
          End
          Else
          Begin
            If Not Directoryexists(ExtractFilePath(fileSource)) Then
            Begin
              If FUT49Running then errorcnt := errorcnt + 1;
              sgEMsg:=#9 + 'Storage Copy' + #9 + 'Could not find the source file or directory' + fileSource;
              FLogger.Error(sgEMsg);
              FErrorLogger.Error(sgEMsg+' ('+UnitName+'.'+ProcName+'.44)');
              ProblemIndex:=3; //File not found:4
              //Result:=False;
              Exit;
            End
            Else
            Begin
              sgEMsg:=#9 + 'Storage Copy' + #9 + 'Could not find the source file ' + fileSource + ', skipped the Copy.';
              FLogger.Error(sgEMsg);
              FErrorLogger.Error(sgEMsg+' ('+UnitName+'.'+ProcName+'.45)');
              ProblemIndex:=3;
              //Result:=True;
              Exit;
            End;
          End;
        End
        Else  //other command other than COPY/MOVE
        Begin
          WinExec(PChar(CmdLine), 0);
          If showdetail=1 then  //T20- CodeCR229
          Begin
            sgEMsg:=#9 + 'Storage Copy' + #9 + 'WinExec: CmdLine=' + CmdLine;
            FLogger.Info(sgEMsg);
            FErrorLogger.Info(sgEMsg+' ('+UnitName+'.'+ProcName+'.61)');
          End;
        End;
  Except
    On e:Exception Do
    Begin
      sgEMsg:=#9 + 'Storage Copy' +#9 + 'Failed Execute Shell Command: ' + #9 + e.Message +' - '+ SysErrorMessage(GetLastError)+ #9 + 'CommandLine='+CmdLine;
      FLogger.Error(sgEMsg);
      FErrorLogger.Error(sgEMsg+' ('+UnitName+'.'+ProcName+'.16)');
      if AnsiContainsStr(e.Message, 'error 112') or AnsiContainsStr(e.Message, 'error 39') then //T21 - Cr 300 disk full
      Begin
           If ProblemIndex=0 Then showmessage(e.Message+#13+ SysErrorMessage(GetLastError)); //T21 -Cr 300
           ProblemIndex:=112; //Disk full
      End
      Else ProblemIndex:=99; //Unknown error:99
      Result:=False;
    End;
  End;
End;

Function TfMagUtility.IsStrANumber(Const s: String): Boolean;
Var
  p: PChar;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.IsStrANumber';
  p := PChar(s);
  Result := False;
  While p^ <> #0 Do
  Begin
    If Not (p^ In ['0'..'9']) Then Exit;
    Inc(p);
  End;
  Result := True;
End;

Function TfMagUtility.FileLastModified(Const TheFile: String): String;
Var
  FileH: THandle;
  LocalFT: TFileTime;
  DosFT: DWORD;
  LastAccessedTime: Tdatetime;
  Finddata: TWin32FindData;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.FileLastModified';
  Result := '';
  FileH := FindFirstFile(PChar(TheFile), Finddata);
  If FileH <> INVALID_HANDLE_VALUE Then
  Begin
    Windows.FindClose(Handle);
    If (Finddata.dwFileAttributes And
      FILE_ATTRIBUTE_DIRECTORY) = 0 Then
    Begin
      FileTimeToLocalFileTime
        (Finddata.ftLastWriteTime, LocalFT);
      FileTimeToDosDateTime
        (LocalFT, LongRec(DosFT).Hi, LongRec(DosFT).Lo);
      LastAccessedTime := FileDateToDateTime(DosFT);
      Result := DateTimeToStr(LastAccessedTime);
    End;
  End;
End;

Procedure TfMagUtility.FormResize(Sender: Tobject);
Var
  ProcName: String;
Begin
  ProcName:='TfMagUtility.FormResize';
  ORaidListView.Width := fMagUtility.Width * 2 Div 3;
  Splitter1.Left := ORaidListView.Left + ORaidListView.Width + 1;
  Panel1.Left := Splitter1.Left + Splitter1.Width + 1;
  ArrangeStatusLabels();
  pnlRetry.Top:=(grpProcessStatus.Height-pnlRetry.Height) div 2;
  pnlRetry.Left:=(grpProcessStatus.Width-pnlRetry.Width) div 2;
  If ORaidListView.Columns.Count>1 Then SetListViewWidth(ORaidListView);
  If NetClnChkListView.Columns.Count>1 Then SetListViewWidth(NetClnChkListView);
End;

Function TfMagUtility.FileTimeToDTime(FTime: TFileTime): Tdatetime;
Var
  LocalFTime: TFileTime;
  STime: TSystemTime;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.FileTimeToDTime';
  FileTimeToLocalFileTime(FTime, LocalFTime);
  FileTimeToSystemTime(LocalFTime, STime);
  Result := SystemTimeToDateTime(STime);
End;

Function TfMagUtility.SetFileDates(Const Filename: String;
  Const CreationDate: Tdatetime;
  Const AccessedDate: Tdatetime;
  Const ModifiedDate: Tdatetime): Boolean;
Var
  FileHandle: THandle;
  FILEDATETIME: TFileTime;
  TimeInt: Integer;
  TimeLocal: TFileTime;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.SetFileDates';
  FileHandle := 0;
  Result := False;
  Try
    Try
      FileHandle := FileOpen(Filename, FmOpenWrite Or fmShareDenyNone);
      If FileHandle > 0 Then
      Begin
        TimeInt := DateTimeToFileDate(CreationDate);
        If DosDateTimeToFileTime(LongRec(TimeInt).Hi, LongRec(TimeInt).Lo, TimeLocal) Then
        Begin
          If Windows.LocalFileTimeToFileTime(TimeLocal, FILEDATETIME) Then
          Begin
            SetFileTime(FileHandle, @FILEDATETIME, Nil, Nil);
            Result := True;
          End;
        End;
        TimeInt := DateTimeToFileDate(AccessedDate);
        If DosDateTimeToFileTime(LongRec(TimeInt).Hi, LongRec(TimeInt).Lo, TimeLocal) Then
        Begin
          If Windows.LocalFileTimeToFileTime(TimeLocal, FILEDATETIME) Then
          Begin
            SetFileTime(FileHandle, Nil, @FILEDATETIME, Nil);
            Result := True;
          End;
        End;
        TimeInt := DateTimeToFileDate(ModifiedDate);
        If DosDateTimeToFileTime(LongRec(TimeInt).Hi, LongRec(TimeInt).Lo, TimeLocal) Then
        Begin
          If Windows.LocalFileTimeToFileTime(TimeLocal, FILEDATETIME) Then
          Begin
            SetFileTime(FileHandle, Nil, Nil, @FILEDATETIME);
            Result := True;
          End;
        End;
      End;
    Except
      Result := False;
    End;
  Finally
    FileClose(FileHandle);
  End;
End;

Procedure TfMagUtility.btnHideClick(Sender: Tobject);
Var
  ProcName: String;
Begin
  ProcName:='TfMagUtility.btnHideClick';
  If btnHide.caption = 'Hide processing detail' Then
  Begin
    grpProcessStatus.Visible := False;
    btnHide.caption := 'Show processing detail';
  End
  Else
  Begin
    grpProcessStatus.Visible := True;
    btnHide.caption := 'Hide processing detail';
  End;
End;

Procedure TfMagUtility.FUT49SetNewLocClick(Sender: Tobject);
Var
  ProcName: String;
Begin
  ProcName:='TfMagUtility.FUT49SetNewLocClick';
  StorageCopyWarning(Sender);
End;

Procedure TfMagUtility.actStorageByDateFromExecute(Sender: Tobject);
Var
  Date: Tdatetime;
  EndDate: Tdatetime;
  sgEndDate: String;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.actStorageByDateFromExecute';
  If FUT49Running Then Exit;
  If rgFut49ChkType.ItemIndex<>0 Then Exit;
  sgEndDate := Trim(EdtEndDT.Text);
  EndDate := 0;
  If sgEndDate <> '' Then EndDate := Strtodatetime(sgEndDate);
  Date := Now()-1; //t17
  If Trim(edtStartDT.Text) <> '' Then
  Begin
    Date := Strtodatetime(edtStartDT.Text);
  End;
  If frmMagCalendar.Execute(Date, 'From Date') Then
  Begin
    SetCaptionFut49byDt4ien('');
    If FormatDateTime('mm/dd/yyyy',Date)=FormatDateTime('mm/dd/yyyy',Now()) Then
    Begin
      Showmessage('Images from today will not be copied');
      InitStorageCopyPanelByDate(True);
      AdjustEnabled(gbcbxShareLoc, False);
      EnableRunButtonStorageCopy();
      MakeFocused(btnSel4Dt);
      Exit;
    End;
    If (Date > Now()) Then
    Begin
      Showmessage('The From date cannot be in the future.');
      InitStorageCopyPanelByDate(True);
      AdjustEnabled(gbcbxShareLoc, False);
      EnableRunButtonStorageCopy();
      MakeFocused(btnSel4Dt);
      Exit;
    End;
    If Date > EndDate Then
    Begin
      If EndDate <> 0 Then
      Begin
        Showmessage('The From date cannot be greater than the To Date.');
        InitStorageCopyPanelByDate(True);
        AdjustEnabled(gbcbxShareLoc, False);
        EnableRunButtonStorageCopy();
        MakeFocused(btnSel2Dt);
        Exit;
      End;
    End;
    edtStartDT.Text := Formatdatetime('MM/DD/YYYY', Date);
    MakeFocused(btnSel2Dt);
    edtStartDTonExit(Sender);
  End;
End;

Procedure TfMagUtility.actStorageByDateToExecute(Sender: Tobject);
Var
  boRejected : Boolean;
  Date       : Tdatetime;
  sgStartDate: String;
  StartDate  : Tdatetime;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.actStorageByDateToExecute';
  If FUT49Running Then Exit;
  If rgFut49ChkType.ItemIndex<>0 Then Exit;
  AdjustEnabled(gbcbxShareLoc, False);
  sgStartDate := Trim(edtStartDT.Text);
  If sgStartDate = '' Then
  Begin
    Showmessage('Please populate the From Date first.');
    EdtEndDT.Text := '';
    EnableRunButtonStorageCopy();
    Exit;
  End;
  StartDate := Strtodatetime(sgStartDate);
  Date := Now()-1; //t17 - no today as default
  If Trim(EdtEndDT.Text) <> '' Then
  Begin
    Date := Strtodatetime(EdtEndDT.Text);
  End;
  If frmMagCalendar.Execute(Date, 'To Date') Then
  Begin
    boRejected := False;
    SetCaptionFUT49byDt2ien('');
    //t17 IR T16-6 allow same day
   {* If Date = StartDate Then
    Begin
      Showmessage('The To Date and From Date cannot be the same.');
      boRejected := True;
    End
    Else
    }
    If Date < StartDate Then
    Begin
      Showmessage('The To Date must be greater than the From Date.');
      boRejected := True;
    End
    Else
    If Date > Now() Then
    Begin
      Showmessage('The To Date cannot be in the future.');
      boRejected := True;
    End;
    If boRejected Then
    Begin
      EdtEndDT.Text := '';
      EnableRunButtonStorageCopy();
      Exit;
    End;
    EdtEndDT.Text := Formatdatetime('MM/DD/YYYY', Date);
    EnableRunButtonStorageCopy();
    If Not EndDTonexit(Sender) Then
    Begin
      EnableRunButtonStorageCopy();
      Exit;
    End;
  End
  Else
    Exit; //t17 - cancel from TO Date entry
  AdjustEnabled(gbcbxShareLoc, True);
  EnableRunButtonStorageCopy();
End;

Procedure TfMagUtility.StorageCopyEnabling(Sender: Tobject);
Var
  ProcName: String;
Begin
  ProcName:='TfMagUtility.StorageCopyEnabling';
  StorageCopyWarning(Sender);
  AdjustEnabled(pnlByDate, False);
  AdjustEnabled(pnlByIEN, False);
  AdjustEnabled(gbcbxShareLoc, False);
  cbxFromShareLoc.Items.Clear;
  cbxFromShareLoc.TabStop := False;
  cbxFromShareLoc.Text := '';
  cbxToShareLoc.Items.Clear;
  cbxToShareLoc.TabStop := False;
  cbxToShareLoc.Text := '';
  cbxFromShareLoc.OnExit := Nil;
  cbxToShareLoc.OnExit := Nil;
  edtEndDT.Text := '';
  edtStartDT.Text := '';
  edtStartIEN.Text := '';
  edtEndIEN.Text := '';
  SetCaptionFut49byDt4ien('');
  SetCaptionFUT49byDt2ien('');
  SetCaptionFut49byIEN4DT('');
  SetCaptionFut49byIEN2DT('');

  If rgFut49ChkType.ItemIndex=0 Then
  Begin
    edtStartDT.Text := '';
    EdtEndDT.Text := '';
    SetCaptionFut49byDt4ien('');
    SetCaptionFUT49byDt2ien('');
    AdjustEnabled(pnlByDate, True);
    edtStartDT.Enabled := True;
    edtStartDT.Color := clBtnFace;
    edtStartDT.TabStop := False;
    edtEndDT.Enabled := True;
    edtEndDT.Color := clBtnFace;
    edtEndDT.TabStop := False;
    btnSel4Dt.Enabled := True;
    btnSel2Dt.Enabled := True;
    MakeFocused(btnSel4Dt);
  End;
  If rgFut49ChkType.ItemIndex=1 Then
  Begin
    edtStartIEN.Text := '';
    edtEndIEN.Text := '';
    SetCaptionFut49byIEN4DT('(e.g.: 12345, `*`)');
    SetCaptionFut49byIEN2DT('');
    AdjustEnabled(pnlByIEN, True);
    pnlByIEN.Enabled := True;
    edtStartIEN.Enabled := True;
    MakeFocused(edtStartIEN);
  End;
  FUT49RemoveOld.Enabled := True;
  EnableRunButtonStorageCopy();
  Application.Processmessages();
End;

Procedure TfMagUtility.MakeFocused(Sender: TWinControl);
Var
  ProcName: String;
Begin
  ProcName:='TfMagUtility.MakeFocused';
  Application.ProcessMessages();
  If Not Active Then Exit;
  Try
    If Sender.Visible Then
    Begin
      If Sender.Enabled Then
      Begin
        Try
        Sender.SetFocus();
        ActiveControl := Sender;
        Except
          On e: Exception Do
          Begin
            (*
            sgError := 'Name='+Sender.Name+#13+#10;
            sgError := sgError+'Class='+Sender.ClassName+#13+#10;
            If Sender.Visible Then
              sgError := sgError+'Visible=True'+#13+#10
            Else
              sgError := sgError+'Visible=False'+#13+#10;
            If Sender.Enabled Then
              sgError := sgError+'Enabled=True'+#13+#10
            Else
              sgError := sgError+'Enabled=False'+#13+#10;
            If Sender.Parent.Visible Then
              sgError := sgError+'Parent.Visible=True'+#13+#10
            Else
              sgError := sgError+'Parent.Visible=False'+#13+#10;
            If Sender.Parent.Enabled Then
              sgError := sgError+'Parent.Enabled=True'+#13+#10
            Else
              sgError := sgError+'Parent.Enabled=False'+#13+#10;
            sgError := sgError+'error='+e.Message+#13+#10;
            ShowMessage(sgError);
            *)
          End;
        End;
      End;
    End;
  Except
  End;
  Application.ProcessMessages();
End;

Procedure TfMagUtility.AdjustEnabled(Sender: TWinControl;
  Enabled: Boolean);
Var
  i: Integer;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.AdjustEnabled';
  Sender.Enabled := Enabled;
  For i := 0 To Sender.ControlCount - 1 Do
  Begin
    Sender.Controls[i].Enabled := Enabled;
    If Sender.Controls[i] Is TLabeledEdit Then
    Begin
      If Enabled Then
      Begin
        TLabeledEdit(Sender.Controls[i]).Color := clWindow;
      End
      Else
      Begin
        TLabeledEdit(Sender.Controls[i]).Color := clBtnFace;
      End;
    End;
    If Sender.Controls[i] Is TComboBox Then
    Begin
      If Enabled Then
      Begin
        TComboBox(Sender.Controls[i]).Color := clWindow;
      End
      Else
      Begin
        TComboBox(Sender.Controls[i]).Color := clBtnFace;
      End;
    End;
    If Sender.Controls[i] is TWinControl Then AdjustEnabled((Sender.Controls[i] As TWinControl),Enabled);
  End;
End;

Function TfMagUtility.EndDTonexit(Sender: Tobject): Boolean;
Var
  chkvar: String;
  dt0: Integer;
  dt1: Integer;
  DTRange: String;
  i: Integer;
  IEN1 :Int64;
  IEN2 :Int64;
  Ien: String;
  iplace: String;
  limit: String;
  Magien: String;
  OnLine: String;
  route: String;
  share: String;
  Shares: TStrings;
  SName: String;
  Stype: String;
  xline: String;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.EndDTonexit';
  Result := False;
  Try
    btnPauseFUT49.Enabled := False;
    DTRange := edtEndDT.Text;
    If DTRange = '' Then
    Begin
      SetCaptionFUT49byDt2ien('');
      Exit;
    End;
    If edtStartDT.Text = '' Then
    Begin
      Showmessage('Error: Must select a From date, click then select a date');
      edtEndDT.Text := '';
      MakeFocused(btnSel4Dt);
      Exit;
    End;
    limit := '2'; //end date
    Magien := MagUtilityBroker.DTRangeIEN(DTRange, limit);
    edtEndDT.Text := DtRange;
    If (MagPiece(Magien, ',', 1) = '-1') Or (Magien = '0') Or (Strtoint(Magien) < 1) Then
    Begin
      Showmessage('Invalid date selected (' + edtEndDT.Text + ') - ' + MagPiece(Magien, ',', 2));
      edtEndDT.Text := '';
      SetCaptionFUT49byDt2ien(''); //t17 - clk to sel only
      MakeFocused(btnSel2Dt);
      MakeFocused(btnSel2Dt);
      Exit;
    End;
    If edtStartDT.Text <> '' Then //chk DT range
    Begin
      dt0 := Strtoint(MagPiece(edtStartDT.Text, '/', 3)) * 10000 +
        Strtoint(MagPiece(edtStartDT.Text, '/', 1)) * 100 +
        Strtoint(MagPiece(edtStartDT.Text, '/', 2));
      dt1 := Strtoint(MagPiece(DtRange, '/', 3)) * 10000 +
        Strtoint(MagPiece(DtRange, '/', 1)) * 100 +
        Strtoint(MagPiece(DtRange, '/', 2));
      If dt0 > dt1 Then //From dt should older than To date
      Begin
        Showmessage('Error: Invalid date, From date must be earlier than To date');
        edtEndDT.Text := '';
        SetCaptionFUT49byDt2ien('');
        MakeFocused(btnSel2Dt);
        Exit
      End;
    End; // chk DT range
    if edtStartDT.EditLabel.Caption<>'' then
    Begin
        Ien := MagPiece(MagPiece(edtStartDT.EditLabel.Caption, '=', 2),' ',1);
        IEN1:=strtoint64(Ien); //begin DT ien  ->>
        IEN2:=strtoint64(Magien); //end DT ien  <<-
        if (IEN1>0) and (IEN2<IEN1) then      //out of ien DT order
        Begin
            Showmessage('Error: No image IEN in selected From/To date range');
            edtEndDT.Text := '';
            SetCaptionFUT49byDt2ien('');
            MakeFocused(btnSel2Dt);
            Exit;
        end;
    End; // chk DT vs. ien order
    SetCaptionFUT49byDt2ien('(approx. IEN=' + Magien + ' )');
    edtEndDT.Text := DTRange;
    cbxFromShareLoc.Items.Clear;
    If MagUtilityBroker.GetNetworkLocations(Shares) Then
    Begin
      For i := 0 To Shares.Count - 1 Do
      Begin
        xline := Uppercase(Shares[i]);
        Ien := MagPiece(xline, '^', 1);
        share := MagPiece(xline, '^', 2);
        Stype := MagPiece(xline, '^', 3); //storage type
        OnLine := MagPiece(xline, '^', 4);
        // P39 * iplace :=  MagPiece(xline,'^',10);
        chkvar := '$P($G(^MAG(2005.2,' + Ien + ',0)),U,10)_" "';
        iplace := MagUtilityBroker.FnmGetNode(chkvar);
        If iplace = '0 ' Then
          iplace := ' '
        Else
          iplace := MagPiece(iplace, ' ', 1);
        If (Stype = 'MAG') Or (Stype = 'WORM-OTG') Then ///RAID JB only
          If ((iplace = ' ') Or (iplace = PLACE)) And (OnLine = '1') Then /// share is the same PLACE/SITE as the user's
          Begin
            chkvar := '$P($G(^MAG(2005.2,' + Ien + ',0)),U,9)_" "';
            route := MagUtilityBroker.FnmGetNode(chkvar);
            If (route = ' ') Or (route = '0 ') Then /// not the router loc
            Begin
              chkvar := '$P($G(^MAG(2005.2,' + Ien + ',0)),U,1)';
              SName := MagUtilityBroker.FnmGetNode(chkvar);
              xline := Ien + '# ' + SName + ' *' + share + '*[' + Stype + ']';
              cbxFromShareLoc.Items.Add(xline);
            End;
          End;
      End; //I:=0
    End; //get shares
    If edtStartDT.Text = '' Then
    Begin
      Showmessage('Error: Must select a From date');
      MakeFocused(btnSel4Dt);
      Exit;
    End;
    MakeFocused(btnFromLoc);
    Result := True;
  Except
  End;
End;

Function TfMagUtility.ExitFromLoc(Sender: Tobject): Boolean;
Var
  chkvar: String;
  cnt: Integer;
  Ien: String;
  netloc0: String;
  sel: Integer;
  share: String;
  stgtype: String;
  xline: String;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.ExitFromLoc';
  Result := False;
  Try
    FUT49RemoveOld.Enabled := True;
    FUT49FromLoc := '';
    sel := cbxFromShareLoc.ItemIndex;
    If sel = -1 Then sel := 0; //take 1st default, without clk from listing
    If rgFut49ChkType.ItemIndex=0 Then
      If (EdtEndDT.Text = '') Or (EdtStartDT.Text = '') Then
      Begin
        Showmessage('Error - Must specify a full date range before selecting the From location.');
        cbxFromShareLoc.ClearSelection;
        cbxToShareLoc.ClearSelection;
        If (EdtEndDT.Text = '') Then
          MakeFocused(btnSel2Dt)
        Else
          If (EdtStartDT.Text = '') Then
            MakeFocused(btnSel4Dt)
          Else
            MakeFocused(rgFut49ChkType);
        Exit;
      End;
    If rgFut49ChkType.ItemIndex=1 Then
      If (EdtEndIen.Text = '') Or (EdtStartIen.Text = '') Then
      Begin
        Showmessage('Error - Must specify a full IEN range before selecting the From location.');
        cbxFromShareLoc.ClearSelection;
        cbxToShareLoc.ClearSelection;
        If (EdtEndIen.Text = '') Then
          MakeFocused(EdtEndIen)
        Else
          If (EdtStartIen.Text = '') Then
            MakeFocused(EdtStartIen)
          Else
            MakeFocused(rgFut49ChkType);
        Exit;
      End;
    If sel < 0 Then Exit;
    FUT49RemoveOld.Enabled := True;
    xline := Uppercase(cbxFromShareLoc.Items[sel]);
    Ien := MagPiece(xline, '#', 1);
    share := MagPiece(xline, '*', 2);
    chkvar := MagPiece(xline, '*', 1);
    stgtype := MagPiece(xline, '*', 1);
    stgtype := MagPiece(stgtype, ' ', 2);
    If Not Directoryexists(share) Then
    Begin
      Showmessage('Storage Copy - ' + #13 + ' Network Path "' + share + '" cannot be accessed.' + #13 + 'Verify that the share is online and accessible.');
      cbxFromShareLoc.ClearSelection;
      Exit;
    End;
    FUT49FromLoc := Ien; //net loc IEN
    //business rule - can't remove source file from JB-WORM
    If ANSIcontainsStr(stgtype, 'WORM') Then
    Begin
      FUT49RemoveOld.Enabled := False; //selected
      If FUT49RemoveOld.Checked Then Showmessage('Storage Copy - cannot remove source files from jukebox');
      FUT49RemoveOld.Checked := False; //JB file can't remove
    End;
    //T21 - Check if is Hashed subdirectory
    chkvar := '$G(^MAG(2005.2,' + Ien + ',0))_" "';
    netloc0 := MagUtilityBroker.FnmGetNode(chkvar);
    If Magpiece(netloc0,'^',8)<>'Y' then showmessage('Warning - Selected '+xline+' is not a hashed subdirectory structure, will require additional time to copy.');
    //remove selected From LOC and fill in TO LOC
    cbxToShareLoc.Items.Clear;
    cnt := cbxFromShareLoc.Items.Count - 1;
    For sel := 0 To cnt Do
    Begin
      xline := Uppercase(cbxFromShareLoc.Items[sel]);
      Ien := MagPiece(xline, '#', 1);
      stgtype := MagPiece(xline, '*', 1);
      stgtype := MagPiece(stgtype, ' ', 2);
      If Ien <> FUT49FromLoc Then
      Begin
        chkvar := '$G(^MAG(2005.2,' + Ien + ',0))_" "';
        netloc0 := MagUtilityBroker.FnmGetNode(chkvar);
        If MagPiece(netloc0, '^', 8) = 'Y' Then //copy dest must set HASH 'Y'
        Begin
          cbxToShareLoc.Items.Add(xline);
        End;
      End
    End;
    If (cbxToShareLoc.Items.Count > 0) Then cbxToShareLoc.Text := cbxToShareLoc.Items[0];
    MakeFocused(btnToLoc);
    Result := True;
  Except
  End;
End;

Function TfMagUtility.LookupFromShareLoc(Sender: Tobject): Boolean;
Var
  boResult: Boolean;
  sgCaption: String;
  sgDefaultDisplay: String;
  sgDisplay: String;
  sgDisplayList: String;
  sgStore: String;
  sgStoreList: String;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.LookupFromShareLoc';
  Result := False;
  Try
    FUT49FromShareLoc := '';
    sgStore := '';
    sgDisplay := '';
    sgCaption := 'Source Location';
    sgDisplayList := cbxFromShareLoc.Items.Text;
    sgStoreList := cbxFromShareLoc.Items.Text;
    sgDefaultDisplay := '';
    If cbxFromShareLoc.Items.Count>0 Then sgDefaultDisplay:=cbxFromShareLoc.Items[0];
    cbxFromShareLoc.Text := '';
    cbxToShareLoc.Text := '';
    boResult :=
      DlgLookup_ads(
      sgStore,
      sgDisplay,
      sgCaption,
      sgDisplayList,
      sgStoreList,
      sgDefaultDisplay
      );
    If boResult Then
    Begin
      FUT49FromShareLoc := sgDisplay;
      cbxFromShareLoc.ItemIndex := cbxFromShareLoc.Items.Indexof(sgDisplay);
      AdjustEnabled(gbcbxShareLoc, True);
      If Not ExitFromLoc(Sender) Then Exit;
      Result := True;
    End;
  Except
  End;
End;

Procedure TfMagUtility.actStorageLUSourceExecute(Sender: Tobject);
Var
  ProcName: String;
Begin
  ProcName:='TfMagUtility.actStorageLUSourceExecute';
  If FUT49Running Then Exit;
  FUT49RemoveOld.Enabled := True;
  cbxFromShareLoc.Text := '';
  cbxToShareLoc.Text := '';
  EnableRunButtonStorageCopy();
  If Not LookupFromShareLoc(Sender) Then Exit;
  Application.Processmessages();
  If cbxFromShareLoc.Text = '' Then cbxFromShareLoc.Text := FUT49FromShareLoc;
  cbxToShareLoc.Text := '';
  EnableRunButtonStorageCopy();
End;

Procedure TfMagUtility.actStorageLUDestExecute(Sender: Tobject);
Var
  ProcName: String;
Begin
  ProcName:='TfMagUtility.actStorageLUDestExecute';
  If FUT49Running Then Exit;
  If Trim(cbxFromShareLoc.Text) = '' Then
  Begin
    Showmessage('Please select the "Source" location first.');
    EnableRunButtonStorageCopy();
    Exit;
  End;
  If cbxToShareLoc.Items.Count = 0 Then
  Begin
    Showmessage('There are no "Destination" locations.');
    EnableRunButtonStorageCopy();
    Exit;
  End;
  If LookupToShareLoc(Sender) Then
  Begin
    Application.Processmessages();
    If cbxToShareLoc.Text = '' Then cbxToShareLoc.Text := FUT49ToShareLoc;
  End;
  EnableRunButtonStorageCopy();
End;

Function TfMagUtility.LookupToShareLoc(Sender: Tobject): Boolean;
Var
  boResult: Boolean;
  sgCaption: String;
  sgDefaultDisplay: String;
  sgDisplay: String;
  sgDisplayList: String;
  sgStore: String;
  sgStoreList: String;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.LookupToShareLoc';
  Result := False;
  Try
    FUT49ToShareLoc := '';
    sgStore := '';
    sgDisplay := '';
    sgCaption := 'Destination Location';
    sgDisplayList := cbxToShareLoc.Items.Text;
    sgStoreList := cbxToShareLoc.Items.Text;
    sgDefaultDisplay := '';
    If cbxToShareLoc.Items.Count>0 Then sgDefaultDisplay:=cbxToShareLoc.Items[0];
    cbxToShareLoc.Text := '';
    boResult :=
      DlgLookup_ads(
      sgStore,
      sgDisplay,
      sgCaption,
      sgDisplayList,
      sgStoreList,
      sgDefaultDisplay
      );
    If boResult Then
    Begin
      FUT49ToShareLoc := sgDisplay;
      cbxToShareLoc.ItemIndex := cbxToShareLoc.Items.Indexof(sgDisplay);
      AdjustEnabled(gbcbxShareLoc, True);
      If Not ExitToLoc(Sender) Then Exit;
      Result := True;
    End;
  Except
  End;
End;

Function TfMagUtility.ExitToLoc(Sender: Tobject): Boolean;
Var
  chkvar: String;
  Ien: String;
  route: String;
  sel: Integer;
  share: String;
  xline: String;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.ExitToLoc';
  Result := False;
  Try
    If cbxFromShareLoc.Text = '' Then
    Begin
      Showmessage('Specify the "FROM image network location" first before selecting the "To image network location".');
      cbxToShareLoc.ClearSelection;
      MakeFocused(btnFromLoc);
      Exit;
    End;
    sel := cbxToShareLoc.ItemIndex;
    If rgFut49ChkType.ItemIndex=1 Then
      If (EdtEndIen.Text = '') Or (EdtStartIen.Text = '') Then
      Begin
        Showmessage('Error - Must specify a full IEN range before selecting the To location.');
        cbxFromShareLoc.ClearSelection;
        cbxToShareLoc.ClearSelection;
        If (EdtEndIen.Text = '') Then
          MakeFocused(EdtEndIen)
        Else
          If (EdtStartIen.Text = '') Then
            MakeFocused(EdtStartIen)
          Else
            MakeFocused(rgFut49ChkType);
        Exit;
      End;
    If rgFut49ChkType.ItemIndex=0 Then
      If (EdtEndDT.Text = '') Or (EdtStartDT.Text = '') Then
      Begin
        Showmessage('Error - Must specify a full date range before selecting the To location.');
        cbxFromShareLoc.ClearSelection;
        cbxToShareLoc.ClearSelection;
        If (EdtEndDT.Text = '') Then
          MakeFocused(btnSel2Dt)
        Else
          If (EdtStartDT.Text = '') Then
            MakeFocused(btnSel4Dt)
          Else
            MakeFocused(rgFut49ChkType);
        Exit;
      End;
    If sel < 0 Then Exit;
    FUT49ToLoc := '';
    //IR#T15-9 don't set //If FUT49SetNewLoc.Enabled Then FUT49SetNewLoc.Checked := True;
    With cbxToShareLoc Do
    Begin
      If ItemIndex = -1 Then ItemIndex := 0;
      sel := ItemIndex;
      If sel < 0 Then Exit;
      xline := Uppercase(Items[sel]);
      Ien := MagPiece(xline, '#', 1);
      share := MagPiece(xline, '*', 2);
      If Not Directoryexists(share) Then //check to see if the share is on-line
      Begin
        Showmessage('Storage Copy - ' + #13 + ' Network Path "' + share + '" cannot be accessed.' + #13 + 'Verify that the share is online and accessible.');
        cbxToShareLoc.ClearSelection;
        Exit;
      End;
      chkvar := '$P($G(^MAG(2005.2,' + Ien + ',0)),U,9)_" "';
      route := MagUtilityBroker.FnmGetNode(chkvar);
      If route = '1 ' Then
      Begin
        Showmessage('Storage Copy - ' + #13 + 'Because the destination share is designated as a Routing share:' + #13
          + '1)	You may need to map a drive to this share to perform the copy;' + #13
          + '2)	#2005 references will not be changed; and' + #13
          + '3)	Source files cannot be deleted.');

        FUT49SetNewLoc.Enabled := False; /// the router loc cannot set #2005 reference
        FUT49SetNewLoc.Checked := False;
        FUT49RemoveOld.Enabled := False; /// nor remove files from source location
        FUT49RemoveOld.Checked := False;
      End
      Else
        If FUT49RemoveOld.Checked = False Then FUT49SetNewLoc.Enabled := True;
      FUT49ToLoc := Ien;
    End;
    MakeFocused(btnProcessFUT49);
    Result := True;
  Except
  End;
End;

Procedure TfMagUtility.cbxFromShareLocEnter(Sender: Tobject);
Var
  ProcName: String;
Begin
  ProcName:='TfMagUtility.cbxFromShareLocEnter';
  MakeFocused(btnFromLoc);
  actStorageLUSourceExecute(Sender);
End;

Procedure TfMagUtility.cbxToShareLocEnter(Sender: Tobject);
Var
  ProcName: String;
Begin
  ProcName:='TfMagUtility.cbxToShareLocEnter';
  MakeFocused(btnToLoc);
  actStorageLUDestExecute(Sender);
End;

Procedure TfMagUtility.FormKeyPress(Sender: Tobject; Var Key: Char);
Var
  ProcName: String;
  KeyWas  : Char;
Begin
  ProcName:='TfMagUtility.FormKeyPress';
  If ActiveControl Is TLabeledEdit Then
  Begin
    If (ActiveControl = edtStartIEN) Or (ActiveControl = edtEndIEN) Then
    Begin
      If FUT49Running Then
      Begin
        Key := #0;
        Exit;
      End;
      If Key=#8 Then Exit;
      If ActiveControl = edtStartIEN Then SetCaptionFut49byIEN4DT('');
      If ActiveControl = edtEndIEN Then SetCaptionFut49byIEN2DT('');
      TimerValidateIENValues.Enabled := True;
      If (ActiveControl = edtEndIEN) And (Trim(edtStartIEN.Text) = '') Then
      Begin
        Key := #0;
        If rgFut49ChkType.ItemIndex=1 Then //CodeCR398
        Begin
          Showmessage('Please populate the From IEN first.');  //t17 - IR T15-33
          MakeFocused(edtStartIEN);
        End;
        Exit;
      End;
      KeyWas  := Key;
      ValidateIENChars(Key);
      If Key = '*' Then
      Begin
        If ActiveControl = edtStartIEN Then
        Begin
          edtStartIEN.Text := '';
        End
        Else
        Begin
          edtEndIEN.Text := '';
        End;
      End
      Else
      Begin
        If Key <> #0 Then
        Begin
          If TLabeledEdit(ActiveControl).Text = '*' Then
          Begin
            TLabeledEdit(ActiveControl).Text := '';
            Exit;
          End;
          If TLabeledEdit(ActiveControl).Text = #8 Then
          Begin
            Exit;
          End;
          If Length(TLabeledEdit(ActiveControl).Text)>=18 Then
          Begin
            Key:=#0;
            Exit;
          End;
        End;
      End;
      If KeyWas=#13 Then   //T18 - Tab or Enter
      Begin
        If (ActiveControl = edtStartIEN)  Then
        Begin
          ActiveControl := edtEndIEN;
        End
        Else
        Begin
          If (ActiveControl = edtEndIEN) Then
          Begin
            edtEndIENexit(Sender);
            If (ProblemIndex = 0 ) then
            Begin
              If btnFromLoc.Visible and btnFromLoc.Enabled Then ActiveControl:= btnFromLoc;
            End;
          End;
        End;
      End;
    End;
  End;
End;


Procedure TfMagUtility.ValidateIENChars(Var Key: Char);
Var
  ProcName: String;
Begin
  ProcName:='TfMagUtility.ValidateIENChars';
  Case Key Of
    '0': Exit;
    '1': Exit;
    '2': Exit;
    '3': Exit;
    '4': Exit;
    '5': Exit;
    '6': Exit;
    '7': Exit;
    '8': Exit;
    '9': Exit;
    '*': Exit;
    #8: Exit; {Backspace}
  End;
  Key := #0; {Throw the key away}
End;

Procedure TfMagUtility.TimerValidateIENValuesTimer(Sender: Tobject);
Var
  boIENEndStar: Boolean;
  boIENStartStar: Boolean;
  inIENEnd: Int64;
  inIENStart: Int64;
  sgIENEnd: String;
  sgIENStart: String;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.TimerValidateIENValuesTimer';
  TimerValidateIENValues.Enabled := False;
  sgIENEnd := Trim(edtEndIEN.Text);
  sgIENStart := Trim(edtStartIEN.Text);
  If (sgIENStart = '') Or (sgIENEnd = '') Then
  Begin
    AdjustEnabled(gbcbxShareLoc, False);
    Exit;
  End;
  boIENStartStar := (Pos('*', sgIENStart) <> 0);
  boIENEndStar := (Pos('*', sgIENEnd) <> 0);
  If (Not boIENStartStar) And (Not boIENEndStar) Then
  Begin
    inIENEnd := StrToInt64(sgIENEnd);
    inIENStart := StrToInt64(sgIENStart);
    If inIENStart > inIENEnd Then
    Begin
      AdjustEnabled(gbcbxShareLoc, False);
      Exit;
    End;
  End;
  edtStartIENexit(Sender);
  //t17 ~~ If Not EndIENexit(Sender) Then Exit;
  //t17 - wait for TO IEN exit ~ IR t15-27 ~~AdjustEnabled(gbcbxShareLoc, True);
End;

Function TfMagUtility.EndIENexit(Sender: Tobject): Boolean;
Var
  Cdate: String;
  chkvar: String;
  Errmsg: String;
  i: Integer;
  Ien: String;
  iplace: String;
  Magien: int64;
  netloc0: String;
  OnLine: String;
  route: String;
  share: String;
  Shares: TStrings;
  SName: String;
  Stype: String;
  xline: String;
  ProcName: String;
  Date: String;
Begin
  ProcName:='TfMagUtility.EndIENexit';
  Result := False;
  Try
    If (edtStartIen.Text = '') Then
    Begin
      Showmessage('Must enter a From IEN.');
      InitStorageCopyPanelByIEN(True);
      AdjustEnabled(gbcbxShareLoc, False);
      EnableRunButtonStorageCopy();
      MakeFocused(edtStartIEN);
      Exit;
    End;
    If edtEndIEN.Text = '*' Then //* default last day IEN #2005
    Begin
      chkvar := '$O(^MAG(2005,"AD",$O(^MAG(2005,"AD"," "),-1)," "),-1)';
      edtEndIEN.Text := MagUtilityBroker.FnmGetNode(chkvar);
    End;

    //If (edtEndIEN.Text ='') or (Not IsStrANumber(edtEndIEN.Text)) Then
    If (edtEndIEN.Text ='') Then Exit;  //allow to switch to DT range
    If Not IsStrANumber(edtEndIEN.Text) Then
    Begin
      Showmessage('IEN must be numeric integer');
      edtEndIEN.Text := '';
      If rgFut49ChkType.ItemIndex=1 Then MakeFocused(edtEndIEN);
      Exit;
    End;
    If Not FUT49Running Then btnPauseFUT49.Enabled := False;
    Magien := strtoint64(edtEndIEN.Text);
    If Magien <> 0 Then //found IMG create DT from VistA
    Begin
      chkvar := '$P($G(^MAG(2005,' + Inttostr(Magien) + ',2)),"^")\1+17000000';
      Cdate := MagUtilityBroker.FnmGetNode(chkvar); // image info
      If Cdate <> '17000000' Then
      Begin
        Cdate := MagPiece(Cdate, '^', 1);
        Date := FormatDateTime('mm/dd/yyyy',Now()); //now date
        If FMEXTDT(Cdate) = Date then  //t17 - exclude today image IR T16-3
        Begin
          Showmessage('To IEN: Image from today will not be copied');
          edtEndIEN.Text := '';
          MakeFocused(edtEndIEN);
          Exit;
        End;
        SetCaptionFut49byIEN2DT('(date. ' + FMEXTDT(Cdate) + ' )');  //T18 - approx.
      End;
      cbxFromShareLoc.Items.Clear;
      If MagUtilityBroker.GetNetworkLocations(Shares) Then
      Begin
        For i := 0 To Shares.Count - 1 Do
        Begin
          xline := Uppercase(Shares[i]);
          Ien := MagPiece(xline, '^', 1);
          share := MagPiece(xline, '^', 2);
          Stype := MagPiece(xline, '^', 3); //storage type
          OnLine := MagPiece(xline, '^', 4);
          // P39 *iplace :=  MagPiece(xline,'^',10);
          chkvar := '$G(^MAG(2005.2,' + Ien + ',0))_" "';
          netloc0 := MagUtilityBroker.FnmGetNode(chkvar);
          iplace := MagPiece(netloc0, '^', 10) + ' ';
          If iplace = '0 ' Then
            iplace := ' '
          Else
            iplace := MagPiece(iplace, ' ', 1);
          If (Stype = 'MAG') Or (Stype = 'WORM-OTG') Then ///RAID JB only
            If ((iplace = ' ') Or (iplace = PLACE)) And (OnLine = '1') Then /// share is the same PLACE/SITE as the user's
            Begin
              route := MagPiece(netloc0, '^', 9) + ' ';
              If (route = ' ') Or (route = '0 ') Then /// not the router loc
              Begin
                SName := MagPiece(netloc0, '^', 1);
                xline := Ien + '# ' + SName + ' *' + share + '*[' + Stype + ']';
                cbxFromShareLoc.Items.Add(xline);
              End;
            End;
        End; //I:=0
      End; //get shares
    End
    Else
      If Magien = 0 Then
      Begin
        Showmessage('"0" is not a valid IEN'); //if MagIEN<>0
        edtStartIen.Text := '';
        MakeFocused(edtEndIEN);
        SetCaptionFut49byIEN2DT('(e.g.: 12345, `*`)');
        Exit;
      End;

    If Not validateRange(edtStartIEN.Text, edtEndIEN.Text, Errmsg, ProblemIndex) Then
    Begin
      ShowMessage('Invalid range specified, ' + Errmsg);//t17 - Fix 508 dialog problem
      InitStorageCopyPanelByIEN(True);
      AdjustEnabled(gbcbxShareLoc, False);
      EnableRunButtonStorageCopy();
      MakeFocused(edtStartIEN);
      Exit;
    End;
    Result := True;
    AdjustEnabled(gbcbxShareLoc, True); //t17 - enable when TO IEN exit
  Except
  End;
End;

Procedure TfMagUtility.EnableRunButtonStorageCopy;
Var
  bogbcbxShareLocEnabled: Boolean;
  boCbxFromShareLocFilled: Boolean;
  boCbxToShareLocFilled: Boolean;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.EnableRunButtonStorageCopy';
  bogbcbxShareLocEnabled := gbcbxShareLoc.Enabled;
  boCbxFromShareLocFilled := (Trim(CbxFromShareLoc.Text) <> '');
  boCbxToShareLocFilled := (Trim(CbxToShareLoc.Text) <> '');
  btnProcessFUT49.Enabled := (bogbcbxShareLocEnabled And boCbxFromShareLocFilled And boCbxToShareLocFilled);
End;

Procedure TfMagUtility.ArrangeStatusLabels;
Var
  ProcName: String;
Begin
  ProcName:='TfMagUtility.ArrangeStatusLabels';
  grpProcessStatus.Visible:= ((pcUtilTabs.ActivePage=MagOrphan) Or (pcUtilTabs.ActivePage=FUT49));
  If FUT49Running Or OrphanFilesRunning Then
  Begin
    If FUT49Running Then
    Begin
      grpProcessStatus.caption := 'Storage Copy process status';
    End
    Else
    Begin
      grpProcessStatus.caption := 'Orphan File process status';
    End;
    Exit;
  End
  Else
  Begin
    If pcUtilTabs.ActivePage=MagOrphan Then
    Begin
      grpProcessStatus.caption := 'Orphan File process status';
    End
    Else
    If pcUtilTabs.ActivePage=FUT49 Then
    Begin
      grpProcessStatus.caption := 'Storage Copy process status';
    End
    Else
      grpProcessStatus.caption := '';
  End;
  lblCurrentFile.Visible := True;
  lblCurrentFileLabel.Visible := True;
  lblCurrentIEN.Visible := True;
  lblErrorCnt.Visible := True;
  //lblFileEmpty.Visible := True;
  lblFileEmptyCnt.Visible := True;
  lblIEN.Visible := True;
  lblRecentErrorCountLabel.Visible := True;
  lblTotalcnt.Visible := True;
  lblTotalCount.Visible := True;

  lblCurrentFile.Enabled := True;
  lblCurrentFileLabel.Enabled := True;
  lblCurrentIEN.Enabled := True;
  lblErrorCnt.Enabled := True;
  //lblFileEmpty.Enabled := True;
  lblFileEmptyCnt.Enabled := True;
  lblIEN.Enabled := True;
  lblRecentErrorCountLabel.Enabled := True;
  lblTotalcnt.Enabled := True;
  lblTotalCount.Enabled := True;

  pnlStatusCurrentFile.Align := alnone;
  pnlStatusCurrentIEN.Align := alnone;
  pnlStatusRecentErrorCountLabel.Align := alnone;
  pnlStatusTotalCount.Align := alnone;
  pnlStatusFileEmpty.Align := alnone;

  pnlStatusCurrentFile.Visible := False;
  pnlStatusCurrentIEN.Visible := False;
  pnlStatusRecentErrorCountLabel.Visible := False;
  pnlStatusTotalCount.Visible := False;
  pnlStatusFileEmpty.Visible := False;
  pnlStatusCurrentFile.Left := 0;
  pnlStatusCurrentIEN.Left := 0;
  pnlStatusRecentErrorCountLabel.Left := 0;
  pnlStatusTotalCount.Left := 0;
  pnlStatusFileEmpty.Left := 0;
  pnlStatusCurrentFile.Width := grpProcessStatus.Width - 10;
  pnlStatusCurrentIEN.Width := grpProcessStatus.Width - 10;
  pnlStatusRecentErrorCountLabel.Width := grpProcessStatus.Width - 10;
  pnlStatusTotalCount.Width := grpProcessStatus.Width - 10;
  pnlStatusFileEmpty.Width := grpProcessStatus.Width - 10;
  If pcUtilTabs.ActivePage = FUT49 Then
  Begin
    pnlStatusCurrentFile.Visible := True; //vetted
    pnlStatusCurrentIEN.Visible := False; //vetted
    pnlStatusRecentErrorCountLabel.Visible := True; //T20-228  Not copied:
    pnlStatusTotalCount.Visible := True; //Copied:
    pnlStatusFileEmpty.Visible := True; //Pointer empty:
    pnlStatusRecentErrorCountLabel.Top := 18;
    pnlStatusTotalCount.Top := pnlStatusRecentErrorCountLabel.Top + pnlStatusRecentErrorCountLabel.Height + 1;
    pnlStatusFileEmpty.Top := pnlStatusTotalCount.Top + pnlStatusTotalCount.Height + 1;
    pnlStatusCurrentFile.Top := pnlStatusFileEmpty.Top + pnlStatusFileEmpty.Height + 1;
    pnlStatusCurrentIEN.Top := pnlStatusCurrentFile.Top + pnlStatusCurrentFile.Height + 1;
    Exit;
  End;
  If pcUtilTabs.ActivePage = MagOrphan Then
  Begin
    pnlStatusCurrentFile.Visible := True; //Current File:
    pnlStatusCurrentIEN.Visible := True; //Current IEN:
    pnlStatusRecentErrorCountLabel.Visible := True; //Orphan files:
    pnlStatusTotalCount.Visible := True; //Copied:
    pnlStatusFileEmpty.Visible := False; //
    pnlStatusCurrentFile.Top := 18;
    pnlStatusCurrentIEN.Top := pnlStatusCurrentFile.Top + pnlStatusCurrentFile.Height + 1;
    pnlStatusRecentErrorCountLabel.Top := pnlStatusCurrentIEN.Top + pnlStatusCurrentIEN.Height + 1;
    pnlStatusTotalCount.Top := pnlStatusRecentErrorCountLabel.Top + pnlStatusRecentErrorCountLabel.Height + 1;
    pnlStatusFileEmpty.Top := pnlStatusTotalCount.Top + pnlStatusTotalCount.Height + 1;
    Exit;
  End;
End;

Procedure TfMagUtility.ORaidListViewMouseUp(Sender: Tobject;
  Button: TMouseButton; Shift: TShiftState; x, y: Integer);
Var
  ProcName: String;
Begin
  ProcName:='TfMagUtility.ORaidListViewMouseUp';
  EnableRunButtonOrphanFiles();
End;

Procedure TfMagUtility.EnableRunButtonOrphanFiles();
Var
  boSelected: Boolean;
  boRemoveOrphanChecked: Boolean;
  boReportOrphanOnlyChecked: Boolean;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.EnableRunButtonOrphanFiles';
  boSelected := (ORaidListView.Selcount > 0);
  boRemoveOrphanChecked := (rgOrphanFiles.ItemIndex=1);
  boReportOrphanOnlyChecked := (rgOrphanFiles.ItemIndex=0);
  btnRunProcess.Enabled :=
    boSelected
    And
    (boRemoveOrphanChecked Or boReportOrphanOnlyChecked)
    And
    (Not OrphanFilesRunning);
  btnStopProcessing.Enabled := OrphanFilesRunning;
End;

Procedure TfMagUtility.TimerOrphanEnablingTimer(Sender: Tobject);
Var
  ProcName: String;
Begin
  ProcName:='TfMagUtility.TimerOrphanEnablingTimer';
  TimerOrphanEnabling.Enabled := False;
  EnableRunButtonOrphanFiles();
End;

Procedure TfMagUtility.EnableRunButtonNetworkCleanup;
Var
  boSelected: Boolean;
  ProcName: String;
Begin
  ProcName:='TfMagUtility.EnableRunButtonNetworkCleanup';
  boSelected := (NetClnChkListView.Selcount > 0);
  btnNtLocDelRun.Enabled := boSelected;
End;

Procedure TfMagUtility.NetClnChkListViewMouseUp(Sender: Tobject;
  Button: TMouseButton; Shift: TShiftState; x, y: Integer);
Var
  ProcName: String;
Begin
  ProcName:='TfMagUtility.NetClnChkListViewMouseUp';
  EnableRunButtonNetworkCleanup();
End;

procedure TfMagUtility.InitStorageCopy(Sender: Tobject);
Var
  ProcName: String;
begin
  ProcName:='TfMagUtility.InitStorageCopy';
  If FUT49Running Then Exit;
  GaugeFUT49     .Visible:=False;
  rgFut49ChkType .Visible:= True;
  rgFut49ChkType .Enabled:= True;
  rgFut49ChkType .ItemIndex:=-1;
  btnSel2DT      .Visible:= True;
  btnSel2DT      .Enabled:= False;
  btnSel4DT      .Visible:= True;
  btnSel4DT      .Enabled:= False;
  btnPauseFUT49  .Visible:= True;
  btnPauseFUT49  .caption:= '&Pause';
  btnPauseFUT49  .Enabled:= False;
  btnProcessFUT49.Visible:= True;
  btnProcessFUT49.caption:= '&Run';
  If bmpGo.Width>3 Then btnProcessFUT49.Glyph.Assign(bmpGo);
  btnProcessFUT49.Enabled:= False;
  cbxFromShareLoc.Visible:= True;
  cbxFromShareLoc.Enabled:= False;
  cbxFromShareLoc.Clear;
  cbxFromShareLoc.OnExit := Nil;
  cbxFromShareLoc.TabStop:= False;
  cbxFromShareLoc.Text   := '';
  cbxToShareLoc  .Visible:= True;
  cbxToShareLoc  .Enabled:= True;
  cbxToShareLoc  .Clear;
  cbxToShareLoc  .OnExit := Nil;
  cbxToShareLoc  .TabStop:= False;
  cbxToShareLoc  .Text   := '';
  FUT49AftHour   .Visible:= True;
  FUT49AftHour   .Checked:= False;
  FUT49AftHour   .Enabled:= True;
  FUT49RemoveOld .Visible:= True;
  FUT49RemoveOld .Checked:= False;
  FUT49RemoveOld .Enabled:= True;
  FUT49SetNewLoc .Visible:= True;
  FUT49SetNewLoc .Checked:= True;
  FUT49SetNewLoc .Enabled:= True;
  SetCaptionFUT49byDt2ien('');
  SetCaptionFut49byIEN4DT('');
  lblCurrentFile .caption:= '';
  UpdateLabel(lblErrorCnt,'');
  lblIEN         .caption:= '';
  lblTotalcnt    .caption:= '';
  FUT49Mark              := '';
  //t17 - (default)InitStorageCopyPanelByDate(False);
   rgFUT49ChkType.ItemIndex:=0;
   InitStorageCopyPanelByDate(True);
  InitStorageCopyPanelByIEN(False);
  StorageCopyEnabling(Sender);
  ArrangeStatusLabels();
end;

procedure TfMagUtility.InitAll(Sender: Tobject);
Var
  ProcName: String;
begin
  ProcName:='TfMagUtility.InitAll';
  InitStorageCopy(Sender);
end;

procedure TfMagUtility.StorageCopyWarning(Sender: TObject);
Var
  ProcName: String;
begin
  ProcName:='TfMagUtility.StorageCopyWarning';
  If (Fut49WarnMsg <> '') And (Not FUT49Running) Then
  Begin //give warn msg once
    ShowMessage(FUT49WarnMsg);//t17 - Fix 508 dialog problem
    Flogger.Info(#9 + 'Storage Copy'+ #9 + 'User: ' + MagUtilityBroker.UserFullName + ' acknowledged notification that Storage Copy is not a disk duplication tool.');
    FErrorlogger.Info(#9 + 'Storage Copy'+ #9 + 'User: ' + MagUtilityBroker.UserFullName + ' acknowledged notification that Storage Copy is not a disk duplication tool.');
    Fut49WarnMsg := '';
  End;
end;

procedure TfMagUtility.rgOrphanFilesClick(Sender: TObject);
Var
  ProcName: String;
begin
  ProcName:='TfMagUtility.rgOrphanFilesClick';
  Case rgOrphanFiles.ItemIndex Of
  0:
    Begin
      RemoveOrphanFile := False;
      //FLogger.Info(#9 + 'Orphan Files' + #9 + #9 + 'Generate report only');
      EnableRunButtonOrphanFiles();
    End;
  1:
    Begin
      RemoveOrphanFile := True;
      EnableRunButtonOrphanFiles();
    End;
  End;
end;

procedure TfMagUtility.InitStorageCopyPanelByDate(IsEnabled: Boolean);
Var
  ProcName: String;
begin
  ProcName:='TfMagUtility.InitStorageCopyPanelByDate';
  AdjustEnabled(pnlByDate, IsEnabled);
  edtEndDT       .Visible:= True;
  edtEndDT       .Enabled:= IsEnabled;
  edtEndDT       .Text   := '';
  edtEndDT       .TabStop:= False;
  edtEndDT       .Color  := clBtnFace;
  edtStartDT     .Visible:= True;
  edtStartDT     .Enabled:= IsEnabled;
  edtStartDT     .Text   := '';
  edtStartDT     .TabStop:= False;
  edtStartDT     .Color  := clBtnFace;
  SetCaptionFut49byDt4ien('');
  SetCaptionFUT49byDt2ien('');
end;

procedure TfMagUtility.edtStartDTEnter(Sender: TObject);
Var
  ProcName: String;
begin
  ProcName:='TfMagUtility.edtStartDTEnter';
  MakeFocused(btnSel4Dt);
end;

procedure TfMagUtility.EdtEndDTEnter(Sender: TObject);
Var
  ProcName: String;
begin
  ProcName:='TfMagUtility.EdtEndDTEnter';
  MakeFocused(btnSel2Dt);
end;

procedure TfMagUtility.InitStorageCopyPanelByIEN(IsEnabled: Boolean);
Var
  ProcName: String;
begin
  ProcName:='TfMagUtility.InitStorageCopyPanelByIEN';
  edtEndIEN      .visible:=True;
  edtStartIEN    .visible:=True;
  pnlByIEN       .visible:=True;
  AdjustEnabled(pnlByIEN, IsEnabled);
  edtEndIEN  .Text :='';
  edtStartIEN.Text :='';
  SetCaptionFut49byIEN2DT('');
  SetCaptionFut49byIEN4DT('(e.g.: 12345, `*`)');
  MakeFocused(edtStartIEN);
end;

function TfMagUtility.checkOrphanFileLoop(LoopName, netlocien,Path: String; FileRec: TSearchRec;boShowMessage: Boolean): Boolean;
Var
  inMinuteDelay: Integer;
  inRetryLoop  : Integer;
  inRetries    : Integer;
  j            : Integer;
  ProcName: String;
begin
  ProcName:='TfMagUtility.checkOrphanFileLoop';
  Result:=False;
  If FProcessingPaused And (Not boInRetryLoop) Then Exit;
  boInRetryLoop:= False;
  inMinuteDelay:=1;
  inRetries    := 3;
  inRetryLoop  := 0;
  pnlRetryTitle.Caption := 'Orphan Files';
  pnlRetryTop.Caption   := 'retrying in '+InttoStr(inMinuteDelay)+' min';
  For j:=0 To inRetries Do
  Begin
    Application.ProcessMessages();
    If Not CheckOrphanFile('Orphan Files', netlocien, Path, FileRec) Then
    Begin
      boInRetryLoop:= True;
      inRetryLoop:=inRetryLoop+1;
      If inRetryLoop<(inRetries+1) Then
      Begin
        pnlRetryBot.Caption:= 'attempt '+IntToStr(inRetryLoop)+' of '+IntToStr(inRetries);
        Application.ProcessMessages();
        SetPnlRetryVis(True);
        FErrorLogger.Error(#9 + LoopName+#9+#9+'with '+IntToStr(inMinuteDelay)+' minute wait('+IntToStr(inRetryLoop)+'): Error received during Orphan File check in checkOrphanFile() with arguments RAID netlocien='+netlocien + ' "'+ Path+'" ('+UnitName+'.'+ProcName+'.1)');  //T21 - log 212(5055)
        FLogger.Error(#9 + LoopName+#9+#9+'with '+IntToStr(inMinuteDelay)+' minute wait('+IntToStr(inRetryLoop)+'): Error received during Orphan File check with RAID share(ien# '+netlocien+ ') ' + Path);  //T21
        If Not TakeANap(1000*60*inMinuteDelay) Then Break;
      End
      Else
      Begin  //Retry failed
        FErrorLogger.Error(#9 + LoopName+#9+#9+'Aborted Orphan File check: in checkOrphanFile() with arguments RAID netlocien='+netlocien+' "'+ Path+'" ('+UnitName+'.'+ProcName+'.2)');
        FLogger.Error(#9 + LoopName+#9+#9+'Aborted Orphan File check with RAID share(ien# '+netlocien+') '+ Path + #9+'Review troubleshooting tips in user manual');//T21
        If boShowMessage Then
        Begin
          SetPnlRetryVis(False);
          Showmessage(LoopName+' did not complete successfully. See log for individual file status and check ' +
                       'patch documentation for troubleshooting suggestions.'); //t17 - Dec 07 meeting IRt15-21 & T15-22
          OrphanFilesCancel('');
        End;
        Break;
      End;
    End
    Else
    Begin //retry success
      Result:=True;
      Break;
    End;
  End; //retry loop
  SetPnlRetryVis(False);
  boInRetryLoop:= False;
end;

function TfMagUtility.ExecuteShellCommandLoop(LoopName, CmdLine: String;hidden: Boolean): Boolean;
Var
  errorcntWas  : Integer;
  inMinuteDelay: Integer;
  inRetries    : Integer;
  inRetryLoop  : Integer;
  j            : Integer;
  ProcName     : String;
begin
  ProcName:='TfMagUtility.ExecuteShellCommandLoop';
  Result:=False;
  If FProcessingPaused And (Not boInRetryLoop) Then Exit;
  boInRetryLoop:= False;
  inMinuteDelay:= 1;
  inRetries    := 3;
  inRetryLoop  := 0;
  pnlRetryTitle.Caption := 'Storage Copy';
  pnlRetryTop.Caption := 'retrying in '+InttoStr(inMinuteDelay)+ ' min';
  errorcntWas:=errorcnt;
  For j:=0 To inRetries Do
  Begin
    If Not ExecuteShellCommand(PChar(CmdLine), True) Then
    Begin
      LastErrorCode:= GetLastError;  //t17- IR T16-10 ref te last failed code
      boInRetryLoop:= True;
      inRetryLoop:=inRetryLoop+1;
      If inRetryLoop<(inRetries+1) Then
      Begin
        pnlRetryBot.Caption:= 'attempt '+IntToStr(inRetryLoop)+' of '+IntToStr(inRetries);
        SetPnlRetryVis(True);
        FErrorLogger.Info(#9 + LoopName+' '+IntToStr(inMinuteDelay)+' minute wait('+IntToStr(inRetryLoop)+'): Error received in ExecuteShellCommand() with arguments CmdLine='+CmdLine+' ('+UnitName+'.'+ProcName+'.19)');
        FLogger.Info(#9 + LoopName+' '+IntToStr(inMinuteDelay)+' minute wait('+IntToStr(inRetryLoop)+'): Error received in executing Shell Command with Command Line="'+CmdLine+'"');
        If Not TakeANap(1000*60*inMinuteDelay) Then Break;
      End
      Else
      Begin //retry failed
        StorageCopyCancel(#9 + LoopName+#9+'Aborted: Error received in executing Shell Command with Command Line="'+ CmdLine+'"',LoopName,errorcntWas+1);  //T21-log 217(5119), t17
        Break;
      End;
    End
    Else
    Begin  //retry success
      Result:=True;
      Break;
    End;
  End;
  errorcnt:=errorcntWas;
  SetPnlRetryVis(False);
  boInRetryLoop:= False;
  LastErrorCode := GetLastError; //t17
end;

Function TfMagUtility.TakeANap(TotalTime: Integer): Boolean;
Var
  EndTime    : TDateTime;
  fltDelta   : Double;
  i          : Integer;
  iMax       : Integer;
  inInterval : Integer;
  StartTime  : TDateTime;
  CurTime    : TDateTime;
  ProcName: String;
begin
  ProcName:='TfMagUtility.TakeANap';
  Result:=True;
  If TotalTime<1 Then Exit;
  inInterval:= 500;
  StartTime := now();
  fltDelta  := TotalTime/(24*60*60*1000);
  EndTime   := StartTime+fltDelta;
  iMax      := TotalTime div inInterval;
  For i:=1 To iMax Do
  Begin
    Sleep(inInterval);
    Application.ProcessMessages();
    If FProcessingPaused And (Not boInRetryLoop) Then
    Begin
      Result:=False;
      Exit;
    End;
    CurTime := now();
    If CurTime>EndTime Then Break;
  End;
end;

procedure TfMagUtility.TestTakeANap;
Var
  StartTime: TDateTime;
  EndTime: TDateTime;
  Delta  : TDateTime;
  ProcName: String;
begin
  ProcName:='TfMagUtility.TestTakeANap';
  StartTime:= now();
  TakeANap(60000);
  EndTime:= now();
  Delta  := EndTime-StartTime;
  ShowMessage(
    'StartTime='+FormatDateTime('hh:nn:ss',StartTime)+#13+#10+
    'EndTime='+FormatDateTime('hh:nn:ss',EndTime)+#13+#10+
    'Delta='+FormatDateTime('hh:nn:ss',Delta)+#13+#10);
end;

procedure TfMagUtility.SetPnlRetryVis(IsVisible: Boolean);
Var
  ProcName: String;
begin
  ProcName:='TfMagUtility.SetPnlRetryVis';
  If IsVisible Then
  Begin
    pnlRetry   .Visible:=IsVisible;
    If Not pnlRetryTitle.Visible Then pnlRetryTitle.Visible:=True;
    If Not pnlRetryTop  .Visible Then pnlRetryTop  .Visible:=True;
    If Not pnlRetryBot  .Visible Then pnlRetryBot  .Visible:=True;
    If Not pnlRetryAVI  .Visible Then pnlRetryAVI  .Visible:=True;
    pnlRetryAVI.Active :=True;
    pnlRetry.BringToFront;
  End
  Else
  Begin
    pnlRetry   .Visible:=False;
    pnlRetryAVI.Active :=False;
  End;
end;

procedure TfMagUtility.OrphanFilesCancel(Msg: String);
Var
  ProcName: String;
begin
  ProcName:='TfMagUtility.OrphanFilesCancel';
  SetPnlRetryVis(False);
  boInRetryLoop:= False;
  btnRunProcess.Enabled := False;
  btnStopProcessing.Enabled := False;
  FLogger.Info(Msg);
  FErrorLogger.Info(Msg + ' ('+UnitName+'.'+ProcName+'.1)');
  FProcessingPaused := True;
  OrphanFilesRunning := False;
  OrphanRAIDListView();
  SetPnlRetryVis(False);
  EnableRunButtonOrphanFiles();
  TimerOrphanEnabling.Enabled := True;
  Application.Processmessages();
end;

procedure TfMagUtility.FUT49Run(Sender: Tobject);
Var
  ProcName: String;
begin
  ProcName:= 'TfMagUtility.FUT49Run';
  FUT49RunInit(Sender);
end;

procedure TfMagUtility.FUT49Resume(curIEN,IEN2: Integer;NET0: String);
Var
  boFUT49AftHourEnabled  : Boolean;
  boFUT49RemoveOldEnabled: Boolean;
  boFUT49SetNewLocEnabled: Boolean;
  borgFut49ChkTypeEnabled: Boolean;
  childIEN               : String;
  chkvar                 : String;
  Copied                 : String;
  CopyFileType           : String;
  doneMsg                : String;
  errormsg               : String;
  Fname                  : String;
  FulFile                : String;
  IEN0                   : String;
  MAGR                   : TStrings;
  NewptrRef              : String;
  NofTot                 : String;
  ProcName               : String;
  Res                    : String;
  Set2005pt              : String;
Begin
  ProcName:= 'TfMagUtility.FUT49Run';
  childIEN               := '';
  doneMsg                := '';
  errormsg               := '';
  Set2005pt              := '';
  boFUT49AftHourEnabled  := FUT49AftHour.Enabled;
  boFUT49RemoveOldEnabled:= FUT49RemoveOld.Enabled;
  boFUT49SetNewLocEnabled:= FUT49SetNewLoc.Enabled;
  borgFut49ChkTypeEnabled:= rgFut49ChkType.Enabled;
  Try
    FUT49AftHour.Enabled  := False;
    FUT49RemoveOld.Enabled:= False;
    FUT49SetNewLoc.Enabled:= False;
    rgFut49ChkType.Enabled:= False;
    GaugeFUT49.Visible:=True;
    If FUT49Mark = '' Then Exit; //nothing to restart
    While FUT49Running And ((curIEN + 1) > IEN2) Do //loop IEN range
    Begin
      FUT49curIEN    := curIEN;
      FUT49IEN2      := IEN2;
      FUT49NET0      := NET0;
      ProgressUpdate(GaugeFUT49,curIEN,FUT49IEN1,IEN2);
      Application.ProcessMessages();
      If btnProcessFUT49.Focused And (btnProcessFUT49.caption = '&Cancel') Then
      Begin
        lblCurrentFile.caption := '';
        lblIEN.caption := '';
        lblTotalcnt.caption := '';
        UpdateLabel(lblErrorCnt,'');
        FUT49Running := False;
        FUT49Mark    := ''; //IR#T15-25
        StorageCopyCancel(#9 + 'Storage Copy' + #9 + 'The task canceled at ' + TimeToStr(Time) +
                           ' by user: '+ MagUtilityBroker.UserFullName ,'Storage Copy');
        Break; //T17 - replace EXIT;
      End;
      While (btnProcessFUT49.caption = '&Cancel') And (btnPauseFUT49.caption = '&Resume') Do
      Begin
        // pause & still waiting for the user action//
        Sleep(100);
        FUT49Running := True;
        Application.Processmessages;
        If btnProcessFUT49.Focused Then
        Begin
          FUT49AftHour.Enabled  := True;
          FUT49RemoveOld.Enabled:= True;
          FUT49SetNewLoc.Enabled:= True;
          rgFut49ChkType.Enabled:= True;
          GaugeFUT49.Visible:=False;
          Break;  //Cancel/Stop
        End;
      End;
      If not FUT49Running and (FUT49Mark='') then Break; //IR#T15-25
        // check 'Run after hours' option
      If FUT49AftHour.Checked Then FUT49chktime(); //waiting for aft-hours to restart
      If not FUT49Running and (FUT49Mark='') then Break;  //t17
      If btnProcessFUT49.Focused And (btnProcessFUT49.caption = '&Cancel') Then Exit; //T17
      UpdateLabel(lblCurrentFile,'');//t17 IR T16-11
      FulFile := ' ';

      While ((curIEN + 1) > IEN2) And (Fulfile = ' ') Do
      Begin
        // check if #2005 entry exist
        chkvar := '$G(^MAG(2005,' + Inttostr(curIEN) + ',0))_" "';
        FulFile := MagUtilityBroker.FnmGetNode(chkvar);
        If Fulfile = ' ' Then
        Begin
          SetCurIEN(curIEN,curIEN-1);
          IEN0 := Inttostr(curIEN) + ':' + Inttostr(IEN2);
          FUT49Mark := IEN0 + '^' + NET0;
        End
        Else //with #2005 file entry
        Begin
          showdetail := 0; //T20 - CodeCR 229
          //check if it is a group/parent node - no full image file ref/no ptr
          If MagPiece(FulFile, '^', 2) = ' ' Then //fileRef
          Begin
            chkvar := '$P($G(^MAG(2005,' + Inttostr(curIEN) + ',1,+$O(^MAG(2005,' + Inttostr(curIEN) + ',1,0)),0)),U,1)_" "';
            childIEN := MagUtilityBroker.FnmGetNode(chkvar);
            If (childIEN <> ' ') Then
            Begin
              childIEN := MagPiece(childIEN, ' ', 1);
              If (Strtoint(childIEN) > IEN2) And (OKCNT > 0) Then
                OKCNT := OKCNT + 1
              Else
                If FUT49Running then errorcnt := errorcnt + 1;
              //T21 - removed, FLogger.Info(#9 + 'Storage Copy' + #9 + 'IEN# ' + Inttostr(curIEN) + ' *Group/parent node of # ' + childIEN);
              //T21 -          FErrorLogger.Info(#9 + 'Storage Copy' + #9 + 'IEN# ' + Inttostr(curIEN) + ' *Group/parent node of # ' + childIEN);
            End
            Else ptrEmptyCnt := ptrEmptyCnt + 1;
            SetCurIEN(curIEN,curIEN-1);//skip to next
            IEN0 := Inttostr(curIEN) + ':' + Inttostr(IEN2);
            FUT49Mark := IEN0 + '^' + NET0;
            lbltotalcnt.caption := Inttostr(OKCNT);
            UpdateLabel(lblErrorCnt,errorcnt);
            //lblFileEmptyCnt.caption := Inttostr(ptrEmptyCnt);
          End //2005 2nd piece null
          Else
            Begin
              If (Length(MagPiece(FulFile, '^', 2))>12) And (Length( MagPiece(FulFile, '^', 2))<18) Then
                  showdetail:=1;//T20 - show Fix 9.3, 10.3 to 14.3 formt #CodeCR 229
            End;
        End;
      End; //while seek next entry curIEN<IEN2, check if #2005 entry exist
      If (curIEN = 0) Then //completed ien =0
      Begin
        lbltotalcnt.caption := Inttostr(OKCNT);
        UpdateLabel(lblErrorCnt,errorcnt);
        ProgressUpdate(GaugeFUT49, IEN2,FUT49IEN1,IEN2); //T18 - complete 100%
        Application.Processmessages; //IR#T15-5
        If OKCNT > 0 Then
        Begin
          If doneMsg = '' Then
            EndProcessMsg(
              'Storage Copy', //ProcessName: String;
              True          , //EndedSuccessfully,
              False         , //ViewLogs: Boolean;
              OKCNT         , //OKCNT,
              errorcnt      , //errorcnt: Integer);
              lblTotalcnt   , //CopiedLabel: TLabel
              lblErrorCnt   );//NotCopiedLabel: TLabel
          doneMsg := '1';
          FUT49Running := False;
          FLogger.Info(#9 + 'Storage Copy' + #9 +#9 + 'Completed copying files for requested IEN or date range - ' + Inttostr(OKCNT) + ' IENs copied, ' + Inttostr(errorcnt) + ' IENs not copied.'); //t17 - IR T16-05 log summary
          FErrorLogger.Info(#9 + 'Storage Copy' + #9 +#9 + 'Completed copying files for requested IEN or date range - ' + Inttostr(OKCNT) + ' IENs copied, ' + Inttostr(errorcnt) + ' IENs not copied.'+' ('+UnitName+'.'+ProcName+'.1)');//T21
        End
        Else //OKCNT=0
          If doneMsg = '' Then
          Begin
            EndProcessMsg(
              'Storage Copy', //ProcessName: String;
              False         , //EndedSuccessfully,
              True          , //ViewLogs: Boolean;
              OKCNT         , //OKCNT,
              errorcnt      , //errorcnt: Integer);
              lblTotalcnt   , //CopiedLabel: TLabel
              lblErrorCnt   );//NotCopiedLabel: TLabel
            FLogger.Info(#9 + 'Storage copy' + #9 + 'Task is complete. No files were copied.'+#9+ 'No files exist at source location');
            FErrorLogger.Info(#9 + 'Storage copy' + #9 + 'Task is complete. No files were copied.'+#9+ 'No files exist at source location'+' ('+UnitName+'.'+ProcName+'.2)'); //T18
            doneMsg := '1';
            FUT49Running := False;
          End;
        SetCurIEN(curIEN,curIEN-1);
      End;

      If curIEN > 0 Then
      Begin
        MAGR := Tstringlist.Create(); //init RPC call - MAGR returned value
       try  //T24 - memory release
        If IEN0='' Then
        Begin
          IEN0:=IntToStr(curIEN)+':'+IntToStr(IEN2);
        End;
        If MagUtilityBroker.FETCH(MAGR, IEN0, NET0) Then //RPC - MAG STORAGE FETCH - success
        Begin ///MAGR[0] (returned value) IM_":"_ENDIM_",CMD summit "_LNO_U_WRTLOC_U_$G(FRMLOC)_U_DT_U_$G(MOVOLD)_"@"_DATE_"|"_DT2_"|"_$G(StorageTYP)_"|"_FileNAME_"|"_+$G(XBIG,0)
          Res := MagPiece(MAGR[0], ',', 1);
          SetCurIEN(curIEN,Strtoint(MagPiece(Res, ':', 1)));
          Fname := MagPiece(MAGR[0], '|', 4);
          If curIEN > 0 Then
          Begin
            If FUT49Mark = '' Then Break; //T17 - task cancel
            totalcnt := totalcnt + 1;
            lbltotalcnt.caption := Inttostr(OKCNT);
            If ToCOPYFILE(MAGR,Copied,NofTot) Then
            Begin
              If (FUT49RemoveOld.Checked) Or (FUT49SetNewLoc.Checked) Then
              Begin
                chkvar := '$G(^MAG(2005,' + Inttostr(curIEN) + ',0))_" "';
                FulFile := MagUtilityBroker.FnmGetNode(chkvar);  //IR#T15-7
                If (MagPiece(FulFile, '^', 3)=Fut49FromLoc) or (MagPiece(FulFile, '^', 5)=Fut49FromLoc) then
                Begin
                   CopyFileType:='*'; //set default full file
                   If (MagPiece(FulFile, '^', 3)=Fut49FromLoc) and (MagPiece(FulFile, '^', 4)<>Fut49FromLoc) then CopyFileType:='r'; //IR# T15-10 P3 only
                End
                Else
                   if (MagPiece(FulFile, '^', 4)=Fut49FromLoc) then CopyFileType:='@' //set ABS
                   Else
                   Begin
                     CopyFileType:='b'; //set BIG
                   End;
                NewptrRef:='RAID';
                If (MagPiece(MAGR[0], '|', 3) = 'RJ') or (MagPiece(MAGR[0], '|', 3) = 'JJ') Then NewptrRef:='JB ';//t17 - IR T16-7
                Set2005pt:='';
                If MagUtilityBroker.setNETloc(Inttostr(curIEN), FUT49ToLoc + CopyFileType + '|' + Fut49FromLoc) Then ///set new Destnation reference
                Begin
                  Set2005pt := ', set new '+NewptrRef+' reference ' + MagPiece(cbxToShareLoc.Text,' ',2) + '.';
                  //FLogger.Info(#9 + 'Storage Copy' + #9 + 'Set image IEN ' + Inttostr(curIEN) + ' with new '+ NewptrRef +' reference ' + MagPiece(cbxToShareLoc.Text,' ',2) + '.');//T19 CodeCR=IMAG0000233
                  //FErrorLogger.Info(#9 + 'Storage Copy' + #9 + 'Set image IEN ' + Inttostr(curIEN) + ' with new '+ NewptrRef +' reference ' + MagPiece(cbxToShareLoc.Text,' ',2) + ' ('+UnitName+'.'+ProcName+'.3)');//T19 CodeCR=IMAG0000233
                End;
                ///Only if copy from RAID to JB, and remove source file, keep only JB (unset 0 P3, P4 or FBIG P1 for RAID)
                If (FUT49RemoveOld.Checked) And (MagPiece(MAGR[0], '|', 3) = 'RJ') Then
                  If MagUtilityBroker.unsetNETloc(Fname, MagPiece(Fname, '.', 2), curIEN) Then
                  Begin
                    MagUtilityBroker.unsetNETloc(Fname, 'ABS', curIEN); //unset ABS ref
                    If (MagPiece(MAGR[0], '|', 5) <> '0') Then MagUtilityBroker.unsetNETloc(Fname, 'BIG', curIEN);
                    Set2005pt := ', cleared RAID pointer '+ MagPiece(cbxFromShareLoc.Text,' ',2);
                    //FLogger.Info(#9 + 'Storage Copy' + #9 + 'Cleared RAID pointer for image IEN ' + Inttostr(curIEN) + ' RAID pointer '+ MagPiece(cbxFromShareLoc.Text,' ',2));  //T19 CodeCR=IMAG0000233
                    //FErrorLogger.Info(#9 + 'Storage Copy' + #9 + 'Cleared RAID pointer for image IEN ' + Inttostr(curIEN) + ' RAID pointer '+ MagPiece(cbxFromShareLoc.Text,' ',2)+' ('+UnitName+'.'+ProcName+'.4)');  //T19 CodeCR=IMAG0000233
                  End
                  Else //can't unset
                  Begin
                    FLogger.Error(#9 + 'Storage Copy' + #9 + 'Error occurred, clearing RAID pointer for image IEN ' + Inttostr(curIEN) + ' RAID pointer '+ MagPiece(cbxFromShareLoc.Text,' ',2)); //T21 - log 232(5446)
                    FErrorLogger.Error(#9 + 'Storage Copy' + #9 + 'Error occurred, clearing RAID pointer for image IEN ' + Inttostr(curIEN) + ' RAID pointer. ,' + MagPiece(cbxFromShareLoc.Text,' ',2)+' *RPC -'+MAGR[0]+' ('+UnitName+'.'+ProcName+'.5)');
                  End;
              End;
              FulFile := MagPiece(FulFile, '^', 2);
              FLogger.Info(#9 + 'Storage Copy' + #9 + 'Copied all files found for image IEN '+ Inttostr(curIEN) + ' ('+Copied+') ('+NofTot+')'+Set2005pt); //T20 CodeCR=IMAG00000233
              FErrorLogger.Info(#9 + 'Storage Copy' + #9 + 'Copied all files found for image IEN ' + Inttostr(curIEN) + ' ('+Copied+') ('+NofTot+')'+ Set2005pt +' ('+UnitName+'.'+ProcName+'.6)');
              OKCNT := OKCNT + 1;
              lbltotalcnt.caption := Inttostr(OKCNT);
            End //ToCopyFILE
            Else
            Begin //Copy failed
              If FUT49Running then
              Begin
                 errorcnt := errorcnt + 1;
                 UpdateLabel(lblErrorCnt,errorcnt);
                 If Copied<>'' then
                  Begin
                    FLogger.Info(#9 + 'Storage Copy' + #9 + 'Not all files copied for image IEN ' + Inttostr(curIEN) + ' ('+Copied+'*) stopped at ('+NofTot+')');
                    FErrorLogger.Info(#9 + 'Storage Copy' + #9 + 'Not all files copied for image IEN ' + Inttostr(curIEN) + ' ('+Copied+'*) stopped at ('+NofTot+')'+ ' ('+UnitName+'.'+ProcName+'.7)'); //T21
                  End;
              End;
            End;
          End; //curIEN >0
          If (curIEN = 0) And (MagPiece(MAGR[0], ',', 2) = 'Done') Then //completed last ien
          Begin
            SetCurIEN(curIEN,IEN2-1);//done & stop loop
            lbltotalcnt.caption := Inttostr(OKCNT);
            UpdateLabel(lblErrorCnt,errorcnt);
            ProgressUpdate(GaugeFUT49,IEN2,FUT49IEN1,IEN2); //T18 - complete 100%
            Application.Processmessages; //IR#T15-5
            If OKCNT > 0 Then
            Begin
              If doneMsg = '' Then
              Begin
                If errorcnt = 0 Then
                  EndProcessMsg(
                    'Storage Copy', //ProcessName: String;
                    True          , //EndedSuccessfully,
                    False         , //ViewLogs: Boolean;
                    OKCNT         , //OKCNT,
                    errorcnt      , //errorcnt: Integer);
                    lblTotalcnt   , //CopiedLabel: TLabel
                    lblErrorCnt   ) //NotCopiedLabel: TLabel
                Else
                  EndProcessMsg(
                    'Storage Copy', //ProcessName: String;
                    False         , //EndedSuccessfully,
                    True          , //ViewLogs: Boolean;
                    OKCNT         , //OKCNT,
                    errorcnt      , //errorcnt: Integer);
                    lblTotalcnt   , //CopiedLabel: TLabel
                    lblErrorCnt   );//NotCopiedLabel: TLabel
                doneMsg := '1';
                FUT49Running := False;
                FLogger.Info(#9 + 'Storage Copy' + #9 +#9 + 'Completed copying files for requested IEN or date range - ' + Inttostr(OKCNT) + ' IENs copied, ' + Inttostr(errorcnt) + ' IENs not copied.'); //t17 - IR T16-05 log summary
                FErrorLogger.Info(#9 + 'Storage Copy' + #9 +#9 + 'Completed copying files for requested IEN or date range - ' + Inttostr(OKCNT) + ' IENs copied, ' + Inttostr(errorcnt) + ' IENs not copied.'+ ' ('+UnitName+'.'+ProcName+'.8)'); //t18 - log summary
             End;
            End
            Else
              If doneMsg = '' Then
              Begin
                EndProcessMsg(
                  'Storage Copy', //ProcessName: String;
                  False        , //EndedSuccessfully,
                  True         , //ViewLogs: Boolean;
                  OKCNT        , //OKCNT,
                  errorcnt     , //errorcnt: Integer);
                  lblTotalcnt  , //CopiedLabel: TLabel
                  lblErrorCnt  ); //NotCopiedLabel: TLabel
                doneMsg := '1';
                FLogger.Info(#9 + 'Storage copy' + #9 + 'Completed copying files for requested IEN or date range - ' + Inttostr(OKCNT) + ' IENs copied, ' + Inttostr(errorcnt) + ' IENs not copied.');  //T18 - Walt replaced
                FErrorLogger.Info(#9 + 'Storage copy' + #9 + 'Completed copying files for requested IEN or date range - ' + Inttostr(OKCNT) + ' IENs copied, ' + Inttostr(errorcnt) + ' IENs not copied.'+ ' ('+UnitName+'.'+ProcName+'.9)');
                FUT49Running := False;
              End;
          End;
          If Res = '-2' Then    //T24 - broker timeout
          Begin
            FLogger.Error(#9 + 'Storage Copy' + #9 + 'Continued, RPC - ' + MagPiece(MAGR[0], ',', 2));
            FErrorLogger.Error(#9 + 'Storage Copy' + #9 + 'Continued, RPC - ' + MagPiece(MAGR[0], ',', 2) + ', ' + FUT49Mark+', '+Fname+' ('+UnitName+'.'+ProcName+'.15)');
            SetCurIEN(curIEN,Strtoint(MagPiece(MagPiece(MAGR[0],'(', 2),'~',1))); //broker timeout, continued
          End;
          If curIEN < 0 Then {Error occured, log and continue next ien}
          Begin
            If FUT49Running then errorcnt := errorcnt + 1;
            UpdateLabel(lblErrorCnt,errorcnt);
            Try
              SetCurIEN(curIEN,Strtoint(MagPiece(FUT49Mark, ':', 1)));
            Except
              SetCurIEN(curIEN,-1);
            End;
            FLogger.Error(#9 + 'Storage Copy' + #9 + 'Error occurred, RPC - ' + MagPiece(MAGR[0], ',', 2)+ ' (IEN ' +IEN0+')');
            FErrorLogger.Error(#9 + 'Storage Copy' + #9 + 'Error occurred, RPC - ' + MagPiece(MAGR[0], ',', 2) + ', stopped at ' + FUT49Mark+' ('+UnitName+'.'+ProcName+'.10)');
            If curIEN= -1 Then
            Begin
            If doneMsg = '' Then
              EndProcessMsg(
                'Storage Copy', //ProcessName: String;
                False         , //EndedSuccessfully,
                True          , //ViewLogs: Boolean;
                OKCNT         , //OKCNT,
                errorcnt      , //errorcnt: Integer);
                lblTotalcnt   , //CopiedLabel: TLabel
                lblErrorCnt   ); //NotCopiedLabel: TLabel
            doneMsg:='1';  //T18 - error to complete
            FLogger.Info(#9 + 'Storage copy' + #9 + 'Completed copying files for requested IEN or date range - ' + Inttostr(OKCNT) + ' IENs copied, ' + Inttostr(errorcnt) + ' IENs not copied.');  //T21 - CR 0304
            FErrorLogger.Info(#9 + 'Storage copy' + #9 + 'Completed copying files for requested IEN or date range - ' + Inttostr(OKCNT) + ' IENs copied, ' + Inttostr(errorcnt) + ' IENs not copied.'+' ('+UnitName+'.'+ProcName+'.11)');
            FUT49Running := False;
            Exit;
            End;
          End;
          SetCurIEN(curIEN,curIEN-1);//process next ien
          If curIEN < IEN2 Then //complete ien range
          Begin
            ProgressUpdate(GaugeFUT49,IEN2,FUT49IEN1,IEN2);  //T18 - complete 100%
            Application.Processmessages;
            If FUT49Running Then
            Begin
              If OKCNT > 0 Then
              Begin
                If doneMsg = '' Then
                Begin
                  If errorcnt = 0 Then
                    EndProcessMsg(
                      'Storage Copy', //ProcessName: String;
                      True          , //EndedSuccessfully,
                      False         , //ViewLogs: Boolean;
                      OKCNT         , //OKCNT,
                      errorcnt      , //errorcnt: Integer);
                      lblTotalcnt   , //CopiedLabel: TLabel
                      lblErrorCnt   ) //NotCopiedLabel: TLabel
                  Else
                    EndProcessMsg(
                      'Storage Copy', //ProcessName: String;
                      False         , //EndedSuccessfully,
                      True          , //ViewLogs: Boolean;
                      OKCNT         , //OKCNT,
                      errorcnt      , //errorcnt: Integer);
                      lblTotalcnt   , //CopiedLabel: TLabel
                      lblErrorCnt   );//NotCopiedLabel: TLabel
                  doneMsg := '1';
                  FUT49Running := False;
                  FLogger.Info(#9 + 'Storage Copy' + #9 +#9 + 'Completed copying files for requested IEN or date range - ' + Inttostr(OKCNT) + ' IENs copied, ' + Inttostr(errorcnt) + ' IENs not copied.'); //t17 - IR T16-05 log summary
                  FErrorLogger.Info(#9 + 'Storage Copy' + #9 +#9 + 'Completed copying files for requested IEN or date range - ' + Inttostr(OKCNT) + ' IENs copied, ' + Inttostr(errorcnt) + ' IENs not copied.'+' ('+UnitName+'.'+ProcName+'.12)'); //t18
                End;
              End
              Else  //OKCNT = 0
                Begin
                If doneMsg = '' Then
                  EndProcessMsg(
                    'Storage Copy', //ProcessName: String;
                    False         , //EndedSuccessfully,
                    True          , //ViewLogs: Boolean;
                    OKCNT         , //OKCNT,
                    errorcnt      , //errorcnt: Integer);
                    lblTotalcnt   , //CopiedLabel: TLabel
                    lblErrorCnt   );//NotCopiedLabel: TLabel
                doneMsg := '1';
                FUT49Running := False;
                FLogger.Info(#9 + 'Storage copy' + #9 + 'Task is complete. No files were copied.'+#9+ 'No files exist at source location');  //T21 -  CR 0304
                FErrorLogger.Info(#9 + 'Storage copy' + #9 + 'Task is complete. No files were copied.'+#9+ 'No files exist at source location'+' ('+UnitName+'.'+ProcName+'.13)');
                End;
            End;
            cbxFromShareLoc.Items.Clear;
            cbxToShareLoc.Items.Clear;
            btnProcessFUT49.Enabled := True;
            FUT49Running := False;
          End;
          IEN0 := Inttostr(curIEN) + ':' + Inttostr(IEN2);
          FUT49Mark := IEN0 + '^' + NET0; //for resume/restart point
        End //RPC - MAG FILE FETCH , success
        Else
        Begin //RPC failed
          lbltotalcnt.caption := Inttostr(OKCNT);
          If FUT49Running then errorcnt := errorcnt + 1;
          UpdateLabel(lblErrorCnt,errorcnt);
          errormsg := '1'; //already displayed
          FLogger.Error(#9 + 'Storage Copy' + #9 + 'Error occurred, RPC error - ' + MagPiece(MAGR[0], ',', 2) + ', (IEN ' + IEN0 +')');
          FErrorLogger.Error(#9 + 'Storage Copy' + #9 + 'Error occurred, RPC error - ' + MAGR[0] + ', stopped at ' + FUT49Mark+' ('+UnitName+'.'+ProcName+'.14)');
        End;
       Finally   
           FreeAndNil(MAGR);
       end; //try - T24 memory release
        Application.Processmessages;
      End; //curIEN>0  before RPC
    End; //while IEN2>curIEN   (in IEN1:IEN2 range)

    UpdateLabel(lblErrorCnt,errorcnt);
    lbltotalcnt.caption := Inttostr(OKCNT);
    //reset all
    FUT49Running := False;
    SetCaptionFut49byDt4ien('');
    SetCaptionFUT49byDt2ien('');
    SetCaptionFut49byIEN4DT('');
    SetCaptionFut49byIEN2DT('');
    lblcurrentfile.caption := '';
    lblIEN.caption := '';
    If FUT49Mark = '' Then //FUT49 task been canceled
    Begin
      lblCurrentFile.caption := '';
      lblIEN.caption := '';
      lblTotalcnt.caption := '';
      UpdateLabel(lblErrorCnt,'');
      lblFileEmptyCnt.caption := '';
    End;
    FUT49RemoveOld.Enabled := True;
    cbxFromShareLoc.ClearSelection;
    cbxFromShareLoc.Items.CleanupInstance;
    cbxToShareLoc.ClearSelection;
    cbxToShareLoc.Items.CleanupInstance;
    btnProcessFUT49.Enabled := False;
    BtnPauseFUT49.Enabled := False;
    AdjustEnabled(gbcbxShareLoc, False);
    //T20 - Application.Processmessages;

  Finally
    pnlVars.Caption       := '';
    FUT49AftHour.Enabled  := boFUT49AftHourEnabled;
    FUT49RemoveOld.Enabled:= boFUT49RemoveOldEnabled;
    FUT49SetNewLoc.Enabled:= boFUT49SetNewLocEnabled;
    rgFut49ChkType.Enabled:= borgFut49ChkTypeEnabled;
    If FUT49Running = False then FUT49RemoveOld.Enabled:= True; //T21 CR239
    If NOT FUT49RemoveOld.Checked then FUT49SetNewLoc.Enabled := True; //T20 - CodeCR239
  End;
  GaugeFUT49.Visible:=False;
End;

procedure TfMagUtility.FUT49RunInit(Sender: Tobject);
Var
  //chkvar                 : String;
  curIEN, IEN1, IEN2     : Integer;
  IEN0                   : String;
  //maxien                 : String;
  NET0                   : String;
  ProcName               : String;
  Selection              : String;
  Today                  : String;
Begin
  ProcName               :='FUT49RunInit';
  If bmpGo  .Width>3 Then btnProcessFUT49.Glyph.Assign(bmpGo  );
  If bmpStop.Width>3 Then btnPauseFUT49  .Glyph.Assign(bmpStop);
  StorageCopyWarning(Sender);
  FProcessingPaused      :=False;
  IEN0                   := '';
  LastErrorCode          := 0; //T17 record error #
  Selection              := '';
  lblCurrentFile.caption := '';
  lblIEN.caption         := '';
  lblTotalcnt.caption    := '';
  UpdateLabel(lblErrorCnt,'');
  Today := DateToStr(Date);
  If btnProcessFUT49.caption = '&Cancel' Then //paused and now Cancel/Stop task
  Begin //reset value
    FUT49Running := False;
    SetPnlRetryVis(False);
    boInRetryLoop:= False;
    InitStorageCopy(Sender);
    MakeFocused(rgFut49ChkType);
    FLogger.Info(#9 + 'Storage Copy' + #9 + 'The task canceled at ' + TimeToStr(Time) + ' by user:'+ MagUtilityBroker.UserFullName);  //T17
    FErrorLogger.Info(#9 + 'Storage Copy' + #9 + 'The task canceled at ' + TimeToStr(Time) + ' by user:'+ MagUtilityBroker.UserFullName);  //T17
    FUT49Mark := '';
    cbxFromShareLoc.ClearSelection;
    cbxFromShareLoc.Items.CleanupInstance;
    cbxToShareLoc.ClearSelection;
    cbxToShareLoc.Items.CleanupInstance;
    Application.Processmessages;
    Showmessage('Storage Copy was canceled');
    GaugeFUT49.Visible:=False;
    EXIT;
  End;
  If OrphanFilesRunning Then
  Begin
    Showmessage(
      'Warning!  A Orphan File Cleanup process is running.' + #13 + #10+
      'Only one process can run at a time.  Try again when ' + #13 + #10+
      'the Orphan File Cleanup process has completed.');
    Exit;
  End;
  If rgFut49ChkType.ItemIndex=0 Then ///date range
    If (EdtStartDT.Text = '') Or (EdtEndDT.Text = '') Then
    Begin
      Showmessage('Specify the date range before running this process.');
      Exit;
    End
    Else
    Begin
      IEN0 := MagPiece(edtStartDT.EditLabel.Caption, '=', 2);
      IEN0 := MagPiece(IEN0, ' ', 1);
      IEN0 := IEN0 + ':' + MagPiece(EdtEndDT.EditLabel.Caption, '=', 2);
      IEN0 := MagPiece(IEN0, ' ', 1);
      Selection := ' By date - ' + edtStartDT.Text + ':' + edtEndDT.Text + ' ';
    End;
  If rgFut49ChkType.ItemIndex=1 Then //IEN range
    If (EdtStartIEN.Text = '') Or (EdtEndIEN.Text = '') Then
    Begin
      Showmessage('Specify an IEN range before running this process.');
      Exit;
    End
    Else
    Begin
      IEN0 := edtStartIEN.Text + ':' + edtEndIEN.Text;
      Selection := ' By IEN - ' + IEN0 + ' ';
    End;
  If IEN0 = '' Then
  Begin
    Showmessage('You must select `By IEN` or `By date` range first, then From/To location.');
    Exit;
  End;
  IEN1 := Strtoint(MagPiece(IEN0, ':', 1));
  IEN2 := Strtoint(MagPiece(IEN0, ':', 2));
  //If IEN1 = IEN2 Then IEN2 := IEN2 - 1; //signal ien copy
  If IEN1 < IEN2 Then //check correct order  e.g.: 9999:1111   LastInFirstOut
  Begin
    IEN0 := MagPiece(IEN0, ':', 2) + ':' + MagPiece(IEN0, ':', 1);
    curIEN := IEN1;
    IEN1 := IEN2;
    IEN2 := curIEN;
  End;
  //chkvar := '$O(^MAG(2005,"AD",$O(^MAG(2005,"AD"," "),-1)," "),-1)';
  //maxien := MagUtilityBroker.FnmGetNode(chkvar); //last (day) max IEN
  //If (IEN1 > Strtoint(maxien)) Then IEN1 := Strtoint(maxien); //adjust IEN1 to last IEN
  IEN0 := Inttostr(IEN1) + ':' + Inttostr(IEN2);

  If (cbxFromShareLoc.Text = '') Or (FUT49FromLoc = '') Then //Check From share loction
  Begin
    Showmessage('Storage Copy - cannot be run until a valid `From share location` is selected.');
    Exit;
  End;
  If (cbxToShareLoc.Text = '') Or (FUT49ToLoc = '') Then //Check To share location
  Begin
    Showmessage('Storage Copy - cannot be run until a valid `To share location` is selected.');
    Exit;
  End;
  NET0 := FUT49FromLoc + '|' + FUT49ToLoc;
  If FUT49RemoveOld.Checked Then
    NET0 := NET0 + '|1' //1 - Remove source file option, from source storage
  Else
    NET0 := NET0 + '|0';
  NET0 := NET0 + '|0';
  FUT49Mark := IEN0 + '^' + NET0;
  Selection := Selection + ' Copy from ' + cbxFromShareLoc.Text + ' to ' + cbxToShareLoc.Text; //user selection
  If FUT49RemoveOld.Checked or FUT49AftHour.Checked or FUT49SetNewLoc.Checked Then Selection := Selection + ' checked option(s): '
  Else Selection := Selection + ' with no option checked'; //t17
  If FUT49RemoveOld.Checked Then Selection := Selection + ' x "Remove source files"';
  If FUT49AftHour.Checked Then Selection := Selection + ' x "Run after-hours"';
  If FUT49SetNewLoc.Checked Then Selection := Selection + ' x "Update #2005 reference"';
  errorcnt    := 0;
  totalcnt    := 0;
  OKCNT       := 0;
  ptrEmptyCnt := 0;
  btnprocessFUT49.Enabled := False;
  btnPauseFUT49.Enabled := True;
  FLogger.Info(#9 + 'Storage copy' + #9 + 'Run' + #9 + Selection + ' by user:' + MagUtilityBroker.UserFullName);
  FErrorLogger.Info(#9 + 'Storage copy' + #9 + 'Run' + #9 + Selection + ' by user:' + MagUtilityBroker.UserFullName);
  FUT49IEN1:= IEN1;
  curIEN := IEN1;
  if (FUT49Mark <> '') then FUT49Running := True;
  FUT49Resume(curIEN,IEN2,NET0);
end;

procedure TfMagUtility.pnlVarsClick(Sender: TObject);
Var
  ProcName: String;
  sgMsg   : String;
  sgVal   : String;
begin
  ProcName:='TfMagUtility.pnlVarsClick';
  sgMsg:='';
  If boInRetryLoop               Then sgVal:='boInRetryLoop              '+#9#9+'=True' Else sgVal:='boInRetryLoop              '+#9#9+'=False';
  sgMsg:=sgMsg+sgVal+#13+#10;
  If btnRunProcess.Enabled       Then sgVal:='btnRunProcess.Enabled      '+#9  +'=True' Else sgVal:='btnRunProcess.Enabled      '+#9  +'=False';
  sgMsg:=sgMsg+sgVal+#13+#10;
  If btnStopProcessing.Enabled   Then sgVal:='btnStopProcessing.Enabled  '+#9  +'=True' Else sgVal:='btnStopProcessing.Enabled  '+#9  +'=False';
  sgMsg:=sgMsg+sgVal+#13+#10;
  If FProcessingPaused           Then sgVal:='FProcessingPaused          '+#9  +'=True' Else sgVal:='FProcessingPaused          '+#9  +'=False';
  sgMsg:=sgMsg+sgVal+#13+#10;
  If FUT49Running                Then sgVal:='FUT49Running               '+#9#9+'=True' Else sgVal:='FUT49Running               '+#9#9+'=False';
  sgMsg:=sgMsg+sgVal+#13+#10;
  If OrphanFilesRunning          Then sgVal:='OrphanFilesRunning         '+#9  +'=True' Else sgVal:='OrphanFilesRunning         '+#9  +'=False';
  sgMsg:=sgMsg+sgVal+#13+#10;
  If TimerOrphanEnabling.Enabled Then sgVal:='TimerOrphanEnabling.Enabled'+#9  +'=True' Else sgVal:='TimerOrphanEnabling.Enabled'+#9  +'=False';
  sgMsg:=sgMsg+sgVal+#13+#10;
  ShowMessage(sgMsg);
end;

procedure TfMagUtility.SetCurIEN(var curIEN: Integer; newValue: Integer);
begin
  curIEN:=newValue;
  FUT49curIEN:=newValue;
  //pnlVars.Caption:=IntToStr(curIEN);
end;

procedure TfMagUtility.NetClnChkListViewSelectItem(Sender: TObject;
  Item: TListItem; Selected: Boolean);
begin
  EnableRunButtonNetworkCleanup;
end;

procedure TfMagUtility.UpdateLabel(l: TLabel; m: String);
begin
  If l = nil Then Exit;
  l.Caption:=m;
  Application.ProcessMessages();
end;

procedure TfMagUtility.UpdateLabel(l: TLabel; i: Integer);
begin
  If l = nil Then Exit;
  UpdateLabel(l,IntToStr(i));
end;

procedure TfMagUtility.FormDestroy(Sender: TObject);
Var
  NameBase: String;
begin
  If boSaveKeystrokes Then
  Begin
    NameBase:= ExtractFileName(ParamStr(0));
    NameBase:=Copy(NameBase,1,Length(NameBase)-4);
    lstKeyStrokes.SaveToFile(WindowsTempDirectory+NameBase+'_KeyStrokes_'+FormatDateTime('YYYYMMDDHHNNSS',now())+'.txt');
  End;
  FreeAndNil(bmpGo);
  FreeAndNil(bmpStop);
  FreeAndNil(lstKeyStrokes);
end;

procedure TfMagUtility.FormActivate(Sender: TObject);
begin
  bmpGo.Assign(btnProcessFUT49.Glyph);
  bmpStop.Assign(btnPauseFUT49.Glyph);
end;

Procedure TfMagUtility.ComputeColumnWidths(LV: TListView;var FColZeroWidth: Integer;var FColWidth: Array Of Integer);
Var
  i, cc: Integer;
  Li: TListItem;
Begin
  For cc := 0 To LV.Columns.Count - 1 Do
  Begin
    If (cc = 0) And (FColZeroWidth > -1) Then
    Begin
      FColWidth[cc] := FColZeroWidth;
      Continue;
    End;
    FColWidth[cc] := LV.Canvas.Textwidth(LV.Columns[cc].caption);
  End;
  Li := Nil;
  Li := LV.Getnextitem(Li, Sdall, [IsNone]);
  While Li <> Nil Do
  Begin
    If (FColZeroWidth = -1) Or (FColZeroWidth = -2) Then
      If (FColWidth[0] < LV.Canvas.Textwidth(Li.caption)) Then FColWidth[0] := LV.Canvas.Textwidth(Li.caption);
    For i := 1 To LV.Columns.Count - 1 Do
      If (FColWidth[i] < LV.Canvas.Textwidth(Li.SubItems[i - 1])) Then FColWidth[i] := LV.Canvas.Textwidth(Li.SubItems[i - 1]);
    Li := LV.Getnextitem(Li, Sdall, [IsNone]);
  End;
End;

Procedure TfMagUtility.FitColumnsToText(LV: TListView);
Var
  FColWidth    : Array Of Integer;
  FColZeroWidth: Integer;
  i            : Integer;
  inWidth      : Integer;
  Xtra         : Integer;
Begin
  Try
    LV.Items.BeginUpdate;
    FColZeroWidth:= -1;
    SetLength(FColWidth,LV.Columns.Count);
    ComputeColumnWidths(LV,FColZeroWidth,FColWidth);
    For i := 0 To LV.Columns.Count - 1 Do
    Begin
      inWidth:= FColWidth[i];
      Xtra:=(inWidth div 10)+19;
      LV.Columns[i].Width := inWidth + Xtra;
    End;
  Finally
    LV.Items.EndUpdate;
  End;
End;

Procedure TfMagUtility.SetListViewWidth(LV: TListView);
Var
  i   : Integer;
  w   : Integer;
  Xtra: Integer;
Begin
  Xtra := 1;
  w:=1;
  FitColumnsToText(LV);
  For i := 0 To LV.Columns.Count - 1 Do
  Begin
    w:=w+LV.Columns[i].Width+ Xtra;
  End;
  LV.Width:=w;
End;

Procedure TfMagUtility.EndProcessMsg(ProcessName: String;EndedSuccessfully,ViewLogs: Boolean;OKCNT,errorcnt: Integer;CopiedLabel,NotCopiedLabel: TLabel);
Var
  sgCopied    : String;
  sgMsg       : String;
  sgNotCopied : String;
  sgSeeLogs   : String;
  sgSuccessMsg: String;
Begin
  ProcessName:=Trim(ProcessName);
  If ProcessName='' Then Exit;
  sgSuccessMsg:=ProcessName;
  If EndedSuccessfully Then
  Begin
    sgSuccessMsg:=sgSuccessMsg+' completed successfully.'+#13+#10;
  End
  Else
  Begin
    //t17 - err code if any
    If LastErrorCode = 0 then sgSuccessMsg:=sgSuccessMsg+' did not complete successfully. '+#13+#10
    Else
    Begin
       sgSuccessMsg:=sgSuccessMsg+' did not complete successfully.'+#13+#10;  //T20  +'Last error#'+IntToStr(LastErrorCode)+ ': '+ SysErrorMessage(LastErrorCode)+#13+#10; //t17
       FErrorLogger.Error(#9+#9+#9+'Last error code #'+ IntToStr(LastErrorCode)+ ': '+ SysErrorMessage(LastErrorCode));   //T20 - added 'last' to error#
    End;
  End;
  sgCopied    := 'Images copied='+Inttostr(OKCNT)+#13+#10; //T20 - CodeCR 228 wording (IEN, image, file)
  sgNotCopied := 'Not copied=' + Inttostr(errorcnt) +#13+#10; //T20 - remove it
  sgSeeLogs:='';
  If ViewLogs Then sgSeeLogs:=#13+#10+#13+#10+'See log for individual file status and check ' +
                       'patch documentation for troubleshooting suggestions.'; //t17 - Dec 07 meeting IRt15-21 & T15-22
  sgMsg       := sgSuccessMsg+ sgSeeLogs; //T20 CodeCR 228 - remove it    := sgSuccessMsg+ sgNotCopied+ sgCopied+ sgSeeLogs;
  If CopiedLabel<>nil Then
  Begin
    CopiedLabel.Caption:=IntToStr(OKCNT);
  End;
  If NotCopiedLabel<>nil Then
  Begin
    NotCopiedLabel.Caption:=IntToStr(errorcnt);
  End;
  Application.ProcessMessages();
  Showmessage(sgMsg);
End;

procedure TfMagUtility.StorageCopyCancel(LogMsg, ShowMsg: String;errorcount: Integer);
Var
  ProcName: String;
begin
  ProcName:='TfMagUtility.StorageCopyCancel';
  SetPnlRetryVis(False);
  If Trim(ShowMsg)<>'' Then //Showmessage(ShowMsg);
  Begin
    EndProcessMsg(
      ShowMsg       , //ProcessName: String;
      False         , //EndedSuccessfully,
      True          , //ViewLogs: Boolean;
      OKCNT         , //OKCNT,
      errorcount    , //errorcnt: Integer);
      lblTotalcnt   , //CopiedLabel: TLabel
      lblErrorCnt   );//NotCopiedLabel: TLabel
  End;
  If LastErrorCode=0 then
  Begin
      FLogger.Info(LogMsg);
      FErrorLogger.Info(LogMsg+' ('+UnitName+'.'+ProcName+'.20)');
  End
  Else
     Begin
        FLogger.Error(LogMsg);  //t17 - IR T16-10
        FErrorLogger.Error(LogMsg+' ('+UnitName+'.'+ProcName+'.20)');
     End;
  cbxFromShareLoc.Text := '';
  cbxToShareLoc.Text   := '';
  FUT49Mark            := '';
  FUT49RUNNING         := False;
  boInRetryLoop        := False;
  FProcessingPaused    := True;
  Application.ProcessMessages();
  InitStorageCopy(nil);
  StorageCopyEnabling(nil);
end;

procedure TfMagUtility.StorageCopyCancel(LogMsg,ShowMsg: String);
Var
  ProcName: String;
begin
  ProcName:='TfMagUtility.StorageCopyCancel1';
  StorageCopyCancel(LogMsg, ShowMsg, errorcnt);
end;


procedure TfMagUtility.TimerCleanupTimer(Sender: TObject);
begin
  TimerCleanup.Enabled:=False;
  ActiveControl:=NetClnChkListView;
  Application.ProcessMessages();
  NetClnChkListView.ItemIndex:= 0;
  NetClnChkListView.Items[0].Selected:=True;
  EnableRunButtonNetworkCleanup();
end;

procedure TfMagUtility.SetCaptionFut49byIEN4DT(s: String);
begin
  edtStartIEN.EditLabel.Caption:=s;
end;

procedure TfMagUtility.SetCaptionFut49byIEN2DT(s: String);
begin
  edtEndIEN.EditLabel.Caption:=s;
end;

procedure TfMagUtility.SetCaptionFUT49byDt2ien(s: String);
begin
  EdtEndDT.EditLabel.Caption:=s;
end;

procedure TfMagUtility.SetCaptionFut49byDt4ien(s: String);
begin
  edtStartDT.EditLabel.Caption:=s;
end;

procedure TfMagUtility.ProgressUpdate(Prog: TGauge;Cur,Max,Min:Integer);
Var
  inDelta: Integer;
  inProg : Integer;
  dblMult: Double;
begin
  inDelta:= Max-Min;
  inProg := Max-Cur;
  If inDelta<100 Then
  Begin
    Prog.MinValue :=0;
    Prog.MaxValue :=inDelta;
    Prog.Progress:=inProg;
    Exit;
  End;
  dblMult:= 100/inDelta;
  Prog.MinValue :=0;
  Prog.MaxValue :=Floor(inDelta*dblMult);
  Prog.Progress:=Floor(inProg*dblMult);
end;

function TfMagUtility.WindowsTempDirectory: String;
var
  tempFolder: array[0..MAX_PATH] of Char;
begin
  GetTempPath(MAX_PATH, @tempFolder);
  result := StrPas(tempFolder);
end;

procedure TfMagUtility.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
Var
  sgShift: String;
  sgKey  : String;
  inKey  : Integer;
begin
  If Not boRecordKeystrokes Then Exit;
  sgKey  := '';
  inKey  := Key;
  Case inKey of
  0..8:sgKey:='#'+IntToStr(inKey);
  9:sgKey:='TAB';
  10:sgKey:='Line Feed';
  11..12:sgKey:='#'+IntToStr(inKey);
  13:sgKey:='ENTER';
  14..31:sgKey:='#'+IntToStr(inKey);
  32:sgKey:='SPACE';
  33..38:sgKey:=chr(inKey);
  39:sgKey:='RIGHTARROW';
  40..126:sgKey:=chr(inKey);
  127:sgKey:='BACKSPACE';
  128..255:sgKey:=chr(inKey);
  End;
  sgShift:='[';
  If ssShift     in Shift Then sgShift:=sgShift+'Shift'    +',';
  If ssAlt       in Shift Then sgShift:=sgShift+'Alt'      +',';
  If ssCtrl      in Shift Then sgShift:=sgShift+'Ctrl'     +',';
  If ssLeft      in Shift Then sgShift:=sgShift+'Left'     +',';
  If ssRight     in Shift Then sgShift:=sgShift+'Right'    +',';
  If ssMiddle    in Shift Then sgShift:=sgShift+'Middle'   +',';
  If ssDouble    in Shift Then sgShift:=sgShift+'Double'   +',';
  If sgShift<>'[' Then sgShift:=Copy(sgShift,1,Length(sgShift)-1);
  If sgShift='[' Then
    sgShift:=''
    Else
    sgShift:=sgShift+']';
  If sgShift='' Then
  Begin
    sgShift:=sgKey;
  End
  Else
  Begin
    sgShift:=sgShift+'+'+sgKey;
  End;
  lstKeyStrokes.Append('Press '+sgShift);
End;

procedure TfMagUtility.ShowMessage(const Msg: string);
Var
  Cap : PAnsichar;
  Text: PAnsichar;
begin
  Text:= PAnsiChar(Msg);
  Cap:= PAnsiChar(Application.Title);
  Windows.MessageBox(
      Application.Handle,
      Text,
      Cap,
      MB_OK
  );
end;


procedure TfMagUtility.MagUtilityHelp1Click(Sender: TObject);
Var
  sgHelpFile: String;
begin
  sgHelpFile:=ExtractFilePath(ParamStr(0))+'MagUtilityHelp.pdf';
  If Not FileExists(sgHelpFile) Then Exit;
  ExecuteKnownFileType(Handle,sgHelpFile);
end;

Function TfMagUtility.ExecuteKnownFileType(
  Handle   : THandle;
  FileName : String): Boolean;
Var
  PFileName : array[0..128] of Char;
  PFilePath : array[0..128] of Char;
  FilePath  : String;
Begin
  Result:=False;
  Try
    If Not FileExists(FileName) Then Exit;
    FilePath := ExtractFilePath(FileName);
    StrPCopy(PFileName,FileName);
    StrPCopy(PFilePath,FilePath);
    ShellExecute(
      Handle,
      nil,
      PFileName,
      nil,
      PFilePath,
      SW_SHOWNORMAL);
    Result := True;
  Except
  End;
End;

Function TfMagUtility.ExecuteNewProcess(
  FileName   : String;
  Visibility : integer):integer;
var
  zAppName:array[0..512] of char;
  zCurDir:array[0..255] of char;
  WorkDir:String;
  StartupInfo:TStartupInfo;
  ProcessInfo:TProcessInformation;
begin
  StrPCopy(zAppName,FileName);
  GetDir(0,WorkDir);
  StrPCopy(zCurDir,WorkDir);
  FillChar(StartupInfo,Sizeof(StartupInfo),#0);
  StartupInfo.cb          := Sizeof(StartupInfo);
  StartupInfo.dwFlags     := STARTF_USESHOWWINDOW;
  StartupInfo.wShowWindow := Visibility;
  If Not CreateProcess(nil,
    zAppName,                      { pointer to command line string }
    nil,                           { pointer to process security attributes}
    nil,                           { pointer to thread security attributes }
    false,                         { handle inheritance flag }
    CREATE_NEW_PROCESS_GROUP,
    nil,                           { pointer to new environment block }
    nil,                           { pointer to current directory name }
    StartupInfo,                   { pointer to STARTUPINFO }
    ProcessInfo)
  Then
  Begin
    Result := -1 { pointer to PROCESS_INF }
  End
  Else
  Begin
    WaitforSingleObject(ProcessInfo.hProcess,300);
    GetExitCodeProcess(ProcessInfo.hProcess,Cardinal(Result));
  End;
End;


End. //uMagUtility.pas
